# Sid Meier's Civilization 4
# Copyright Firaxis Games 2006
#
# CvEventManager
# This class is passed an argsList from CvAppInterface.onEvent
# The argsList can contain anything from mouse location to key info
# The EVENTLIST that are being notified can be found

from CvPythonExtensions import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import sys
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser
import pickle

import CvIntroMovieScreen
import CustomFunctions
import ScenarioFunctions

#FfH: Card Game: begin
import CvSomniumInterface
import CvCorporationScreen
#FfH: Card Game: end

#FfH: Added by Kael 10/15/2008 for OOS Logging
import OOSLogger
#FfH: End Add

import Blizzards		#Added in Frozen: Blizzards: TC01

## *******************
## Modular Python: ANW 16-feb-2010
##                     29-may-2010
## ArcticNightWolf on CivFanatics
## ArcticNightWolf@gmail.com

# For dynamic plugin loading
import imp    # dynamic importing of libraries
import glob   # Unix style pathname pattern expansion
import CvPath # path to current assets
import inspect

# Maps modules to the function name
# Syntax: {'functionName': [module1, module2]}
command = {}

# functionList is a list of strings of functions we will directly map
functionList = ['onMouseEvent', 'onKbdEvent', 'onModNetMessage', 'onInit', 'onUpdate', 'onUnInit', 'onSaveGame', 'onPreSave', 'onLoadGame',
 'onEndGameTurn', 'onBeginPlayerTurn', 'onEndPlayerTurn', 'onEndTurnReady', 'onCombatResult', 'onCombatLogCalc', 'onCombatLogHit',
 'onImprovementBuilt', 'onImprovementDestroyed', 'onRouteBuilt', 'onFirstContact', 'onCityBuilt', 'onCityRazed', 'onCityAcquired',
 'onCityAcquiredAndKept', 'onCityLost', 'onCultureExpansion', 'onCityGrowth', 'onCityDoTurn', 'onCityBuildingUnit', 'onCityBuildingBuilding',
 'onCityRename', 'onCityHurry', 'onSelectionGroupPushMission', 'onUnitMove', 'onUnitSetXY', 'onUnitCreated', 'onUnitBuilt', 'onUnitKilled',
 'onUnitLost', 'onUnitPromoted', 'onUnitSelected', 'onUnitRename', 'onUnitPillage', 'onUnitSpreadReligionAttempt', 'onUnitGifted',
 'onUnitBuildImprovement', 'onGoodyReceived', 'onGreatPersonBorn', 'onBuildingBuilt', 'onProjectBuilt', 'onTechAcquired', 'onTechSelected',
 'onReligionFounded', 'onReligionSpread', 'onReligionRemove', 'onCorporationFounded', 'onCorporationSpread', 'onCorporationRemove', 'onGoldenAge',
 'onEndGoldenAge', 'onChat', 'onVictory', 'onVassalState', 'onChangeWar', 'onSetPlayerAlive', 'onPlayerChangeStateReligion', 'onPlayerGoldTrade',
 'onWindowActivation', 'onGameUpdate']

## Modular Python End
## *******************

# globals
cf = CustomFunctions.CustomFunctions()
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
sf = ScenarioFunctions.ScenarioFunctions()

#FfH: Card Game: begin
cs = CvCorporationScreen.cs
#FfH: Card Game: end

Blizzards = Blizzards.Blizzards()		#Added in Frozen: Blizzards: TC01

#import GreyFoxCustom
import FoxDebug
import FoxTools
import time
from BasicFunctions import *

FoxGlobals = {
	"USE_DEBUG_WINDOW" 		: False,
	"USE_AIAUTOPLAY_SOUND" 	: True,
}
SoundSettings = {
	"SOUND_MASTER_VOLUME" 	: 0,
	"SOUND_SPEECH_VOLUME" 	: 0,
	"SOUND_MASTER_NO_SOUND" : False,
}

# globals
###################################################
class CvEventManager:
	def __init__(self):
		#################### ON EVENT MAP ######################
		#print "EVENTMANAGER INIT"
		self.bCtrl = False
		self.bShift = False
		self.bAlt = False
		self.bAllowCheats = False

		# Dictionaries
		self.Defines			= {}
		self.Eras				= {}
		self.Techs				= {}
		self.Victories			= {}
		self.GameSpeeds			= {}
		self.GameOptions		= {}
		self.EventTriggers		= {}
		
		# Civs, etc
		self.Civilizations		= {}
		self.Leaders 			= {}
		self.LeaderStatus		= {}
		self.Traits 			= {}
		self.Civics 			= {}
		self.Religions			= {}
		self.Corporations		= {}
		self.Alignments			= {}
		
		# Buildings, etc
		self.Projects 			= {}
		self.Buildings 			= {}
		self.Specialists		= {}
		self.BuildingClasses	= {}
		self.Processes			= {}
		
		# Improvements, etc
		self.Builds				= {}
		self.Lairs 				= {}
		self.ManaNodes			= {}
		self.Improvements 		= {}
		self.CivImprovements	= {}
		self.UniqueImprovements	= {}
		
		# Terrain, etc
		self.Mana	 			= {}
		self.Terrain 			= {}
		self.Resources 			= {}
		self.WorldSizes			= {}
		self.Goodies			= {}
		
		# Units, etc
		self.Units 				= {}
		self.Heroes				= {}
		self.UnitAI				= {}
		self.Promotions 		= {}
		self.UnitClasses		= {}
		self.UnitCombats 		= {}
		self.GreatPeople 		= {}
		self.DamageTypes		= {}
		
		self.cf					= None
		self.Tools 				= None
		self.DbgWnd				= None
		self.LoadedData			= False
		
		# OnEvent Enums
		self.EventLButtonDown=1
		self.EventLcButtonDblClick=2
		self.EventRButtonDown=3
		self.EventBack=4
		self.EventForward=5
		self.EventKeyDown=6
		self.EventKeyUp=7

		self.__LOG_MOVEMENT = 0
		self.__LOG_BUILDING = 0
		self.__LOG_COMBAT = 0
		self.__LOG_CONTACT = 0
		self.__LOG_IMPROVEMENT =0
		self.__LOG_CITYLOST = 0
		self.__LOG_CITYBUILDING = 0
		self.__LOG_TECH = 0
		self.__LOG_UNITBUILD = 0
		self.__LOG_UNITKILLED = 1
		self.__LOG_UNITLOST = 0
		self.__LOG_UNITPROMOTED = 0
		self.__LOG_UNITSELECTED = 0
		self.__LOG_UNITPILLAGE = 0
		self.__LOG_GOODYRECEIVED = 0
		self.__LOG_GREATPERSON = 0
		self.__LOG_RELIGION = 0
		self.__LOG_RELIGIONSPREAD = 0
		self.__LOG_GOLDENAGE = 0
		self.__LOG_ENDGOLDENAGE = 0
		self.__LOG_WARPEACE = 0
		self.__LOG_PUSH_MISSION = 0

		## EVENTLIST
		self.EventHandlerMap = {
			'mouseEvent'					: self.onMouseEvent,
			'kbdEvent' 						: self.onKbdEvent,
			'ModNetMessage'					: self.onModNetMessage,
			'Init'							: self.onInit,
			'Update'						: self.onUpdate,
			'UnInit'						: self.onUnInit,
			'OnSave'						: self.onSaveGame,
			'OnPreSave'						: self.onPreSave,
			'OnLoad'						: self.onLoadGame,
			'GameStart'						: self.onGameStart,
			'GameEnd'						: self.onGameEnd,
			'plotRevealed' 					: self.onPlotRevealed,
			'plotFeatureRemoved' 			: self.onPlotFeatureRemoved,
			'plotPicked'					: self.onPlotPicked,
			'nukeExplosion'					: self.onNukeExplosion,
			'gotoPlotSet'					: self.onGotoPlotSet,
			'BeginGameTurn'					: self.onBeginGameTurn,
			'EndGameTurn'					: self.onEndGameTurn,
			'BeginPlayerTurn'				: self.onBeginPlayerTurn,
			'EndPlayerTurn'					: self.onEndPlayerTurn,
			'endTurnReady'					: self.onEndTurnReady,
			'combatResult' 					: self.onCombatResult,
			'combatLogCalc'	 				: self.onCombatLogCalc,
			'combatLogHit'					: self.onCombatLogHit,
			'improvementBuilt' 				: self.onImprovementBuilt,
			'improvementDestroyed' 			: self.onImprovementDestroyed,
			'routeBuilt' 					: self.onRouteBuilt,
			'firstContact' 					: self.onFirstContact,
			'cityBuilt' 					: self.onCityBuilt,
			'cityRazed'						: self.onCityRazed,
			'cityAcquired' 					: self.onCityAcquired,
			'cityAcquiredAndKept' 			: self.onCityAcquiredAndKept,
			'cityLost'						: self.onCityLost,
			'cultureExpansion' 				: self.onCultureExpansion,
			'cityGrowth' 					: self.onCityGrowth,
			'cityDoTurn' 					: self.onCityDoTurn,
			'cityBuildingUnit'				: self.onCityBuildingUnit,
			'cityBuildingBuilding'			: self.onCityBuildingBuilding,
			'cityRename'					: self.onCityRename,
			'cityHurry'						: self.onCityHurry,
			'selectionGroupPushMission'		: self.onSelectionGroupPushMission,
			'unitMove' 						: self.onUnitMove,
			'unitSetXY' 					: self.onUnitSetXY,
			'unitCreated' 					: self.onUnitCreated,
			'unitBuilt' 					: self.onUnitBuilt,
			'unitKilled'					: self.onUnitKilled,
			'unitLost'						: self.onUnitLost,
			'unitPromoted'					: self.onUnitPromoted,
			'unitSelected'					: self.onUnitSelected,
			'UnitRename'					: self.onUnitRename,
			'unitPillage'					: self.onUnitPillage,
			'unitSpreadReligionAttempt'		: self.onUnitSpreadReligionAttempt,
			'unitGifted'					: self.onUnitGifted,
			'unitBuildImprovement'			: self.onUnitBuildImprovement,
			'goodyReceived'					: self.onGoodyReceived,
			'greatPersonBorn'	  			: self.onGreatPersonBorn,
			'buildingBuilt' 				: self.onBuildingBuilt,
			'projectBuilt' 					: self.onProjectBuilt,
			'techAcquired'					: self.onTechAcquired,
			'techSelected'					: self.onTechSelected,
			'religionFounded'				: self.onReligionFounded,
			'religionSpread'				: self.onReligionSpread,
			'religionRemove'				: self.onReligionRemove,
			'corporationFounded'			: self.onCorporationFounded,
			'corporationSpread'				: self.onCorporationSpread,
			'corporationRemove'				: self.onCorporationRemove,
			'goldenAge'						: self.onGoldenAge,
			'endGoldenAge'					: self.onEndGoldenAge,
			'chat' 							: self.onChat,
			'victory'						: self.onVictory,
			'vassalState'					: self.onVassalState,
			'changeWar'						: self.onChangeWar,
			'setPlayerAlive'				: self.onSetPlayerAlive,
			'playerChangeStateReligion'		: self.onPlayerChangeStateReligion,
			'playerGoldTrade'				: self.onPlayerGoldTrade,
			'windowActivation'				: self.onWindowActivation,
			'gameUpdate'					: self.onGameUpdate,		# sample generic event
		}
		
		################## Events List ###############################
		#
		# Dictionary of Events, indexed by EventID (also used at popup context id)
		#   entries have name, beginFunction, applyFunction [, randomization weight...]
		#
		# Normal events first, random events after
		#
		################## Events List ###############################
		self.Events={
			CvUtil.EventEditCityName : ('EditCityName', self.__eventEditCityNameApply, self.__eventEditCityNameBegin),
			CvUtil.EventEditCity : ('EditCity', self.__eventEditCityApply, self.__eventEditCityBegin),
			CvUtil.EventPlaceObject : ('PlaceObject', self.__eventPlaceObjectApply, self.__eventPlaceObjectBegin),
			CvUtil.EventAwardTechsAndGold: ('AwardTechsAndGold', self.__eventAwardTechsAndGoldApply, self.__eventAwardTechsAndGoldBegin),
			CvUtil.EventEditUnitName : ('EditUnitName', self.__eventEditUnitNameApply, self.__eventEditUnitNameBegin),
			CvUtil.EventWBAllPlotsPopup : ('WBAllPlotsPopup', self.__eventWBAllPlotsPopupApply, self.__eventWBAllPlotsPopupBegin),
			CvUtil.EventWBLandmarkPopup : ('WBLandmarkPopup', self.__eventWBLandmarkPopupApply, self.__eventWBLandmarkPopupBegin),
			CvUtil.EventWBScriptPopup : ('WBScriptPopup', self.__eventWBScriptPopupApply, self.__eventWBScriptPopupBegin),
			CvUtil.EventWBStartYearPopup : ('WBStartYearPopup', self.__eventWBStartYearPopupApply, self.__eventWBStartYearPopupBegin),
			CvUtil.EventShowWonder: ('ShowWonder', self.__eventShowWonderApply, self.__eventShowWonderBegin),
			CvUtil.EventSetTurnsAutoPlayPopup : ('SetTurnsAutoPlayPopup', self.__eventSetTurnsAutoPlayApply, self.__eventSetTurnsAutoPlayBegin),
		}
## FfH Card Game: begin
		self.Events[CvUtil.EventSelectSolmniumPlayer] = ('selectSolmniumPlayer', self.__EventSelectSolmniumPlayerApply, self.__EventSelectSolmniumPlayerBegin)
		self.Events[CvUtil.EventSolmniumAcceptGame] = ('solmniumAcceptGame', self.__EventSolmniumAcceptGameApply, self.__EventSolmniumAcceptGameBegin)
		self.Events[CvUtil.EventSolmniumConcedeGame] = ('solmniumConcedeGame', self.__EventSolmniumConcedeGameApply, self.__EventSolmniumConcedeGameBegin)
## FfH Card Game: end

#################### EVENT STARTERS ######################
	def handleEvent(self, argsList):
		'EventMgr entry point'
		# extract the last 6 args in the list, the first arg has already been consumed
		self.origArgsList = argsList	# point to original
		tag = argsList[0]				# event type string
		idx = len(argsList)-6
		bDummy = False
		self.bDbg, bDummy, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[idx:]
		ret = 0
		if self.EventHandlerMap.has_key(tag):
			fxn = self.EventHandlerMap[tag]
			ret = fxn(argsList[1:idx])
		return ret

#################### EVENT APPLY ######################
	def beginEvent( self, context, argsList=-1 ):
		'Begin Event'
		entry = self.Events[context]
		return entry[2]( argsList )

	def applyEvent( self, argsList ):
		'Apply the effects of an event '
		context, playerID, netUserData, popupReturn = argsList

		if context == CvUtil.PopupTypeEffectViewer:
			return CvDebugTools.g_CvDebugTools.applyEffectViewer( playerID, netUserData, popupReturn )

		entry = self.Events[context]

		if ( context not in CvUtil.SilentEvents ):
			self.reportEvent(entry, context, (playerID, netUserData, popupReturn) )
		return entry[1]( playerID, netUserData, popupReturn )   # the apply function

	def reportEvent(self, entry, context, argsList):
		'Report an Event to Events.log '
		if (CyGame().getActivePlayer() != -1):
			message = "DEBUG Event: %s (%s)" %(entry[0], CyGlobalContext().getActivePlayer().getName())
			CyInterface().addImmediateMessage(message,"")
			CvUtil.pyPrint(message)
		return 0
	
################# MODULAR PYTHON HANDLER ############
## Modular Python: ANW 16-feb-2010
##                     29-may-2010
	
	def pluginScan(self):
		#print ("PluginScan called")
		for function in functionList:
			command[function] = []
		# Load internal definitions
		# Note: this was mainly for initial debug, so this part of code shouldn't be used
		if (CvPath.assetsPath[2] + "\\python\\plugins") in (glob.glob ( CvPath.assetsPath[2] + "\\python\\*")):
			for moduleSource in glob.glob ( CvPath.assetsPath[2] + "\\python\\plugins\\*.py" ):
				name = moduleSource.replace ( '.py', '' ).replace ( '\\', '/' ).split ( '/' ) [ -1 ].upper()
				if name[0:5] == "SPELL":
					continue
				handle = open ( moduleSource )
				module = imp.load_module ( name, handle, ( 'plugins/' + moduleSource ), ( '.py', 'r', imp.PY_SOURCE ) )
				# List all the functions the plugin has.
				functs = inspect.getmembers(module, inspect.isfunction)
				#each function is returned as a tuple (or maybe a list) 0, being the name, and 1 being the function itself
				for tuple in functs:
					for possFuncts in functionList:
						#since we only need the name of the function to match up, we only use the name in [0]
						if tuple[0] == possFuncts:
							#add it to our list of the applicable functions.
							command[possFuncts].append(module)
		# Load modules
		for modules in glob.glob(CvPath.assetsPath[2] + "\\Modules\\NormalModules\\*"):
			# Ignore all xml files in this folder, since they are MLF control files
			if modules[-4:] != ".xml":
				if modules+"\\python" in glob.glob(modules+"\\*"):
					for pythonFileSource in glob.glob(modules+"\\python\\*.py"):
						if pythonFileSource.replace ( '.py', '' ).replace ( '\\', '/' ).split ( '/' ) [ -1 ].upper()[0:5] == "SPELL":
							continue
						tempFileName = file(pythonFileSource)
						tempModuleName = pythonFileSource.replace ( '.py', '' ).replace ( '\\', '/' ).split ( '/' ) [ -1 ]
						module = imp.load_module( tempModuleName, tempFileName, tempModuleName+".py", ("","",1))
						# onLoad is a function all modules need so if they need to initialize anything they can
						# module.onLoad()
						# List all the functions the plugin has.
						functs = inspect.getmembers(module, inspect.isfunction)
						#each function is returned as a tuple (or maybe a list) 0, being the name, and 1 being the function itself
						for tuple in functs:
							for possFuncts in functionList:
								#since we only need the name of the function to match up, we only use the name in [0]
								if tuple[0] == possFuncts:
									#add it to our list of the applicable functions.
									command[possFuncts].append(module)
						tempFileName.close()
	
## Modular Python end
#################### ON EVENTS ######################
	def onKbdEvent(self, argsList):
		'keypress handler - return 1 if the event was consumed'

		eventType,key,mx,my,px,py = argsList
		game 		= CyGame()
		map 		= CyMap()
		cf			= self.cf

		if (self.bAllowCheats):
			# notify debug tools of input to allow it to override the control
			argsList = (eventType,key,self.bCtrl,self.bShift,self.bAlt,mx,my,px,py,game.isNetworkMultiPlayer())
			if ( CvDebugTools.g_CvDebugTools.notifyInput(argsList) ):
				return 0

		if ( eventType == self.EventKeyDown ):
			theKey=int(key)

#FfH: Added by Kael 07/05/2008
			if (theKey == int(InputTypes.KB_LEFT)):
				if self.bCtrl:
						CyCamera().SetBaseTurn(CyCamera().GetBaseTurn() - 45.0)
						return 1
				elif self.bShift:
						CyCamera().SetBaseTurn(CyCamera().GetBaseTurn() - 10.0)
						return 1

			if (theKey == int(InputTypes.KB_RIGHT)):
					if self.bCtrl:
							CyCamera().SetBaseTurn(CyCamera().GetBaseTurn() + 45.0)
							return 1
					elif self.bShift:
							CyCamera().SetBaseTurn(CyCamera().GetBaseTurn() + 10.0)
							return 1
#FfH: End Add

			CvCameraControls.g_CameraControls.handleInput( theKey )

			if (self.bAllowCheats):
				# Shift - T (Debug - No MP)
				if (theKey == int(InputTypes.KB_T)):
					if ( self.bShift ):
						self.beginEvent(CvUtil.EventAwardTechsAndGold)
						#self.beginEvent(CvUtil.EventCameraControlPopup)
						return 1

				elif (theKey == int(InputTypes.KB_W)):
					if ( self.bShift and self.bCtrl):
						self.beginEvent(CvUtil.EventShowWonder)
						return 1

				# Shift - ] (Debug - currently mouse-overd unit, health += 10
				elif (theKey == int(InputTypes.KB_LBRACKET) and self.bShift ):
					unit = map.plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = min( unit.maxHitPoints()-1, unit.getDamage() + 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )

				# Shift - [ (Debug - currently mouse-overd unit, health -= 10
				elif (theKey == int(InputTypes.KB_RBRACKET) and self.bShift ):
					unit = map.plot(px, py).getUnit(0)
					if ( not unit.isNone() ):
						d = max( 0, unit.getDamage() - 10 )
						unit.setDamage( d, PlayerTypes.NO_PLAYER )

				elif (theKey == int(InputTypes.KB_F1)):
					if ( self.bShift ):
						CvScreensInterface.replayScreen.showScreen(False)
						return 1
					# don't return 1 unless you want the input consumed

				elif (theKey == int(InputTypes.KB_F2)):
					if ( self.bShift ):
						import CvDebugInfoScreen
						CvScreensInterface.showDebugInfoScreen()
						return 1

				elif (theKey == int(InputTypes.KB_F3)):
					if ( self.bShift ):
						CvScreensInterface.showDanQuayleScreen(())
						return 1

				elif (theKey == int(InputTypes.KB_F4)):
					if ( self.bShift ):
						CvScreensInterface.showUnVictoryScreen(())
						return 1
						
						
			if (self.bAllowCheats or game.isMPOption(MultiplayerOptionTypes.MPOPTION_ENABLE_AUTOPLAY)):
				if (theKey == int(InputTypes.KB_X) and self.bShift and self.bCtrl):
					CyMessageControl().sendModNetMessage(CvUtil.AutoPlay, 1, -1, -1, -1)
				elif (theKey == int(InputTypes.KB_Z) and self.bShift and self.bCtrl):
					if game.getAIAutoPlay() > 0:
						CyMessageControl().sendModNetMessage(CvUtil.AutoPlay, 0, -1, -1, -1)
					else:
						cf.showAutoPlayPopup()
			
# Grey Fox Speed Tweaks: START
			if ( self.bShift ):
				if (theKey == int(InputTypes.KB_C)):
					if FoxGlobals["USE_DEBUG_WINDOW"]:
						self.DbgWnd.updateDebugPanel()
# END
		return 0

	def onModNetMessage(self, argsList):
		'Called whenever CyMessageControl().sendModNetMessage() is called - this is all for you modders!'
		gc = CyGlobalContext()
		game = CyGame()
		iData1, iData2, iData3, iData4, iData5 = argsList

#FfH Card Game: begin
#		print("Modder's net message!")
#		CvUtil.pyPrint( 'onModNetMessage' )
		if iData1 == CvUtil.Somnium : # iData1 == 0 : Somnium message, iData2 = function, iData3 to iData5 = parameters
			if iData2 == 0:
				if (iData3 == game.getActivePlayer()):
					self.__EventSelectSolmniumPlayerBegin()
			elif iData2 == 1:
				if (iData4 == game.getActivePlayer()):
					self.__EventSolmniumConcedeGameBegin((iData3, iData4))
			else :
				cs.applyAction(iData2, iData3, iData4, iData5)
# FfH Card Game: end
		elif iData1 == CvUtil.CivSelector:  #iData1==1 : CivSelect message,  iData2 = iCiv, iData3 = bAIPlayable, iData4 = bPlayable
			gc.getCivilizationInfo(iData2).setAIPlayable(iData3)
			gc.getCivilizationInfo(iData2).setPlayable(iData4)

		elif iData1 == CvUtil.AutoPlay:	#iData1 == 2: AutoPlay Config, iData2 = SetTurns
			game.setAIAutoPlay(iData2)

	def onInit(self, argsList):
		'Called when Civ starts up'
		CvUtil.pyPrint( 'OnInit' )
		
		## *******************
		## Modular Python: ANW 29-may-2010
		
		self.pluginScan()
		
		## Modular Python End
		## *******************

	def onUpdate(self, argsList):
		'Called every frame'
		fDeltaTime = argsList[0]

		# allow camera to be updated
		CvCameraControls.g_CameraControls.onUpdate( fDeltaTime )

	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]

	def onUnInit(self, argsList):
		'Called when Civ shuts down'
		CvUtil.pyPrint('OnUnInit')

	def onPreSave(self, argsList):
		"called before a game is actually saved"
		CvUtil.pyPrint('OnPreSave')
		
		if self.DbgWnd != None:
			CyGame().setScriptData(pickle.dumps(self.DbgWnd.saveData()))

	def onSaveGame(self, argsList):
		"return the string to be saved - Must be a string"
		return ""
	
	def verifyLoaded(self, bLoad = False):
		if self.LoadedData == False:
			self.loadData(bLoad)
	
	def loadData(self, bLoad = False):
		' Loading data and initializing variables '
		self.Tools 			= FoxTools.FoxTools()
		self.Tools.initialize()
		
		self.Defines		= self.Tools.getDefineDict()
		self.Eras 			= self.Tools.getEraDict()
		self.Techs			= self.Tools.getTechDict()
		self.Victories		= self.Tools.getVictoryDict()
		self.GameSpeeds 	= self.Tools.getGameSpeedDict()
		self.GameOptions 	= self.Tools.getGameOptionDict()
		self.EventTriggers	= self.Tools.getEventTriggerDict()
		
		self.Civilizations 	= self.Tools.getCivilizationDict()
		self.Leaders 		= self.Tools.getLeaderDict()
		self.LeaderStatus	= self.Tools.getLeaderStatusDict()
		self.Traits 		= self.Tools.getTraitDict()
		self.Civics 		= self.Tools.getCivicDict()
		self.Religions 		= self.Tools.getReligionDict()
		self.Corporations	= self.Tools.getCorporationDict()
		self.Alignments		= self.Tools.getAlignmentDict()
		
		self.Projects 		= self.Tools.getProjectDict()
		self.Buildings 		= self.Tools.getBuildingDict()
		self.Specialists	= self.Tools.getSpecialistDict()
		self.BuildingClasses= self.Tools.getBuildingClassDict()
		self.Processes		= self.Tools.getProcessesDict()
		
		self.Resources 		= self.Tools.getResourcesDict()
		self.WorldSizes 	= self.Tools.getWorldSizesDict()
		self.Terrain 		= self.Tools.getTerrainDict()
		self.Mana	 		= self.Tools.getManaDict()
		self.Goodies 		= self.Tools.getGoodyDict()
		
		self.Builds				= self.Tools.getBuildDict()
		self.Lairs 				= self.Tools.getLairDict()
		self.ManaNodes 			= self.Tools.getManaNodeDict()
		self.Improvements 		= self.Tools.getImprovementDict()
		self.CivImprovements 	= self.Tools.getCivImprovementDict()
		self.UniqueImprovements	= self.Tools.getUniqueImprovementDict()
		
		self.Units 			= self.Tools.getUnitDict()
		self.Heroes			= self.Tools.getHeroesDict()
		self.UnitAI			= self.Tools.getUnitAIDict()
		self.UnitClasses 	= self.Tools.getUnitClassDict()
		self.UnitCombats 	= self.Tools.getUnitCombatDict()
		self.GreatPeople 	= self.Tools.getGreatPeopleDict()
		self.Promotions 	= self.Tools.getPromotionDict()
		self.DamageTypes 	= self.Tools.getDamageTypesDict()
		
		self.cf				= CustomFunctions.CustomFunctions()
		self.cf.initialize()
		
		if bLoad:
			if FoxGlobals["USE_DEBUG_WINDOW"]:
				self.DbgWnd 		= FoxDebug.FoxDebug()
				self.DbgWnd.loadData(pickle.loads(CyGame().getScriptData()))
		
		self.LoadedData = True
		
	def onLoadGame(self, argsList):
		CvAdvisorUtils.resetNoLiberateCities()
		self.verifyLoaded(True)
		return 0

	def onGameStart(self, argsList):
		'Called at the start of the game'
		
		# Loading Debug data and Initializing dictionaries
		self.verifyLoaded()
		
		gc = CyGlobalContext() 
		cf				= self.cf
		getInfoType		= gc.getInfoTypeForString
		game 			= CyGame()
		map 			= CyMap()
		Option			= self.GameOptions
		getPlayer 		= gc.getPlayer
		plotByIndex 	= map.plotByIndex
		iNumPlots 		= map.numPlots()
		randNum 		= game.getSorenRandNum
		iMaxPlayers		= gc.getMAX_PLAYERS()
		iNoAI			= UnitAITypes.NO_UNITAI
		iSouth 			= DirectionTypes.DIRECTION_SOUTH
		
		if game.getWBMapScript():
			sf.gameStart()
		else:
			introMovie = CvIntroMovieScreen.CvIntroMovieScreen()
			introMovie.interfaceScreen()
		
		Terrain			= self.Terrain
		
		if Option["Wild Mana"] or ["Dark Forests"]:
			Animal		= self.Units["Animal"]
			
# WILD MANA 
		if Option["Wild Mana"]:
			# Units
			Summon 			= self.Units["Summons"]
			Kahdi			= self.Units["Kahdi"]
			Mercurian		= self.Units["Mercurian"]
			Elohim			= self.Units["Elohim"]
			Grigori			= self.Units["Grigori"]
			Promo	 		= self.Promotions
			
			# Mana
			Mana 			= self.Mana	
		
			bOrcPlayer 		= getPlayer(gc.getORC_PLAYER())
			bAnimalPlayer 	= getPlayer(gc.getANIMAL_PLAYER())
			bDemonPlayer 	= getPlayer(gc.getDEMON_PLAYER())
			demonInit		= bDemonPlayer.initUnit
			animalInit		= bAnimalPlayer.initUnit
			orcInit			= bOrcPlayer.initUnit
			
			lList = ['BONUS_MANA_SUN', 'BONUS_MANA_ICE', 'BONUS_MANA_AIR', 'BONUS_MANA_BODY', 'BONUS_MANA_CHAOS', 'BONUS_MANA_DEATH', 'BONUS_MANA_EARTH', 'BONUS_MANA_ENCHANTMENT', 'BONUS_MANA_ENTROPY', 'BONUS_MANA_FIRE', 'BONUS_MANA_LAW', 'BONUS_MANA_LIFE', 'BONUS_MANA_METAMAGIC', 'BONUS_MANA_MIND', 'BONUS_MANA_NATURE', 'BONUS_MANA_SHADOW', 'BONUS_MANA_SPIRIT', 'BONUS_MANA_WATER', 'BONUS_MANA_CREATION', 'BONUS_MANA_FORCE', 'BONUS_MANA_DIMENSIONAL', 'BONUS_MANA']
			
			# World Sizes
			WorldSize 	= self.WorldSizes
			
			iMana = 15
			worldSize = map.getWorldSize()
			if worldSize 	== WorldSize["Duel"]: 	iMana = iMana - 7;
			elif worldSize 	== WorldSize["Tiny"]: 	iMana = iMana - 5;
			elif worldSize 	== WorldSize["Small"]:	iMana = iMana - 3;
			elif worldSize 	== WorldSize["Large"]:	iMana = iMana + 3;
			elif worldSize 	== WorldSize["Huge"]:	iMana = iMana + 6;
			iMana=(int)(iMana)
			addBonus('BONUS_MANA',iMana,-1)
			
			randNum = game.getSorenRandNum
			for i in xrange(map.numPlots()):
				pPlot = plotByIndex(i)
				if pPlot.getImprovementType() == -1:
					if not pPlot.isWater():
						iBonus = pPlot.getBonusType(-1)
						if iBonus == Mana["Mana"]:
							iManaRnd = randNum(100, "Mana Creation")
							iConvertRnd = 60
							if Option["Feral Mana"]:
								iConvertRnd = 100
							if iManaRnd <= iConvertRnd:
								sMana = lList[randNum(len(lList), "Pick Mana")-1]
								iBonus = getInfoType(sMana)
								pPlot.setBonusType(iBonus)
								if Option["Mana Guardians"]:
									iX = pPlot.getX(); iY = pPlot.getY();
									if iBonus == Mana["Air"] :
										newUnit = demonInit( Summon["Lightning Elemental"], iX, iY, iNoAI, iSouth)
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Body"] :
										newUnit = demonInit( Summon["Flesh Golem"], iX, iY, iNoAI, iSouth)
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Chaos"]:
										newUnit = demonInit( Summon["Chaos Marauder"], iX, iY, iNoAI, iSouth)
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Death"]:
										newUnit = demonInit( Summon["Lich"], iX, iY, iNoAI, iSouth)
										newUnit.setHasPromotion( Promo["Held"], True)
									elif iBonus == Mana["Earth"]:
										newUnit = demonInit( Summon["Earth Elemental"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Enchantment"]:
										newUnit = orcInit( Summon["Wood Golem"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Entropy"]:
										newUnit = demonInit( Summon["Tar Golem"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Fire"]:
										newUnit = demonInit( Summon["Fire Elemental"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Law"]:
										newUnit = orcInit( Summon["Einherjar"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Life"]:
										newUnit = orcInit( Mercurian["Angel"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Metamagic"]:
										newUnit = demonInit( Kahdi["Thade"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Mind"]:
										newUnit = demonInit( Kahdi["Psion"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Shadow"]:
										newUnit = demonInit( Summon["Spectre"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Spirit"]:
										newUnit = orcInit( Elohim["Monk"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Sun"]:
										newUnit = orcInit( Summon["Aurealis"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Water"]:
										newUnit = demonInit( Summon["Water Elemental"], iX, iY, iNoAI, iSouth)
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Creation"]:
										newUnit = bAnimalPlayer.initUnit( Animal["Elk"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Dict["Force"]:
										newUnit = orcInit( Grigori["Dragon Slayer"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Dict["Dimensional"]:
										newUnit = demonInit( Kahdi["Uber Gnosling"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Ice"]:
										newUnit = demonInit( Summon["Ice Elemental"], iX, iY, iNoAI, iSouth)	
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)
									elif iBonus == Mana["Nature"]:
										newUnit = bAnimalPlayer.initUnit( Animal["Guardian Vines"], iX, iY, iNoAI, iSouth)				
										newUnit.setHasPromotion( Promo["Mana Guardian"], True)	
# END WILD MANA 

		if Option["Thaw"]:
# FlavourMod: Changed by Jean Elcard 11/06/2008 (Extended End of Winter Option)
			FLAT_WORLDS = ["ErebusWrap", "Erebus"]			# map scripts with wrapping but no equator
			MAX_EOW_PERCENTAGE = 0.35 						# percentage of EoW on total game turns
			THAW_DELAY_PERCENTAGE = 0.05 					# don't start thawing for x percent of EoW

			# forest varieties
			DECIDUOUS_FOREST = 0
			CONIFEROUS_FOREST = 1
			SNOWY_CONIFEROUS_FOREST = 2

			dice = game.getSorenRand()

			iTotalGameTurns = gc.getGameSpeedInfo(game.getGameSpeedType()).getGameTurnInfo(0).iNumGameTurnsPerIncrement
			iMaxEOWTurns = max(1, int(iTotalGameTurns * MAX_EOW_PERCENTAGE))
			iThawDelayTurns = max(1, int(iMaxEOWTurns * THAW_DELAY_PERCENTAGE))

			iMaxLatitude = max(map.getTopLatitude(), abs(map.getBottomLatitude()))
			bIsFlatWorld = not (map.isWrapX() or map.isWrapY()) or map.getMapScriptName() in FLAT_WORLDS
			
			getBonusInfo = gc.getBonusInfo
			for i in xrange(iNumPlots):
				pPlot 		= plotByIndex(i)
				eTerrain 	= pPlot.getTerrainType()
				eFeature 	= pPlot.getFeatureType()
				iVariety 	= pPlot.getFeatureVariety()
				eBonus 		= pPlot.getBonusType(TeamTypes.NO_TEAM)
				setTempT 	= pPlot.setTempTerrainTypeFM
				setTempF 	= pPlot.setTempFeatureType
				setTempB 	= pPlot.setTempBonusType

				iTurns = dice.get(iMaxEOWTurns - iThawDelayTurns, "End of Winter Thaw Requirement")
				if not bIsFlatWorld:
					iLatitude = abs(pPlot.getLatitude())
					iTurns = int(iTurns * ((float(iLatitude) / iMaxLatitude) ** 0.4))
				iTurns += iThawDelayTurns

				# cover erebus' oceans and lakes in ice
				if pPlot.isWater():
					if bIsFlatWorld:
						if dice.get(100, "End of Winter Iceburg Placement") < 90:
							setTempF(Terrain["Ice"], 0, iTurns)
					elif iLatitude + 10 > dice.get(50, "End of Winter Glacier Placement"):
						setTempF(Terrain["Ice"], 0, iTurns)

				# change terrains to colder climate versions
				if eTerrain == Terrain["Tundra"]:
					if dice.get(100, "Tundra to Snow") < 90:
						setTempT(Terrain["Snow"], iTurns, False, False)
				elif eTerrain == Terrain["Grass"]:
					if eFeature != Terrain["Jungle"]:
						if dice.get(100, "Grass to Snow or Tundra") < 60:
							setTempT(Terrain["Snow"], iTurns, False, False)
						else:
							setTempT(Terrain["Tundra"], iTurns, False, False)
				elif eTerrain == Terrain["Plains"]:
					if dice.get(100, "Plains to Snow or Tundra") < 30:
						setTempT(Terrain["Snow"], iTurns, False, False)
					else:
						setTempT(Terrain["Tundra"], iTurns, False, False)
				elif eTerrain == Terrain["Desert"]:
					if dice.get(100, "Desert to Tundra or Plains") < 50:
						setTempT(Terrain["Tundra"], iTurns, False, False)
					else:
						setTempT(Terrain["Plains"], iTurns, False, False)
				elif eTerrain == Terrain["Marsh"]:
					setTempT(Terrain["Grass"], iTurns, False, False)

				# change all features (except ice) to colder climate versions
				if eFeature == Terrain["Forest"]:
					if iVariety == DECIDUOUS_FOREST:
						setTempF(Terrain["Forest"], CONIFEROUS_FOREST, iTurns)
					elif iVariety == CONIFEROUS_FOREST:
						setTempF(Terrain["Forest"], SNOWY_CONIFEROUS_FOREST, iTurns)
				elif eFeature != FeatureTypes.NO_FEATURE:
					if eFeature != Terrain["Ice"]:
						setTempF(Terrain["Forest"], DECIDUOUS_FOREST, iTurns)

				# remove invalid bonuses or replace them (if food) with a valid surrogate
				if eBonus != BonusTypes.NO_BONUS:
					setBonusType = pPlot.setBonusType
					setBonusType(BonusTypes.NO_BONUS)
					canHaveBonus = pPlot.canHaveBonus
					if not canHaveBonus(eBonus, True):
						if getBonusInfo(eBonus).getYieldChange(YieldTypes.YIELD_FOOD) > 0:
							iPossibleTempFoodBonuses = []
							for iLoopBonus in xrange(gc.getNumBonusInfos()):
								if getBonusInfo(iLoopBonus).getYieldChange(YieldTypes.YIELD_FOOD) > 0:
									if canHaveBonus(iLoopBonus, True):
										iPossibleTempFoodBonuses.append(iLoopBonus)
							setBonusType(eBonus)
							if len(iPossibleTempFoodBonuses) > 0:
								setTempB(iPossibleTempFoodBonuses[dice.get(len(iPossibleTempFoodBonuses), "End of Winter Food Thawing")], iTurns)
							else:
								setTempB(BonusTypes.NO_BONUS, iTurns)
						else:
							setBonusType(eBonus)
							setTempB(BonusTypes.NO_BONUS, iTurns)
					else:
						setBonusType(eBonus)
# FlavourMod: End Add
		
		ScorpClan 	= self.Units["Scorpion Clan"]
		Civ 		= self.Civilizations
		Trait 		= self.Traits
		Lair 		= self.Lairs
		
		bPlayer 	= getPlayer(gc.getORC_PLAYER())
		initUnit 	= bPlayer.initUnit
		for i in xrange(iNumPlots):
			pPlot = plotByIndex(i)
			iImprovement = pPlot.getImprovementType()
			if iImprovement == Lair["Goblin Fort"]:
				if not Option["No Barbarians"]:
					initUnit( ScorpClan["Archer"], pPlot.getX(), pPlot.getY(), iNoAI, iSouth)
		
		for iPlayer in xrange(iMaxPlayers):
			player = getPlayer(iPlayer)
			if (player.isAlive() and player.isHuman()):
				if player.getCivilizationType() == Civ["Elohim"]:
					showUniqueImprovements(iPlayer)

			if (player.isAlive()):
				if player.hasTrait(Trait["Feral"]):
					iAnimalTeam = gc.getANIMAL_TEAM()
					eTeam = gc.getTeam(player.getTeam())
					eTeam.makePeace(iAnimalTeam)
				
		if (game.getGameTurnYear() == self.Defines["Start Year"] and not Option["Advanced Start"]):
			if not game.getWBMapScript():
				for iPlayer in xrange(iMaxPlayers):
					player = getPlayer(iPlayer)
					if (player.isAlive() and player.isHuman()):
						popupInfo = CyPopupInfo()
						popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
						popupInfo.setText(u"showDawnOfMan")
						popupInfo.addPopup(iPlayer)
		else:
			CyInterface().setSoundSelectionReady(True)
		
		if game.isPbem():
			for iPlayer in xrange(iMaxPlayers):
				player = gc.getPlayer(iPlayer)
				if (player.isAlive() and player.isHuman()):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_DETAILS)
					popupInfo.setOption1(True)
					popupInfo.addPopup(iPlayer)

		CvAdvisorUtils.resetNoLiberateCities()

# FF: Added by Jean Elcard 14/01/2009 (speed tweak)
		rebuildGraphics()
# FF: End Add

# FF Wilderness
		if Option["Dark Forests"]:
			pPlayer  = getPlayer(gc.getANIMAL_PLAYER())
			initUnit = pPlayer.initUnit
			iRnd 	 = randNum(100, "Wilderness chance")
			if iRnd < 55:
				iBestValue = 0
				pBestPlot = -1
				for i in xrange(iNumPlots):
					iValue = 0
					pPlot = plotByIndex(i)
					iFeature = pPlot.getFeatureType()
					if not pPlot.isWater():
						if (iFeature == Terrain["Forest"] or iFeature == Terrain["Jungle"]):
								iValue = randNum(1000, "Looking for Blighted Forest spot")
								if not pPlot.isOwned():
									iValue += 1000
								if iValue > iBestValue:
									iBestValue = iValue
									pBestPlot = pPlot
				if pBestPlot != -1:
					iForestCreeper = Animal["Forest Creeper"]
					iX = pBestPlot.getX(); iY = pBestPlot.getY();
					newUnit = initUnit(Animal["Malignant Flora"], iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iForestCreeper, iX, iY, iNoAI, iSouth)
					pBestPlot.setImprovementType( self.Lairs["Blighted Forest"] )
			if iRnd >= 45:
				iBestValue 	= 0
				pBestPlot 	= -1
				for i in xrange(iNumPlots):
					iValue = 0
					pPlot = plotByIndex(i)
					if not pPlot.isWater():
						iFeature = pPlot.getFeatureType()
						if (iFeature == Terrain["Forest"] or iFeature == Terrain["Jungle"] or pPlot.getTerrainType() == Terrain["Desert"]):
								iValue = randNum(1000, "Diakonos placement spot")
								if not pPlot.isOwned():
									iValue += 1000
								if iValue > iBestValue:
									iBestValue = iValue
									pBestPlot = pPlot
				if pBestPlot != -1:
					iDiakonos = Animal["Diakonos"]
					iX = pBestPlot.getX(); iY = pBestPlot.getY() 
					newUnit = initUnit(iDiakonos, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iDiakonos, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iDiakonos, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iDiakonos, iX, iY, iNoAI, iSouth)
					newUnit = initUnit(iDiakonos, iX, iY, iNoAI, iSouth)

# FF end Wilderness

	def onGameEnd(self, argsList):
		'Called at the End of the game'
		print("Game is ending")
		return

	def onBeginGameTurn(self, argsList):
		'Called at the beginning of the end of each turn'
		iGameTurn = argsList[0]
	
		gc = CyGlobalContext() 
		cf			= self.cf
		game 		= CyGame()
		map 		= CyMap()
		isOption 	= game.isOption
		getPlayer 	= gc.getPlayer
		iOrcPlayer 	= gc.getORC_PLAYER()
		
		if iGameTurn == 3:
			if not game.isNetworkMultiPlayer():
				t = "TROPHY_FEAT_INTRODUCTION"
				if not game.isHasTrophy(t):
					game.changeTrophyValue(t, 1)
					addPopupWB(CyTranslator().getText("TXT_KEY_FFH_INTRO",()),'art/interface/popups/FfHIntro.dds')

		iOrthusTurn = 75
		isMaxedOut	= game.isUnitClassMaxedOut
		eSpeed		= game.getGameSpeedType()
		Speed		= self.GameSpeeds
		if not isMaxedOut(self.UnitClasses["Orthus"], 0):
			if not isOption(self.GameOptions["No Orthus"]):
				bOrthus = False
				if eSpeed == Speed["Quick"]:
					if iGameTurn >= iOrthusTurn / 3 * 2: bOrthus = True
				elif eSpeed == Speed["Normal"]:
					if iGameTurn >= iOrthusTurn: 		 bOrthus = True
				elif eSpeed == Speed["Epic"]:
					if iGameTurn >= iOrthusTurn * 3 / 2: bOrthus = True
				elif eSpeed == Speed["Marathon"]:
					if iGameTurn >= iOrthusTurn * 3: 	 bOrthus = True
				if bOrthus:
					addUnit(self.Heroes["Orthus"], iOrcPlayer)
					
		iZarcazTurn = 100
		if not isMaxedOut(self.UnitClasses["Zarcaz"], 0):
			if not isOption(self.GameOptions["No Orthus"]):
				bZarcaz = False
				if eSpeed == Speed["Quick"]:
					if iGameTurn >= iZarcazTurn / 3 * 2: bZarcaz = True
				elif eSpeed == Speed["Normal"]:
					if iGameTurn >= iZarcazTurn: 		 bZarcaz = True
				elif eSpeed == Speed["Epic"]:
					if iGameTurn >= iZarcazTurn * 3 / 2: bZarcaz = True
				elif eSpeed == Speed["Marathon"]:
					if iGameTurn >= iZarcazTurn * 3: 	 bZarcaz = True
				if bZarcaz:
					addUnit(self.Heroes["Zarcaz"], iOrcPlayer)
					Race 		= self.Promotions["Race"]
					Equipment 	= self.Promotions["Equipment"]
					Unit = self.Units["Scorpion Clan"]
					
					pPlayer = getPlayer(iOrcPlayer)
					py = PyPlayer(iOrcPlayer)
					iNoAI = UnitAITypes.NO_UNITAI
					iSouth = DirectionTypes.DIRECTION_SOUTH
					initUnit = pPlayer.initUnit
					for pUnit in py.getUnitList():
						if (pUnit.getRace() == Race["Goblinoid"] and pUnit.isHasPromotion(Equipment["Zarcazs Bow"]) == True):
							pPlot = pUnit.plot()
							iX = pPlot.getX(); iY = pPlot.getY()
							
							newUnit2 = initUnit(Unit["Archer"], iX, iY, iNoAI, iSouth)
							pUnit.addMinion(newUnit2)
							
							newUnit3 = initUnit(Unit["Goblin"], iX, iY, iNoAI, iSouth)
							pUnit.addMinion(newUnit3)
							
							newUnit4 = initUnit(Unit["Wolf Rider"], iX, iY, iNoAI, iSouth)
							pUnit.addMinion(newUnit4)

# FfH Card Game: begin
		cs.doTurn()
# FfH Card Game: end

		cf.doFFTurn()
		
		
		## *******************
		## Modular Python: ANW 29-may-2010
		
		for module in command['onBeginGameTurn']:
			module.onBeginGameTurn(self, argsList)
			
		## Modular Python End
		## *******************
		
#Added in Frozen: TC01 (Blizzard Utils)
		Blizzards.doBlizzardTurn()
#End of Frozen

		if game.getAIAutoPlay() == 0:
			CvTopCivs.CvTopCivs().turnChecker(iGameTurn)

	def onEndGameTurn(self, argsList):
		'Called at the end of the end of each turn'
		iGameTurn = argsList[0]
		
		gc = CyGlobalContext() 
		game = CyGame()
		map = CyMap()
		getPlayer = gc.getPlayer
		iDemonTeam = gc.getDEMON_TEAM()
		
# FF: Added by Jean Elcard 14/01/2009 (speed tweak)
		rebuildGraphics()
# FF: End Add

		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onEndGameTurn']:
			module.onEndGameTurn(self, argsList)
			
		## Modular Python End
		## *******************
	
				
		if game.isVictoryValid(self.Victories["Gone to Hell"]):
			iMaxPlayers = gc.getMAX_PLAYERS()
			iTotalTiles = map.numPlots()
			iEvilTiles = 0

			for iLoop in xrange(map.getNumAreas()):
				iEvilTiles += map.getArea(iLoop).getNumEvilTiles()


			if (100.0 * iEvilTiles) / iTotalTiles > self.Defines["Gone to Hell"]:

				pMostEvilPlayer = -1
				iMostEvilTotalContrib = 0

				for iLoopPlayer in xrange(iMaxPlayers):
					pLoopPlayer = getPlayer(iLoopPlayer)
					if pLoopPlayer.isAlive():
						if pLoopPlayer.getGlobalCounterContrib() > iMostEvilTotalContrib:
							pMostEvilPlayer = pLoopPlayer
							iMostEvilTotalContrib = pLoopPlayer.getGlobalCounterContrib()
				
				if pMostEvilPlayer != -1:
					pMostEvilPlayer.setTeam(iDemonTeam)

				game.setWinner(iDemonTeam, self.Victories["Gone to Hell"])
		
		if FoxGlobals["USE_AIAUTOPLAY_SOUND"]:
			if game.getAIAutoPlay() == 1:
				pCapital = getPlayer(0).getCapitalCity().plot()
				if pCapital != None:
					CyCamera().JustLookAtPlot(pCapital)
					point 			= pCapital.getPoint()
					profile			= CyUserProfile()
					Play3DSound		= CyAudioGame().Play3DSound
					profile.setMasterNoSound(False)
					profile.setMasterVolume(50)
					profile.setSpeechVolume(50)
					Play3DSound("AS3D_ENGLAND_SELECT",point.x,point.y,point.z)
					profile.setMasterVolume(SoundSettings["SOUND_MASTER_VOLUME"])
					profile.setSpeechVolume(SoundSettings["SOUND_SPEECH_VOLUME"])
					profile.setMasterNoSound(SoundSettings["SOUND_MASTER_NO_SOUND"])

	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList
		gc 				= CyGlobalContext() 
		cf				= self.cf
		game 			= CyGame()
		getPlayer 		= gc.getPlayer
		iDemonTeam 		= gc.getDEMON_TEAM()
		pPlayer 		= getPlayer(iPlayer)
		bAI		 		= self.Tools.isAI(iPlayer)
		hasTrait 		= pPlayer.hasTrait
		isCivic 		= pPlayer.isCivic
		randNum 		= game.getSorenRandNum
		
		eCiv 			= pPlayer.getCivilizationType()
		eRel 			= pPlayer.getStateReligion()
		eEra 			= pPlayer.getCurrentEra()
		setEra 			= pPlayer.setCurrentEra
		Rel				= self.Religions
		Civ				= self.Civilizations
		Civic			= self.Civics
		Trait			= self.Traits
		Era				= self.Eras
		eSpeed 			= game.getGameSpeedType()
		Speed			= self.GameSpeeds
		Status			= self.LeaderStatus
		Event			= self.EventTriggers
		trigger			= pPlayer.trigger
		triggerData		= pPlayer.initTriggeredData
		
		print ("GameTurn: "+str(iGameTurn))
		print (PyPlayer(iPlayer).getCivilizationName())
		
		
		if   eRel == Rel["Ashen Veil"]	 		and eEra != Era["AV"]: 	 setEra(Era["AV"])
		elif eRel == Rel["Order"]				and eEra != Era["Order"]:setEra(Era["Order"])
		elif eRel == Rel["Fellowship"]			and eEra != Era["FoL"]:  setEra(Era["FoL"])
		elif eRel == Rel["Runes of Kilmorph"]	and eEra != Era["RoK"]:  setEra(Era["RoK"])
		elif eRel == Rel["Octopus Overlords"]  	and eEra != Era["OO"]: 	 setEra(Era["OO"])
		elif eRel == Rel["Empyrean"]			and eEra != Era["Empy"]: setEra(Era["Empy"])
		elif eRel == Rel["Council of Esus"]		and eEra != Era["CoE"]:  setEra(Era["CoE"])
		elif eRel == Rel["Machinarum"]			and eEra != Era["Mach"]: setEra(Era["Mach"])

		if bAI and not game.getWBMapScript(): cf.warScript(iPlayer)
		if isCivic(Civic["Crusade"]): 	cf.doCrusade(iPlayer)
		
		if   eCiv == Civ["Khazad"]: 	cf.doTurnKhazad(iPlayer)
		elif eCiv == Civ["Luchuirp"]: 	cf.doTurnLuchuirp(iPlayer)
		elif eCiv == Civ["Archos"]: 	cf.doTurnArchos(iPlayer)
		elif eCiv == Civ["Scions"]: 	cf.doTurnScions(iPlayer)			
		elif eCiv == Civ["Grigori"]:	cf.doTurnGrigori(iPlayer)
		elif eCiv == Civ["Cualli"]:		cf.doTurnCualli(iPlayer)
		
		if hasTrait(Trait["Insane"]): 
			if randNum(1000, "Insane") < 20: trigger( Event["Insane"])

		if hasTrait(Trait["Adaptive"]):
			iCycle = 100
			if   eSpeed == Speed["Quick"]: 	  iCycle = 75
			elif eSpeed == Speed["Epic"]: 	  iCycle = 150
			elif eSpeed == Speed["Marathon"]: iCycle = 300
			for i in xrange(10):
				if (i * iCycle) - 5 == iGameTurn:
					triggerData( Event["Adaptive"], True, -1, -1, -1, iPlayer, -1, -1, -1, -1, -1)

		if pPlayer.getLeaderStatus() == Status["Emergent"]:
			if pPlayer.getPlayersKilled() >= 1 and not hasTrait(Trait["Aggressive"]):
				triggerData( Event["Aggressive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		if hasTrait(Trait["Barbarian"]):
			eTeam = gc.getTeam(gc.getPlayer(gc.getORC_PLAYER()).getTeam())
			iTeam = pPlayer.getTeam()
			getScore = game.getPlayerScore
			if eTeam.isAtWar(iTeam) == False:
				if getScore(iPlayer) >= 2 * getScore(game.getRankPlayer(1)):
					if iGameTurn >= 20:
						eTeam.declareWar(iTeam, False, WarPlanTypes.WARPLAN_LIMITED)
						if (iPlayer == game.getActivePlayer() and not bAI):
							addPopup(CyTranslator().getText("TXT_KEY_POPUP_BARBARIAN_DECLARE_WAR",()), 'art/interface/popups/Barbarian.dds')
		
		
		if pPlayer.isCivic(Civic["Republic"]):
			if randNum(3, "Election Year") > 0:
				iCycle = 40
				if   eSpeed == Speed["Quick"]: 	  iCycle = 20
				elif eSpeed == Speed["Epic"]: 	  iCycle = 60
				elif eSpeed == Speed["Marathon"]: iCycle = 80
				if iGameTurn % iCycle == 0:
					cf.resetRepublicTraits(pPlayer)
					iElection = randNum(4, "Election Type")
					if   iElection == 0: trigger( Event["Hawk vs Dove"])
					elif iElection == 1: trigger( Event["Landowner vs Peasant"])
					elif iElection == 2: trigger( Event["Church vs State"])
					elif iElection == 3: trigger( Event["Labor vs Academia"])

		## *******************	
		## Modular Python: ANW 29-may-2010
		
		for module in command['onBeginPlayerTurn']:
			module.onBeginPlayerTurn(self, argsList)

		## Modular Python End
		## *******************
		
		if pPlayer.isHuman():
			self.Tools.showTraitPopup()
		
	def onEndPlayerTurn(self, argsList):
		'Called at the end of a players turn'
		iGameTurn, iPlayer = argsList
		gc 			= CyGlobalContext() 
		cf			= self.cf
		game 		= CyGame()
		getPlayer 	= gc.getPlayer
		pPlayer 	= getPlayer(iPlayer)
		
		if (game.getElapsedGameTurns() == 1):
			if (pPlayer.isHuman()):
				if (pPlayer.canRevolution(0)):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
					popupInfo.addPopup(iPlayer)

		CvAdvisorUtils.resetAdvisorNags()
		CvAdvisorUtils.endTurnFeats(iPlayer)
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onEndPlayerTurn']:
			module.onEndPlayerTurn(self, argsList)

		## Modular Python End
		## *******************
		
# FF: Added by Jean Elcard 14/01/2009 (speed tweak)
		rebuildGraphics()
# FF: End Add

	def onEndTurnReady(self, argsList):
		iGameTurn = argsList[0]

	def onFirstContact(self, argsList):
		'Contact'
		iTeamX,iHasMetTeamY = argsList
		if (not self.__LOG_CONTACT):
			return
		CvUtil.pyPrint('Team %d has met Team %d' %(iTeamX, iHasMetTeamY))

	def onCombatResult(self, argsList):
		'Combat Result'
		pWinner,pLoser = argsList
		playerX = PyPlayer(pWinner.getOwner())
		unitX = PyInfo.UnitInfo(pWinner.getUnitType())
		playerY = PyPlayer(pLoser.getOwner())
		unitY = PyInfo.UnitInfo(pLoser.getUnitType())
		if (not self.__LOG_COMBAT):
			return
		if playerX and playerX and unitX and playerY:
			CvUtil.pyPrint('Player %d Civilization %s Unit %s has defeated Player %d Civilization %s Unit %s'
				%(playerX.getID(), playerX.getCivilizationName(), unitX.getDescription(),
				playerY.getID(), playerY.getCivilizationName(), unitY.getDescription()))

	def onCombatLogCalc(self, argsList):
		'Combat Result'
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iCombatOdds = genericArgs[2]
		CvUtil.combatMessageBuilder(cdAttacker, cdDefender, iCombatOdds)

	def onCombatLogHit(self, argsList):
		'Combat Message'
		global gCombatMessages, gCombatLog
		gc 			= CyGlobalContext()
		genericArgs = argsList[0][0]
		cdAttacker = genericArgs[0]
		cdDefender = genericArgs[1]
		iIsAttacker = genericArgs[2]
		iDamage = genericArgs[3]

		if cdDefender.eOwner == cdDefender.eVisualOwner:
			szDefenderName = gc.getPlayer(cdDefender.eOwner).getNameKey()
		else:
			szDefenderName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())
		if cdAttacker.eOwner == cdAttacker.eVisualOwner:
			szAttackerName = gc.getPlayer(cdAttacker.eOwner).getNameKey()
		else:
			szAttackerName = localText.getText("TXT_KEY_TRAIT_PLAYER_UNKNOWN", ())

		if (iIsAttacker == 0):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szDefenderName, cdDefender.sUnitName, iDamage, cdDefender.iCurrHitPoints, cdDefender.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdDefender.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szAttackerName, cdAttacker.sUnitName, szDefenderName, cdDefender.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
		elif (iIsAttacker == 1):
			combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_HIT", (szAttackerName, cdAttacker.sUnitName, iDamage, cdAttacker.iCurrHitPoints, cdAttacker.iMaxHitPoints))
			CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
			CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)
			if (cdAttacker.iCurrHitPoints <= 0):
				combatMessage = localText.getText("TXT_KEY_COMBAT_MESSAGE_DEFEATED", (szDefenderName, cdDefender.sUnitName, szAttackerName, cdAttacker.sUnitName))
				CyInterface().addCombatMessage(cdAttacker.eOwner,combatMessage)
				CyInterface().addCombatMessage(cdDefender.eOwner,combatMessage)

	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList
		self.verifyLoaded()
		gc = CyGlobalContext() 
		Improvement = self.Improvements
		map = CyMap()
		pPlot = map.plot(iX, iY)
		
		if gc.getImprovementInfo(iImprovement).isUnique():
			CyEngine().addLandmark(pPlot, CvUtil.convertToStr(gc.getImprovementInfo(iImprovement).getDescription()))
			
			Unique = self.UniqueImprovements
			Malakim = self.CivImprovements["Malakim"]
			
			if   iImprovement == Unique["Ring of Carcer"]:
				pPlot.setMinLevel(15)

			elif iImprovement == Unique["Blair of Lacuna"]:
				pPlot.setMinLevel(6)
				pPlot.setRouteType(Improvement["Road"])

		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was built at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onImprovementDestroyed(self, argsList):
		'Improvement Destroyed'
		iImprovement, iOwner, iX, iY = argsList
		Improvement = self.Improvements
		gc = CyGlobalContext() 
		game = CyGame()
		map = CyMap()
		
		if iImprovement != -1:
			if gc.getImprovementInfo(iImprovement).isUnique():
				Unique = self.UniqueImprovements
				pPlot = map.plot(iX, iY)
				CyEngine().removeLandmark(pPlot)

				if   iImprovement == Unique["Ring of Carcer"]:
					pPlot.setMinLevel(-1)

				elif iImprovement == Unique["Blair of Lacuna"]:
					pPlot.setMinLevel(-1)

		if iImprovement == Improvement["Necrototem"]:
			game.changeGlobalCounter(-2)

		if game.getWBMapScript():
			sf.onImprovementDestroyed(iImprovement, iOwner, iX, iY)

		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Improvement %s was Destroyed at %d, %d'
			%(PyInfo.ImprovementInfo(iImprovement).getDescription(), iX, iY))

	def onRouteBuilt(self, argsList):
		'Route Built'
		iRoute, iX, iY = argsList
		gc = CyGlobalContext() 
		
		if (not self.__LOG_IMPROVEMENT):
			return
		CvUtil.pyPrint('Route %s was built at %d, %d'
			%(gc.getRouteInfo(iRoute).getDescription(), iX, iY))

	def onPlotRevealed(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iTeam = argsList[1]

	def onPlotFeatureRemoved(self, argsList):
		'Plot Revealed'
		pPlot = argsList[0]
		iFeatureType = argsList[1]
		pCity = argsList[2] # This can be null

	def onPlotPicked(self, argsList):
		'Plot Picked'
		pPlot = argsList[0]
		CvUtil.pyPrint('Plot was picked at %d, %d'
			%(iX, iY))

	def onNukeExplosion(self, argsList):
		'Nuke Explosion'
		pPlot, pNukeUnit = argsList
		CvUtil.pyPrint('Nuke detonated at %d, %d'
			%(iX, iY))

	def onGotoPlotSet(self, argsList):
		'Nuke Explosion'
		pPlot, iPlayer = argsList

	def onBuildingBuilt(self, argsList):
		'Building Completed'
		pCity, iBuildingType = argsList
		player = pCity.getOwner()
		
		gc 			= CyGlobalContext() 
		cf			= self.cf
		getInfoType	= gc.getInfoTypeForString
		game 		= CyGame()
		randNum 	= game.getSorenRandNum
		getPlayer 	= gc.getPlayer
		pPlayer 	= getPlayer(player)
		getTeam		= gc.getTeam
		hasTrait 	= pPlayer.hasTrait
		iStatus 	= pPlayer.getLeaderStatus()
		iNoAI		= UnitAITypes.NO_UNITAI
		iNorth 		= DirectionTypes.DIRECTION_NORTH
		iSouth 		= DirectionTypes.DIRECTION_SOUTH
		Building	= self.Buildings
		setNumRealBuilding = pCity.setNumRealBuilding
		iX			= pCity.getX()
		iY			= pCity.getY()
		Trait		= self.Traits
		Status		= self.LeaderStatus
		Event		= self.EventTriggers
		Civ			= self.Civilizations
		triggerData	= pPlayer.initTriggeredData
		
		pPlot = pCity.plot()
		
		iBuildingClass = gc.getBuildingInfo(iBuildingType).getBuildingClassType()

		if ((not game.isNetworkMultiPlayer()) and (pCity.getOwner() == game.getActivePlayer()) and isWorldWonderClass(iBuildingClass)):
			if gc.getBuildingInfo(iBuildingType).getMovie():
				# If this is a wonder...
				popupInfo = CyPopupInfo()
				popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
				popupInfo.setData1(iBuildingType)
				popupInfo.setData2(pCity.getID())
				popupInfo.setData3(0)
				popupInfo.setText(u"showWonderMovie")
				popupInfo.addPopup(pCity.getOwner())

		if iBuildingType == Building["Infernal Grimoire"]:
			if randNum(100, "Grimoire Effect") <= 20:
				pPlot2 = findClearPlot(-1, pPlot)
				if pPlot2 != -1:
					Unit	= self.Units["Infernal"]
					bPlayer = getPlayer(gc.getDEMON_PLAYER())
					newUnit = bPlayer.initUnit(Unit["Balor"], pPlot2.getX(), pPlot2.getY(), iNoAI, iNorth)
					CyInterface().addMessage(pCity.getOwner(),True,25,CyTranslator().getText("TXT_KEY_MESSAGE_INFERNAL_GRIMOIRE_BALOR",()),'AS2D_BALOR',1,'Art/Interface/Buttons/Units/Balor.dds',ColorTypes(7),newUnit.getX(),newUnit.getY(),True,True)
					if pCity.getOwner() == game.getActivePlayer():
						addPopup(CyTranslator().getText("TXT_KEY_POPUP_INFERNAL_GRIMOIRE_BALOR",()), 'art/interface/popups/Balor.dds')
		
		elif iBuildingType == Building["Pact of the Nilhorn"]:
			initUnit = pPlayer.initUnit
			Unit 	 = self.Units["Savage"]
			Promo	 = self.Promotions["Effects"]
			Race	 = self.Promotions["Race"]
			newUnit1 = initUnit(Unit["Hill Giant"], iX, iY, iNoAI, iNorth)
			newUnit1.setHasPromotion(Promo["Hidden Nationality"], True)
			newUnit1.setHasPromotion(Race["Undead"], False)

			if newUnit1.getRace() != -1:
				newUnit1.setHasPromotion(newUnit1.getRace(), False)
			newUnit1.setHasPromotion(Race["Giantkin"], True)
			newUnit1.setName("Larry")
			newUnit2 = initUnit(Unit["Hill Giant"], iX, iY, iNoAI, iNorth)
			newUnit2.setHasPromotion(Promo["Hidden Nationality"], True)
			newUnit2.setHasPromotion(Race["Undead"], False)
			if newUnit2.getRace() != -1:
				newUnit2.setHasPromotion(newUnit2.getRace(), False)
			newUnit2.setHasPromotion(Race["Giantkin"], True)
			newUnit2.setName("Curly")

			newUnit3 = initUnit(Unit["Hill Giant"], iX, iY, iNoAI, iNorth)
			newUnit3.setHasPromotion(Promo["Hidden Nationality"], True)
			newUnit3.setHasPromotion(Race["Undead"], False)
			if newUnit3.getRace() != -1:
				newUnit3.setHasPromotion(newUnit3.getRace(), False)
			newUnit3.setHasPromotion(Race["Giantkin"], True)
			newUnit3.setName("Moe")

		elif iBuildingType == Building["Jotnar Monument"]:
			Unit = self.Units["Jotnar"]
			newUnit1 = pPlayer.initUnit(Unit["Jotnar Citizens"], iX, iY, iNoAI, iNorth)
		
		elif iBuildingType == Building["Catacomb Libralus"]:
			if iStatus == Status["Emergent"] and not hasTrait(Trait["Arcane"]):
				triggerData(Event["Arcane"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
		
		elif iBuildingType == Building["Forbidden Palace"]:
			if iStatus == Status["Emergent"] and not hasTrait(Trait["Organized"]):
				triggerData(Event["Organized"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
		
		elif iBuildingType ==  Building["Altar - Final"]:
			setNumRealBuilding(Building["Altar - Exalted"], 0)
		elif iBuildingType ==  Building["Altar - Exalted"]:
			setNumRealBuilding(Building["Altar - Divine"], 0)
		elif iBuildingType ==  Building["Altar - Divine"]:
			setNumRealBuilding(Building["Altar - Consecrated"], 0)
		elif iBuildingType ==  Building["Altar - Consecrated"]:
			setNumRealBuilding(Building["Altar - Blessed"], 0)
		elif iBuildingType ==  Building["Altar - Blessed"]:
			setNumRealBuilding(Building["Altar - Anointed"], 0)
		elif iBuildingType ==  Building["Altar - Anointed"]:
			setNumRealBuilding(Building["Altar of Luonnotar"], 0)
			if iStatus == Status["Emergent"] and not hasTrait(Trait["Magic Resistant"]):
				if not hasTrait(Trait["Ingenuity"]): 
					triggerData(Event["Magic Resistant"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		elif iBuildingType == Building["Mercurian Gate"]:
			if not game.isCivEverActive(Civ["Mercurians"]):
				iMercurianPlayer = getOpenPlayer()
				pMercurians = gc.getPlayer(iMercurianPlayer)
				iTeam = pPlayer.getTeam()
				pPlot2 = findClearPlot(-1, pCity.plot())
				if (iMercurianPlayer != -1 and pPlot2 != -1):
					Unit 	 = self.Units["Mercurian"]
					Generic	 = self.Units["Generic"]
					initUnit = pMercurians.initUnit
					getUnit  = pPlot.getUnit
					iX = pPlot.getX(); iY = pPlot.getY();
					for i in xrange(pPlot.getNumUnits(), -1, -1):
						pUnit = getUnit(i)
						pUnit.setXY(pPlot2.getX(), pPlot2.getY(), True, True, True)
					game.addPlayerAdvanced(iMercurianPlayer, iTeam, self.Leaders["Basium"], Civ["Mercurians"])
					basiumUnit = initUnit(self.Heroes["Basium"], iX, iY, iNoAI, iNorth)
					basiumUnit.setExperienceTimes100(2500, -1)
					initUnit(Generic["Settler"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					initUnit(Unit["Angel"], iX, iY, iNoAI, iNorth)
					if pPlayer.isHuman():
						popupInfo = CyPopupInfo()
						popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
						popupInfo.setText(CyTranslator().getText("TXT_KEY_POPUP_CONTROL_MERCURIANS",()))
						popupInfo.setData1(player)
						popupInfo.setData2(iMercurianPlayer)
						popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_YES", ()), "")
						popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_NO", ()), "")
						popupInfo.setOnClickedPythonCallback("reassignPlayer")
						popupInfo.addPopup(player)
			else:
				if not game.isUnitClassMaxedOut(self.UnitClasses["Basium"], 0):
					if game.getNumCivActive(Civ["Mercurians"]) > 0:
						pMercurians  = getPlayer(game.getCivActivePlayer(Civ["Mercurians"], 0))
						pCapitalPlot = pMercurians.getCapitalCity()
						basiumUnit   = pMercurians.initUnit(iBasium, pCapitalPlot.getX(), pCapitalPlot.getY(), iNoAI, iNorth)
						basiumUnit.setExperienceTimes100(2500, -1)

		elif iBuildingType == Building["Tower of Elements"]:
			Promo	= self.Promotions["Effects"]
			Race	= self.Promotions["Race"]
			lList 	= ['UNIT_AIR_ELEMENTAL', 'UNIT_EARTH_ELEMENTAL', 'UNIT_FIRE_ELEMENTAL', 'UNIT_WATER_ELEMENTAL']
			iUnit 	= getInfoType(lList[randNum(len(lList), "Pick Elemental")-1])
			newUnit = pPlayer.initUnit(iUnit, iX, iY, iNoAI, iSouth)
			newUnit.setHasPromotion(Promo["Held"], True)
			CyInterface().addMessage(player,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_TOWER_OF_THE_ELEMENTS_SPAWN",()),'',1,gc.getUnitInfo(iUnit).getButton(),ColorTypes(8),iX,iY,True,True)
			apUnitList 	= PyPlayer(player).getUnitList()

			for pUnit in apUnitList:
				if pUnit.isHasPromotion(Race["Elemental"]):
					pUnit.setHasPromotion(Promo["Strong"], True)

		elif iBuildingType == Building["Tower of Necromancy"]:
			Promo	= self.Promotions["Effects"]
			Race	= self.Promotions["Race"]
#scions start
			Summon	= self.Units["Summons"]
			Unit	= self.Units["Scion"]
			Veil	= self.Units["Veil"]
#scions end
			apUnitList = PyPlayer(player).getUnitList()
			for pUnit in apUnitList:
#scions start - All Undead units from non-Scions civs get the benefits of the Tower of N.
				if pPlayer.getCivilizationType() != Civ["Scions"]:
#scions end
					if pUnit.isHasPromotion(Race["Undead"]):
						pUnit.setHasPromotion(Promo["Strong"], True)
#scions start - Only certain Scions units get the benefits.  (A balance measure)
				else:
					iUnitType = pUnit.getUnitType()
					if   iUnitType == Summon["Skeleton"]: 	   pUnit.setHasPromotion(Promo["Strong"], True)
					elif iUnitType == Unit["Bone Horde"]: 	   pUnit.setHasPromotion(Promo["Strong"], True)
					elif iUnitType == Veil["Diseased Corpse"]: pUnit.setHasPromotion(Promo["Strong"], True)

		elif iBuildingType == Building["Temple of the Gift"]:
			pCity.setNumRealBuilding( Building["Emperors Mark"], 0)
#scions end

		elif iBuildingType == Building["Grand Menagerie"]:
			if pPlayer.isHuman():
				if not game.getWBMapScript():
					t = "TROPHY_FEAT_GRAND_MENAGERIE"
					if not game.isHasTrophy(t):
						game.changeTrophyValue(t, 1)

#Added in Frozen: TC01
		elif iBuildingType == Building["Swamp of Souls"]:
			scriptData = {
				"iNumTarSpawns" : 0,
			}
			pCity.setScriptData(pickle.dumps(scriptData))
#End of Frozen

		elif iBuildingType == Building["Fisher Guild"]:
			pCity.setHasCorporation( self.Corporations["Fishermans"], True, True, False)
		elif iBuildingType == Building["Masquerade Gypsy Camp"]:
			pCity.setHasCorporation( self.Corporations["Masquerade"], True, True, False)
		elif iBuildingType == Building["Fabricaforma"]:
			pCity.setHasCorporation( self.Corporations["Farbricaforma"], True, True, False)
		elif iBuildingType == Building["Farmers Guild"]:
			pCity.setHasCorporation( self.Corporations["Farmers"], True, True, False)
		elif iBuildingType == Building["Stonefire Guild"]:
			pCity.setHasCorporation( self.Corporations["Stonefire"], True, True, False)

		CvAdvisorUtils.buildingBuiltFeats(pCity, iBuildingType)

		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onBuildingBuilt']:
			module.onBuildingBuilt(self, argsList)

		## Modular Python End
		## *******************
		
		if (not self.__LOG_BUILDING):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s'
			%(PyInfo.BuildingInfo(iBuildingType).getDescription(), pCity.getOwner(), gc.getPlayer(pCity.getOwner()).getCivilizationDescription(0)))

	def onProjectBuilt(self, argsList):
		'Project Completed'
		pCity, iProjectType = argsList
		
		gc = CyGlobalContext() 
		cf				= self.cf
		getInfoType		= gc.getInfoTypeForString
		getPlayer 		= gc.getPlayer
		game 			= CyGame()
		map 			= CyMap()
		iNumPlots 		= map.numPlots()
		plotByIndex	 	= map.plotByIndex
		iPlayer 		= pCity.getOwner()
		pPlayer 		= getPlayer(iPlayer)
		iMaxPlayer 		= gc.getMAX_PLAYERS()
		Civ				= self.Civilizations
		eCiv			= pPlayer.getCivilizationType()
		isOption 		= game.isOption
		iTeam 			= pPlayer.getTeam()
		iOrcPlayer 		= gc.getORC_PLAYER()
		iAnimalPlayer 	= gc.getANIMAL_PLAYER()
		getTeam	 		= gc.getTeam
		iDemonPlayer 	= gc.getDEMON_PLAYER()
		randNum 		= game.getSorenRandNum
		Religion		= self.Religions
		Terrain 		= self.Terrain
		Unit			= self.Units["Generic"]
		Animal 			= self.Units["Animal"]
		Frozen 			= self.Units["Frozen"]
		Promo			= self.Promotions["Effects"]
		Generic 		= self.Promotions["Generic"]
		Race  			= self.Promotions["Race"]
		UnitClass		= self.UnitClasses
		Project			= self.Projects
		initUnit		= pPlayer.initUnit
		iNoPlayer 		= PlayerTypes.NO_PLAYER
		iNoAI			= UnitAITypes.NO_UNITAI
		iNorth 			= DirectionTypes.DIRECTION_NORTH
		iSouth 			= DirectionTypes.DIRECTION_SOUTH
		iX = pCity.getX(); iY = pCity.getY()
		
		if ((not game.isNetworkMultiPlayer()) and (iPlayer == game.getActivePlayer())):
			popupInfo = CyPopupInfo()
			popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
			popupInfo.setData1(iProjectType)
			popupInfo.setData2(pCity.getID())
			popupInfo.setData3(2)
			popupInfo.setText(u"showWonderMovie")
			popupInfo.addPopup(iPlayer)

		if iProjectType == Project["Bane Divine"]:
			iCombatDisciple = self.UnitCombats["Disciple"]
			for iLoopPlayer in xrange(iMaxPlayer):
				pLoopPlayer = getPlayer(iLoopPlayer)
				if pLoopPlayer.isAlive() :
					apUnitList = PyPlayer(iLoopPlayer).getUnitList()
					for pUnit in apUnitList:
						if pUnit.getUnitCombatType() == iCombatDisciple:
							pUnit.kill(False, iPlayer)

		elif iProjectType == Project["Genesis"]:
			cf.genesis(iPlayer)

		elif iProjectType == Project["Glory Everlasting"]:
			iEvil = self.Alignments["Evil"]
			for iLoopPlayer in xrange(iMaxPlayer):
				pLoopPlayer = getPlayer(iLoopPlayer)
				player = PyPlayer(iLoopPlayer)
				if pLoopPlayer.isAlive():
					apUnitList = player.getUnitList()
					if pLoopPlayer.getTeam() != iTeam:
						if (pLoopPlayer.isBarbarian() or pLoopPlayer.getAlignment() == iEvil):
							for pUnit in apUnitList:
								hasPromo = pUnit.isHasPromotion
								if (hasPromo(Race["Demon"]) or hasPromo(Race["Undead"])):
									pUnit.kill(False, iPlayer)

		elif iProjectType == Project["Rites of Oghma"]:
			i = 7
			iWorldSize 	= map.getWorldSize()
			WorldSize 	= self.WorldSizes
			if   iWorldSize == WorldSize["Duel"]: i = i - 3
			elif iWorldSize == WorldSize["Tiny"]: i = i - 2
			elif iWorldSize == WorldSize["Small"]:i = i - 1
			elif iWorldSize == WorldSize["Large"]:i = i + 1
			elif iWorldSize == WorldSize["Huge"]: i = i + 3
			addBonus('BONUS_MANA',i,'Art/Interface/Buttons/WorldBuilder/mana_button.dds')

		elif iProjectType == Project["Nature's Revolt"]:
			bPlayer 	  = getPlayer(iAnimalPlayer)
			py 			  = PyPlayer(iOrcPlayer)
			
			for pUnit in py.getUnitList():
				bValid = False
				iUC = pUnit.getUnitClassType()
				if   iUC == UnitClass["Worker"]: 	iNewUnit = Animal["Wild Man Troop"];	bValid = True
				elif iUC == UnitClass["Scout"]: 	iNewUnit = Animal["Dread Hamster"]; 	bValid = True
				elif iUC == UnitClass["Warrior"]: 	iNewUnit = Animal["Werewolf"]; 			bValid = True
				elif iUC == UnitClass["Hunter"]: 	iNewUnit = Animal["Allosaur"]; 			bValid = True
				elif iUC == UnitClass["Axeman"]: 	iNewUnit = Animal["Cave Bear"]; 		bValid = True
				elif iUC == UnitClass["Fawn"]: 		iNewUnit = Animal["Griffon"]; 			bValid = True
				elif iUC == UnitClass["Hill Giant"]:iNewUnit = Animal["Red Dragon"]; 		bValid = True
				elif iUC == UnitClass["Archer"]: 	iNewUnit = Animal["Kong"]; 				bValid = True
				elif iUC == UnitClass["Cyklop"]: 	iNewUnit = Animal["Spider Swarm"]; 		bValid = True
				elif iUC == UnitClass["Minotaur"]: 	iNewUnit = Animal["Dire Boar"]; 		bValid = True
				elif iUC == UnitClass["Horseman"]: 	iNewUnit = Animal["Wild Hunt"]; 		bValid = True
				elif iUC == UnitClass["Frostling"]:	iNewUnit = Animal["White Drake Flight"];bValid = True
				
				if bValid:
					initAnimal = bPlayer.initUnit
					iX = pUnit.getX(); iY = pUnit.getY()
					newUnit = initAnimal(iNewUnit, iX, iY, iNoAI, iNorth)
					newUnit = initAnimal(iNewUnit, iX, iY, iNoAI, iNorth)
					newUnit = initAnimal(iNewUnit, iX, iY, iNoAI, iNorth)
					pUnit.kill(True, iNoPlayer)
				
			for iLoopPlayer in xrange(iMaxPlayer):
				pLoopPlayer = getPlayer(iLoopPlayer)
				if pLoopPlayer.isAlive():
					py = PyPlayer(iLoopPlayer)
					for pUnit in py.getUnitList():
						if pUnit.isAnimal():
							pUnit.setHasPromotion( Promo["Heroic Defense I"], True)
							pUnit.setHasPromotion( Promo["Heroic Defense II"], True)
							pUnit.setHasPromotion( Promo["Heroic Strength I"], True)
							pUnit.setHasPromotion( Promo["Heroic Strength II"], True)

		elif iProjectType == Project["Blood of the Phoenix"]:
			py = PyPlayer(iPlayer)
			apUnitList = py.getUnitList()
			for pUnit in apUnitList:
				if pUnit.isAlive() and pUnit.getUnitCombatType() != -1:
					pUnit.setHasPromotion( Promo["Immortal"], True)

		elif iProjectType == Project["Purge the Unfaithful"]:
			getBuildingInfo = gc.getBuildingInfo
			iNumReligions 	= gc.getNumReligionInfos()
			iNumBuildings 	= gc.getNumBuildingInfos()
			pPlayer.getStateReligion()
			for pyCity in PyPlayer(iPlayer).getCityList():
				pCity2 = pyCity.GetCy()
				iRnd = randNum(2, "Purge the Unfaithful Revolt")
				if iRel == Religion["Order"]: 
					iRnd = iRnd - 1
				for iTarget in xrange(iNumReligions):
					if (StateBelief != iTarget and pCity2.isHasReligion(iTarget) and pCity2.isHolyCityByType(iTarget) == False):
						pCity2.setHasReligion(iTarget, False, True, True)
						iRnd = iRnd + 1
						for i in xrange(iNumBuildings):
							if getBuildingInfo(i).getPrereqReligion() == iTarget:
								pCity2.setNumRealBuilding(i, 0)
				if iRnd > 0:
					pCity2.setOccupationTimer(iRnd)

		elif iProjectType == Project["Birthright Regained"]:
			pPlayer.setFeatAccomplished(FeatTypes.FEAT_GLOBAL_SPELL, False)

		elif iProjectType == Project["Prepare Expedition"]:
			pPlayer.setFeatAccomplished(FeatTypes.FEAT_EXPEDITION_READY, True)

		elif iProjectType == Project["Samhain"]:
			Frostling = self.Units["Frostling"]
			iCount = game.countCivPlayersAlive() + int(game.getHandicapType()) - 5
			for i in xrange(iCount):
				addUnit( Frostling["Frostling"], iOrcPlayer)
				addUnit( Frostling["Frostling"], iOrcPlayer)
				addUnit( Frostling["Archer"], iOrcPlayer)
				addUnit( Frostling["Wolf Rider"], iOrcPlayer)
			addUnit(self.Heroes["Mokka"], iOrcPlayer)

		elif iProjectType == Project["The White Hand"]:
			Unit = self.Units["White Hand"]
			pCity.setHasReligion( Religion["White Hand"], True, True, True)
			newUnit1 = initUnit( Unit["Priest"], iX, iY, iNoAI, iSouth)
			newUnit1.setName("Dumannios")
			newUnit1.changeStrBoost(1)
			newUnit2 = initUnit( Unit["Priest"], iX, iY, iNoAI, iSouth)
			newUnit2.setName("Riuros")
			newUnit2.changeStrBoost(1)
			newUnit3 = initUnit( Unit["Priest"], iX, iY, iNoAI, iSouth)
			newUnit3.setName("Anagantios")
			newUnit3.changeStrBoost(1)

		elif iProjectType == Project["The Deepening"]:
			iTimer = 40 + (game.getGameSpeedType() * 20)
			for i in xrange(iNumPlots):
				pPlot = plotByIndex(i)
				bValid = False
				if pPlot.isWater() == False:
					if randNum(100, "The Deepening") < 25:
						iTerrain = pPlot.getTerrainType()
						if iTerrain == Terrain["Snow"]:
							bValid = True
# FF: Changed by Jean Elcard 14/01/2009 (speed tweak)
						elif iTerrain == Terrain["Tundra"]:
							pPlot.setTempTerrainTypeFM( Terrain["Snow"], randNum(iTimer, "Deepening Terrain Timer (Tundra)") + 10, False, False)
							bValid = True
						elif iTerrain == Terrain["Grass"]:
							pPlot.setTempTerrainTypeFM( Terrain["Tundra"], randNum(iTimer, "Deepening Terrain Timer (Grass)") + 10, False, False)
							bValid = True
						elif iTerrain == Terrain["Plains"]:
							pPlot.setTempTerrainTypeFM( Terrain["Tundra"], randNum(iTimer, "Deepening Terrain Timer (Plains)") + 10, False, False)
							bValid = True
						elif iTerrain == Terrain["Desert"]:
							pPlot.setTempTerrainTypeFM( Terrain["Plains"], randNum(iTimer, "Deepening Terrain Timer (Desert)") + 10, False, False)
# FF: End Change
						if bValid:
							if randNum(750, "The Deepening, Blizzard Creation") < 10:
								pPlot.setFeatureType(Terrain["Blizzard"], -1)

		elif iProjectType == Project["Stir From Slumber"]:
			initUnit(self.Heroes["Drifa"], iX, iY, iNoAI, iSouth)
			if pPlayer.getLeaderType() == self.Leaders["Raitlor"]:
				pPlayer.setHasTrait(self.Traits["Ice Touched"], True)

		elif iProjectType == Project["The Draw"]:
			pPlayer.changeNoDiplomacyWithEnemies(1)
			iTeam = pPlayer.getTeam()
			eTeam = getTeam(iTeam)
			for iLoopTeam in xrange(gc.getMAX_TEAMS()):
				if iLoopTeam != iTeam:
					if iLoopTeam != getPlayer(iOrcPlayer).getTeam() and iLoopTeam != getPlayer(iAnimalPlayer).getTeam() and iLoopTeam != getPlayer(iDemonPlayer).getTeam():
						eLoopTeam = getTeam(iLoopTeam)
						if eLoopTeam.isAlive():
							if not eLoopTeam.isAVassal():
								eTeam.declareWar(iLoopTeam, False, WarPlanTypes.WARPLAN_LIMITED)
			py = PyPlayer(iPlayer)
			for pUnit in py.getUnitList():
				iDmg = pUnit.getDamage() * 2
				if iDmg > 99:
					iDmg = 99
				if iDmg < 50:
					iDmg = 50
				pUnit.setDamage(iDmg, iPlayer)
			for pyCity in PyPlayer(iPlayer).getCityList():
				pCity = pyCity.GetCy()
				iPop = int(pCity.getPopulation() / 2)
				if iPop < 1:
					iPop = 1
				pCity.setPopulation(iPop)

		elif iProjectType == Project["Ascension"]:
#	Changed in Frozen: TC01
			Hero = self.Heroes
			if eCiv == Civ["Illians"]:
				for iPlot in xrange(iNumPlots):
					pAuricPlot = plotByIndex(iPlot)
					for iUnit in xrange(pAuricPlot.getNumUnits()):
						pAuric = pAuricPlot.getUnit(iUnit)
						iAuric = 0
						if pAuric.getUnitType() == Hero["Auric"]:
							newUnit = initUnit( Hero["Auric Ascended"], pAuric.getX(), pAuric.getY(), iNoAI, iSouth)
							newUnit.convert(pAuric)	
							iAuric = 1
							break
						elif pAuric.getUnitType() == Hero["Auric Winter"]:
							newUnit = initUnit( Hero["Auric Ascended"], pAuric.getX(), pAuric.getY(), iNoAI, iSouth)
							newUnit.convert(pAuric)
							iAuric = 1
							break
			if eCiv == Civ["Frozen"]:
				initUnit( Hero["Taranis Ascended"], iX, iY, iNoAI, iSouth)
				for iPlot in xrange(iNumPlots):
					pTaranisPlot = plotByIndex(iPlot)
					for iUnit in xrange(pTaranisPlot.getNumUnits()):
						pTaranis = pTaranisPlot.getUnit(iUnit)
						if pTaranis.getUnitType() == Hero["Taranis"]:
							pTaranis.kill(False, -1)
#End of Frozen

			for iLoopPlayer in xrange(iMaxPlayer):
				pLoopPlayer = getPlayer(iLoopPlayer)
				if pLoopPlayer.isAlive():
					if not pLoopPlayer.isBarbarian():
						if pLoopPlayer.getTeam() != pPlayer.getTeam():
							if pLoopPlayer.getStateReligion() == Religion["White Hand"]:
								getTeam(pLoopPlayer.getTeam()).setVassal(pPlayer.getTeam(), True, False)
			if pPlayer.isHuman():
				t = "TROPHY_FEAT_ASCENSION"
				if not game.isHasTrophy(t):
					game.changeTrophyValue(t, 1)
			if not game.getWBMapScript():
				iBestPlayer = -1
				iBestValue = 0
				getRank = game.getPlayerRank
				for iLoopPlayer in xrange(iMaxPlayer):
					pLoopPlayer = getPlayer(iLoopPlayer)
					if pLoopPlayer.isAlive():
						if not pLoopPlayer.isBarbarian():
							if pLoopPlayer.getTeam() != pPlayer.getTeam():
								if not pLoopPlayer.getStateReligion() == Religion["White Hand"]:
									iValue = randNum(500, "Ascension")
									if pLoopPlayer.isHuman():
										iValue += 2000
									iValue += (20 - getRank(iLoopPlayer)) * 50
									if iValue > iBestValue:
										iBestValue = iValue
										iBestPlayer = iLoopPlayer
				if iBestPlayer != -1:
					Equipment = self.Promotions["Equipment"]
					Item = self.Units["Equipment"]
					pBestPlayer = getPlayer(iBestPlayer)
					pBestCity = pBestPlayer.getCapitalCity()
					if pBestPlayer.isHuman():
						getPlayer(iBestPlayer).initTriggeredData(self.EventTriggers["Godslayer"], True, -1, pBestCity.getX(), pBestCity.getY(), iBestPlayer, -1, -1, -1, -1, -1)
					else:
						containerUnit = -1
						pPlot = pBestCity.plot()
						getUnit = pPlot.getUnit
						for i in xrange(pPlot.getNumUnits()):
							if getUnit(i).getUnitType() == Item["Container"]:
								containerUnit = getUnit(i)
						if containerUnit == -1:
							containerUnit = getPlayer(iOrcPlayer).initUnit(Item["Container"], iX, iY, iNoAI, iSouth)
						containerUnit.setHasPromotion( Equipment["Godslayer"], True)

#	Added in Frozen: TC01
#	Will spawn the Frozen when completed.
		elif iProjectType == Project["Liberation"]:
			map 	   = CyMap()
			iNumGolems = randNum(2, "Liberation of Golems") + 3	
			for i in xrange(iNumGolems):
				initUnit(Frozen["Ice Golem"], iX, iY, iNoAI, iSouth)
		
			iFrozenPlayer = getOpenPlayer()
			pBestPlot = -1
			iBestPlot = -1
			for i in xrange(iNumPlots):
				pPlot = plotByIndex(i)
				iPlot = -1
				if pPlot.isWater() == False:
					if pPlot.getNumUnits() == 0:
						if pPlot.isCity() == False:
							if pPlot.isImpassable() == False:
								iPlot = randNum(500, "Place Taranis")
								iPlot = iPlot + (pPlot.area().getNumTiles() * 2)
								iPlot = iPlot + (pPlot.area().getNumUnownedTiles() * 10)
								if pPlot.isOwned() == False:
									iPlot = iPlot + 500
								if pPlot.getOwner() == iPlayer:
									iPlot = iPlot + 200
				if iPlot > iBestPlot:
					iBestPlot = iPlot
					pBestPlot = pPlot
					
			if (iFrozenPlayer != -1 and pBestPlot != -1):
				CyGame().addPlayerAdvanced(iFrozenPlayer, -1, self.Leaders["Taranis"], Civ["Frozen"])
				iFounderTeam 	= getPlayer(iPlayer).getTeam()
				eFounderTeam 	= getTeam(gc.getPlayer(iPlayer).getTeam())
				iFrozenTeam 	= getPlayer(iFrozenPlayer).getTeam()
				eFrozenTeam 	= getTeam(iFrozenTeam)
				for iTech in xrange(gc.getNumTechInfos()):
					if eFounderTeam.isHasTech(iTech):
						eFrozenTeam.setHasTech(iTech, True, iFrozenPlayer, True, False)
				eFounderTeam.signOpenBorders(iFounderTeam)
				eFrozenTeam.signOpenBorders(iFrozenTeam)
				pFrozenPlayer = gc.getPlayer(iFrozenPlayer)
				pFrozenPlayer.AI_changeAttitudeExtra(iPlayer,4)
				init = pFrozenPlayer.initUnit
				iX = pBestPlot.getX(); iY = pBestPlot.getY();
				newUnit1  = init( Frozen["Ice Golem"], iX, iY, iNoAI, iNorth)
				newUnit1.setHasPromotion( Generic["Mobility I"], True)
				newUnit2  = init( Frozen["Ice Golem"], iX, iY, iNoAI, iNorth)
				newUnit2.setHasPromotion( Generic["Mobility I"], True)
				newUnit3  = init( Unit["Champion"], iX, iY, iNoAI, iNorth)
				newUnit3.setHasPromotion( Promo["Iron Weapons"], True)
				newUnit3.setHasPromotion( Generic["Mobility I"], True)
				newUnit4  = init( Unit["Champion"], iX, iY, iNoAI, iNorth)
				newUnit4.setHasPromotion( Promo["Iron Weapons"], True)
				newUnit4.setHasPromotion( Generic["Mobility I"], True)
				newUnit5  = init( Unit["Worker"], iX, iY, iNoAI, iNorth)
				newUnit5.setHasPromotion( Race["Demon"], True)
				newUnit6  = init( Unit["Adept"], iX, iY, iNoAI, iNorth)
				newUnit6.setHasPromotion( Generic["Mobility I"], True)
				newUnit7  = init( Frozen["Frozen Souls"], iX, iY, iNoAI, iNorth)
				newUnit8  = init( Frozen["Frozen Souls"], iX, iY, iNoAI, iNorth)
				newUnit9  = init( Frozen["Frozen Souls"], iX, iY, iNoAI, iNorth)
				newUnit10 = init( Unit["Settler"], iX, iY, iNoAI, iNorth)
				newUnit10.setHasPromotion( Promo["Starting Settler"], True)
				newUnit10.setHasPromotion( Race["Demon"], True)
				newUnit11 = init( Unit["Settler"], iX, iY, iNoAI, iNorth)
				newUnit11.setHasPromotion( Promo["Starting Settler"], True)
				newUnit11.setHasPromotion( Race["Demon"], True)
				if gc.getPlayer(iPlayer).isHuman():
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
					popupInfo.setText(CyTranslator().getText("TXT_KEY_POPUP_CONTROL_FROZEN",()))
					popupInfo.setData1(iPlayer)
					popupInfo.setData2(iFrozenPlayer)
					popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_YES", ()), "")
					popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_NO", ()), "")
					popupInfo.setOnClickedPythonCallback("reassignPlayer")
					popupInfo.addPopup(iPlayer)
#End of Frozen

		elif iProjectType == Project["Pax Diabolis"]:
			pCity.setNumRealBuilding( self.Buildings["Pax Diabolis"], 1)
			
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onProjectBuilt']:
			module.onProjectBuilt(self, argsList)

		## Modular Python End
		## *******************


	def onSelectionGroupPushMission(self, argsList):
		'selection group mission'
		eOwner = argsList[0]
		eMission = argsList[1]
		iNumUnits = argsList[2]
		listUnitIds = argsList[3]

		if (not self.__LOG_PUSH_MISSION):
			return
		if pHeadUnit:
			CvUtil.pyPrint("Selection Group pushed mission %d" %(eMission))

	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit,pOldPlot = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		
		if (not self.__LOG_MOVEMENT):
			return
		if player and unitInfo:
			CvUtil.pyPrint('Player %d Civilization %s unit %s is moving to %d, %d'
				%(player.getID(), player.getCivilizationName(), unitInfo.getDescription(),
				pUnit.getX(), pUnit.getY()))

	def onUnitSetXY(self, argsList):
		'units xy coords set manually'
		pPlot,pUnit = argsList
		player = PyPlayer(pUnit.getOwner())
		unitInfo = PyInfo.UnitInfo(pUnit.getUnitType())
		if (not self.__LOG_MOVEMENT):
			return

	def onUnitCreated(self, argsList):
		'Unit Completed'
		pUnit = argsList[0]
		self.verifyLoaded()
		gc 					= CyGlobalContext() 
		getInfoType			= gc.getInfoTypeForString
		cf					= self.cf
		game 				= CyGame()
		player 				= PyPlayer(pUnit.getOwner())
		getPlayer 			= gc.getPlayer
		pPlayer 			= getPlayer(pUnit.getOwner())
		Civ					= self.Civilizations
		Promo				= self.Promotions["Effects"]
		Generic				= self.Promotions["Generic"]
		Race				= self.Promotions["Race"]
		Equipment			= self.Promotions["Equipment"]
		UnitCombat			= self.UnitCombats
		Tech				= self.Techs
		Mana				= self.Mana
		Mechanos			= self.Units["Mechanos"]
		Machinarum			= self.Units["Ordo Machinarum"]
		ScorpClan			= self.Units["Scorpion Clan"]
		Summon 				= self.Units["Summons"]
		Scions 				= self.Units["Scions"]
		Veil 				= self.Units["Veil"]
		setPromo 			= pUnit.setHasPromotion
		hasTrait 			= pPlayer.hasTrait
		Trait				= self.Traits
		iUnitType 			= pUnit.getUnitType()
		initUnit 			= pPlayer.initUnit
		getNumAvailBonuses 	= pPlayer.getNumAvailableBonuses
		getTeam 			= gc.getTeam
		hasPromo 			= pUnit.isHasPromotion
		randNum 			= game.getSorenRandNum
		iNoAI				= UnitAITypes.NO_UNITAI
		iSouth 				= DirectionTypes.DIRECTION_SOUTH
		pPlot 				= pUnit.plot()
		iX = pPlot.getX(); iY = pPlot.getY()
		
		if game.getAIAutoPlay() == 0:
			if (pUnit.isImage()):
				activePlayer = getPlayer(game.getActivePlayer())
				sPlayerName = player.getName()
				sUnitName = pUnit.getName()
				sQuote = pUnit.getQuote()
				if getTeam(pUnit.getTeam()).isHasMet(activePlayer.getTeam()):
					sPopupText = CyTranslator().getText('TXT_KEY_MISC_UNIT_POPUP',(sPlayerName, sUnitName, sQuote))
				else:
					sPopupText = CyTranslator().getText('TXT_KEY_MISC_UNIT_POPUP_UNKNOWN',(sUnitName, sQuote))
				addPopup(sPopupText, str(pUnit.getImage()))
		
		if hasTrait(Trait["Defender"]):
			if pUnit.getUnitCombatType() == UnitCombat["Worker"]:
				setPromo(Generic["Hardy I"], True)
				
		if hasTrait(Trait["Instructor"]):
			if pUnit.getUnitCombatType() != UnitCombat["Siege"] and pUnit.getUnitCombatType() != UnitCombat["Naval"]:
				pUnit.changeFreePromotionPick(1)
				
		if hasTrait(Trait["Spiderkin"]):
			pNest = pPlayer.getCapitalCity()
			iNestPop = pNest.getPopulation()
			if iNestPop >= 15: setPromo(Promo["Spiderkin"], True)
						
		if iUnitType == Mechanos["Adeptus"]: 	pUnit.setReligion(self.Religions["Machinarum"])
		if iUnitType == Mechanos["Techpriest"]:	pUnit.setReligion(self.Religions["Machinarum"])
		if iUnitType == Machinarum["Adeptus"]: 	pUnit.setReligion(self.Religions["Machinarum"])

		if iUnitType == ScorpClan["Whelp"]:
			if pPlot.getNumUnits() < 3:
				newUnit = initUnit( ScorpClan["Archer"], iX, iY, iNoAI, iSouth)
			else:
				iRiderWeight = 15
				iChariotWeight = 5
				if   pPlayer.isHasTech( Tech["Construction"]): iChariotWeight += 25
				elif pPlayer.isHasTech( Tech["Horseback Riding"]): iRiderWeight += 25
				iRnd = randNum(100, "Goblin Whelp Upgrade")
				if iRnd < iChariotWeight:
					newUnit = initUnit( ScorpClan["Chariot"], iX, iY, iNoAI, iSouth)
				elif iRnd < (iChariotWeight+iRiderWeight):
					newUnit = initUnit(ScorpClan["Wolf Rider"], iX, iY, iNoAI, iSouth)
				else:
					newUnit = initUnit(ScorpClan["Goblin"], iX, iY, iNoAI, iSouth)
			newUnit.convert(pUnit)
			pUnit = newUnit

		if pUnit.getUnitCombatType() == UnitCombat["Adept"]:
			iNum = getNumAvailBonuses( Mana["Air"])
			eCiv = pPlayer.getCivilizationType()
			if iNum > 1:
				setPromo(Generic["Air I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Air II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Air III"], True)
			iNum = getNumAvailBonuses( Mana["Body"])
			if eCiv != Civ["Scions"]:
				if iNum > 1:
					setPromo(Generic["Body I"], True)
					if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
						setPromo(Generic["Body II"], True)
						if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
							setPromo(Generic["Body III"], True)
			if eCiv == Civ["Scions"]:
				if iNum > 1:
					setPromo(Generic["Corpus I"], True)
					if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
						setPromo(Generic["Corpus II"], True)
						if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
							setPromo(Generic["Corpus III"], True)
			iNum = getNumAvailBonuses( Mana["Chaos"])
			if iNum > 1:
				setPromo(Generic["Chaos I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Chaos II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Chaos III"], True)
			iNum = getNumAvailBonuses( Mana["Death"])
			if iNum > 1:
				setPromo(Generic["Death I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Death II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Death III"], True)
			iNum = getNumAvailBonuses( Mana["Earth"])
			if iNum > 1:
				setPromo(Generic["Earth I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Earth II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Earth III"], True)
			iNum = getNumAvailBonuses( Mana["Enchantment"])
			if iNum > 1:
				setPromo(Generic["Enchantment I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Enchantment II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Enchantment III"], True)
			iNum = getNumAvailBonuses( Mana["Entropy"])
			if iNum > 1:
				setPromo(Generic["Entropy I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Entropy II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Entropy III"], True)
			iNum = getNumAvailBonuses( Mana["Fire"])
			if iNum > 1:
				setPromo(Generic["Fire I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Fire II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Fire III"], True)
			iNum = getNumAvailBonuses( Mana["Ice"])
			if iNum > 1:
				setPromo(Generic["Ice I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Ice II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Ice III"], True)
			iNum = getNumAvailBonuses( Mana["Law"])
			if iNum > 1:
				setPromo(Generic["Law I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Law II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Law III"], True)
			iNum = getNumAvailBonuses( Mana["Life"])
			if iNum > 1:
				setPromo(Generic["Life I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Life II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Life III"], True)
			iNum = getNumAvailBonuses( Mana["Metamagic"])
			if iNum > 1:
				setPromo(Generic["Metamagic I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Metamagic II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Metamagic III"], True)
			iNum = getNumAvailBonuses( Mana["Mind"])
			if iNum > 1:
				setPromo(Generic["Mind I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Mind II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Mind III"], True)
			iNum = getNumAvailBonuses( Mana["Nature"])
			if iNum > 1:
				setPromo(Generic["Nature I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Nature II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Nature III"], True)
			iNum = getNumAvailBonuses( Mana["Shadow"])
			if iNum > 1:
				setPromo(Generic["Shadow I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Shadow II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Shadow III"], True)
			iNum = getNumAvailBonuses( Mana["Spirit"])
			if iNum > 1:
				setPromo(Generic["Spirit I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Spirit II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Spirit III"], True)
			iNum = getNumAvailBonuses( Mana["Sun"])
			if iNum > 1:
				setPromo(Generic["Sun I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Sun II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Sun III"], True)
			iNum = getNumAvailBonuses( Mana["Water"])
			if iNum > 1:
				setPromo(Generic["Water I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Water II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Water III"], True)
			iNum = getNumAvailBonuses( Mana["Creation"])
			if iNum > 1:
				setPromo(Generic["Creation I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Creation II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Creation III"], True)
			iNum = getNumAvailBonuses( Mana["Force"])
			if iNum > 1:
				setPromo(Generic["Force I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Force II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Force III"], True)
			iNum = getNumAvailBonuses( Mana["Dimensional"])
			if iNum > 1:
				setPromo(Generic["Dimensional I"], True)
				if (iNum > 2 and hasPromo( Promo["Channeling II"] )):
					setPromo(Generic["Dimensional II"], True)
					if (iNum > 3 and hasPromo( Promo["Channeling III"] )):
						setPromo(Generic["Dimensional III"], True)

		if hasPromo(Race["Elemental"]):
			if pPlayer.getNumBuilding(self.Buildings["Tower of Elements"]) > 0:
				setPromo(Promo["Strong"], True)

		if hasPromo(Race["Undead"]):
			if pPlayer.getNumBuilding(self.Buildings["Tower of Necromancy"]) > 0:
#scions start - Same purpose and rationale as other Tower of N code above.
				if pPlayer.getCivilizationType() != Civ["Scions"]:
#scions end - make sure next line is tabbed properly
					setPromo( Promo["Strong"], True)
#scions start
				if pPlayer.getCivilizationType() == Civ["Scions"]:
					if   iUnitType == Summon["Skeleton"]: 		setPromo( Promo["Strong"], True)
					elif iUnitType == Summon["Spectre"]: 		setPromo( Promo["Strong"], True)
					elif iUnitType == Summon["Wraith"]: 		setPromo( Promo["Strong"], True)
					elif iUnitType == Veil["Diseased Corpse"]: 	setPromo( Promo["Strong"], True)
					elif iUnitType == Scions["Bone Horde"]: 	setPromo( Promo["Strong"], True)

#scions end
		if pUnit.getUnitClassType() == self.UnitClasses["Kahd"]:
			if pPlayer.hasTrait(Trait["Ambitious"]):
				setPromo( Generic["Mind I"], True)
				setPromo( Generic["Mind II"], True)
				setPromo( Generic["Mind III"], True)
				setPromo( Promo["Ambition"], True)
				setPromo( Equipment["Compelling Jewel"], True)
				setPromo( Promo["Hero"], True)

			else:
				setPromo( Generic["Metamagic I"], True)
				setPromo( Generic["Metamagic II"], True)
				setPromo( Generic["Metamagic III"], True)
				setPromo( Generic["Magic Resistance"], True)
				setPromo( Generic["Kahd Redeemed"], True)

				sphereList = ['PROMOTION_AIR','PROMOTION_EARTH','PROMOTION_ENCHANTMENT','PROMOTION_LAW','PROMOTION_LIFE','PROMOTION_NATURE','PROMOTION_SPIRIT','PROMOTION_SUN','PROMOTION_WATER' ]
				for sphere in sphereList:
					iRnd = randNum(100, "Kahd Free Promotions")
					if iRnd	< 30:
						setPromo(getInfoType(sphere + "1"), True)
					if iRnd	< 10:
						setPromo(getInfoType(sphere + "2"), True)
					if iRnd	< 3:
						setPromo(getInfoType(sphere + "3"), True)
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onUnitCreated']:
			module.onUnitCreated(self, argsList)

		## Modular Python End
		## *******************

		if (not self.__LOG_UNITBUILD):
			return

	def onUnitBuilt(self, argsList):
		'Unit Completed'
		pCity = argsList[0]
		pUnit = argsList[1]
		
		gc = CyGlobalContext() 
		cf			= self.cf
		getInfoType	= gc.getInfoTypeForString
		game 		= CyGame()
		map 		= CyMap()
		player 		= PyPlayer(pCity.getOwner())
		getPlayer	= gc.getPlayer
		pPlayer 	= getPlayer(pUnit.getOwner())
		iFreeProm 	= pUnit.getFreePromotionPick()
		getNumB 	= pCity.getNumBuilding
		getNumAvailBonuses = pPlayer.getNumAvailableBonuses
		iCombatType = pUnit.getUnitCombatType()
		setPromo 	= pUnit.setHasPromotion
		getNumAvailBonues = pPlayer.getNumAvailableBonuses
		randNum		= game.getSorenRandNum
		
		Promo	 	= self.Promotions["Effects"]
		Generic	 	= self.Promotions["Generic"]
		Race			= self.Promotions["Race"]
		Mana 		= self.Mana
		Building 		= self.Buildings
		Civ			= self.Civilizations
		Veil			= self.Units["Veil"]
		UnitCombat	= self.UnitCombats
		Hero			= self.Heroes
		Terrain		= self.Terrain
		
		if pUnit.getUnitType() == Veil["Beast of Agares"]:
			if pCity.getCivilizationType() != Civ["Infernal"]:
				iPop = pCity.getPopulation() - 4
				if iPop <= 1:
					iPop = 1
				pCity.setPopulation(iPop)
				pCity.setOccupationTimer(4)
		
		if getNumB( Building["Chancel of Guardians"]) > 0:
			if randNum(100, "Chancel of Guardians") < 20:
				setPromo( Generic["Defensive"], True)

		if getNumB( Building["Wizards Hall"]) > 0:
			if (iCombatType == UnitCombat["Melee"] or iCombatType == UnitCombat["Archer"] or iCombatType == UnitCombat["Mounted"] or iCombatType == UnitCombat["Recon"] or iCombatType == UnitCombat["Adept"] or iCombatType == UnitCombat["Disciple"]):
				lPromoList = []
				iAirNum = getNumAvailBonues( Mana["Air"])
				if iAirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_AIR1']
				iBodyNum = getNumAvailBonues( Mana["Body"])
				if iBodyNum > 0:
					lPromoList = lPromoList + ['PROMOTION_BODY1']
				iChaosNum = getNumAvailBonues( Mana["Chaos"])
				if iChaosNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CHAOS1']
				iDeathNum = getNumAvailBonues( Mana["Death"])
				if iDeathNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DEATH1']
				iEarthNum = getNumAvailBonues( Mana["Earth"])
				if iEarthNum > 0:
					lPromoList = lPromoList + ['PROMOTION_EARTH1']
				iEncNum = getNumAvailBonues( Mana["Enchantment"])
				if iEncNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENCHANTMENT1']
				iEntNum = getNumAvailBonues( Mana["Entropy"])
				if iEntNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENTROPY1']
				iFirNum = getNumAvailBonues( Mana["Fire"])
				if iFirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FIRE1']
				iIceNum = getNumAvailBonues( Mana["Ice"])
				if iIceNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ICE1']
				iLawNum = getNumAvailBonues( Mana["Law"])
				if iLawNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LAW1']
				iLifNum = getNumAvailBonues( Mana["Life"])
				if iLifNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LIFE1']
				iMetNum = getNumAvailBonues( Mana["Metamagic"])
				if iMetNum > 0:
					lPromoList = lPromoList + ['PROMOTION_METAMAGIC1']
				iMinNum = getNumAvailBonues( Mana["Mind"])
				if iMinNum > 0:
					lPromoList = lPromoList + ['PROMOTION_MIND1']
				iNatNum = getNumAvailBonues( Mana["Nature"])
				if iNatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_NATURE1']
				iShaNum = getNumAvailBonues( Mana["Shadow"])
				if iShaNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SHADOW1']
				iSpiNum = getNumAvailBonues( Mana["Spirit"])
				if iSpiNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SPIRIT1']
				iSunNum = getNumAvailBonues( Mana["Sun"])
				if iSunNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SUN1']
				iWatNum = getNumAvailBonues( Mana["Water"])
				if iWatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_WATER1']
				iDimNum = getNumAvailBonues( Mana["Dimensional"])
				if iDimNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DIMENSIONAL1']
				iCreNum = getNumAvailBonues( Mana["Creation"])
				if iCreNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CREATION1']
				iForNum = getNumAvailBonues( Mana["Force"])
				if iForNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FORCE1']
				sPromo = lPromoList[randNum(len(lPromoList), "Pick Promotion")-1]
				if randNum(100, "Spell Gain Check") < 20 + ((iAirNum + iBodyNum + iChaosNum + iDeathNum + iEarthNum + iEncNum + iEntNum + iFirNum + iIceNum + iLawNum + iLifNum + iMetNum + iMinNum + iNatNum + iShaNum + iSpiNum + iSunNum + iWatNum + iDimNum + iCreNum + iForNum)*5):
					setPromo(getInfoType(sPromo), True)
		elif getNumB( Building["Wizards Hall"]) > 0 and getNumB(Building["Cave of Ancestors"]) > 0:
			if (iCombatType == UnitCombat["Melee"] or iCombatType == UnitCombat["Archer"] or iCombatType == UnitCombat["Mounted"] or iCombatType == UnitCombat["Recon"] or iCombatType == UnitCombat["Adept"] or iCombatType == UnitCombat["Disciple"]):
				lPromoList = []
				iAirNum = getNumAvailBonues( Mana["Air"])
				if iAirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_AIR1', 'PROMOTION_AIR2']
				iBodyNum = getNumAvailBonues( Mana["Body"])
				if iBodyNum > 0:
					lPromoList = lPromoList + ['PROMOTION_BODY1', 'PROMOTION_BODY2']
				iChaosNum = getNumAvailBonues( Mana["Chaos"])
				if iChaosNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CHAOS1', 'PROMOTION_CHAOS2']
				iDeathNum = getNumAvailBonues( Mana["Death"])
				if iDeathNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DEATH1', 'PROMOTION_DEATH2']
				iEarthNum = getNumAvailBonues( Mana["Earth"])
				if iEarthNum > 0:
					lPromoList = lPromoList + ['PROMOTION_EARTH1', 'PROMOTION_EARTH2']
				iEncNum = getNumAvailBonues( Mana["Enchantment"])
				if iEncNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENCHANTMENT1', 'PROMOTION_ENCHANTMENT2']
				iEntNum = getNumAvailBonues( Mana["Entropy"])
				if iEntNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENTROPY1', 'PROMOTION_ENTROPY2']
				iFirNum = getNumAvailBonues( Mana["Fire"])
				if iFirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FIRE1', 'PROMOTION_FIRE2']
				iIceNum = getNumAvailBonues( Mana["Ice"])
				if iIceNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ICE1', 'PROMOTION_ICE2']
				iLawNum = getNumAvailBonues( Mana["Law"])
				if iLawNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LAW1', 'PROMOTION_LAW2']
				iLifNum = getNumAvailBonues( Mana["Life"])
				if iLifNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LIFE1', 'PROMOTION_LIFE2']
				iMetNum = getNumAvailBonues( Mana["Metamagic"])
				if iMetNum > 0:
					lPromoList = lPromoList + ['PROMOTION_METAMAGIC1', 'PROMOTION_METAMAGIC2']
				iMinNum = getNumAvailBonues( Mana["Mind"])
				if iMinNum > 0:
					lPromoList = lPromoList + ['PROMOTION_MIND1', 'PROMOTION_MIND2']
				iNatNum = getNumAvailBonues( Mana["Nature"])
				if iNatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_NATURE1', 'PROMOTION_NATURE2']
				iShaNum = getNumAvailBonues( Mana["Shadow"])
				if iShaNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SHADOW1', 'PROMOTION_SHADOW2']
				iSpiNum = getNumAvailBonues( Mana["Spirit"])
				if iSpiNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SPIRIT1', 'PROMOTION_SPIRIT2']
				iSunNum = getNumAvailBonues( Mana["Sun"])
				if iSunNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SUN1', 'PROMOTION_SUN2']
				iWatNum = getNumAvailBonues( Mana["Water"])
				if iWatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_WATER1', 'PROMOTION_WATER2']
				iDimNum = getNumAvailBonues( Mana["Dimensional"])
				if iDimNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CREATION1', 'PROMOTION_CREATION2']
				iCreNum = getNumAvailBonues( Mana["Creation"])
				if iCreNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FORCE1', 'PROMOTION_FORCE2']
				iForNum = getNumAvailBonues( Mana["Force"])
				if iForNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DIMENSIONAL1', 'PROMOTION_DIMENSIONAL2']
				sPromo = lPromoList[randNum(len(lPromoList), "Pick Promotion")-1]
				if randNum(100, "Spell Gain Check") < 10 + ((iAirNum + iBodyNum + iChaosNum + iDeathNum + iEarthNum + iEncNum + iEntNum + iFirNum + iIceNum + iLawNum + iLifNum + iMetNum + iMinNum + iNatNum + iShaNum + iSpiNum + iSunNum + iWatNum + iDimNum + iCreNum + iForNum)*5):
					setPromo(getInfoType(sPromo), True)
		
		if getNumB( Building["School of Govannon"]) > 0:
			if (iCombatType == UnitCombat["Melee"] or iCombatType == UnitCombat["Archer"] or iCombatType == UnitCombat["Mounted"] or iCombatType == UnitCombat["Recon"] or iCombatType == UnitCombat["Adept"] or iCombatType == UnitCombat["Disciple"]):
				lPromoList = []
				iAirNum = getNumAvailBonues( Mana["Air"])
				if iAirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_AIR1', 'PROMOTION_AIR2', 'PROMOTION_AIR3']
				iBodyNum = getNumAvailBonues( Mana["Body"])
				if iBodyNum > 0:
					lPromoList = lPromoList + ['PROMOTION_BODY1', 'PROMOTION_BODY2', 'PROMOTION_BODY3']
				iChaosNum = getNumAvailBonues( Mana["Chaos"])
				if iChaosNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CHAOS1', 'PROMOTION_CHAOS2', 'PROMOTION_CHAOS3']
				iDeathNum = getNumAvailBonues( Mana["Death"])
				if iDeathNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DEATH1', 'PROMOTION_DEATH2', 'PROMOTION_DEATH3']
				iEarthNum = getNumAvailBonues( Mana["Earth"])
				if iEarthNum > 0:
					lPromoList = lPromoList + ['PROMOTION_EARTH1', 'PROMOTION_EARTH2', 'PROMOTION_EARTH3']
				iEncNum = getNumAvailBonues( Mana["Enchantment"])
				if iEncNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENCHANTMENT1', 'PROMOTION_ENCHANTMENT2', 'PROMOTION_ENCHANTMENT3']
				iEntNum = getNumAvailBonues( Mana["Entropy"])
				if iEntNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ENTROPY1', 'PROMOTION_ENTROPY2', 'PROMOTION_ENTROPY3']
				iFirNum = getNumAvailBonues( Mana["Fire"])
				if iFirNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FIRE1', 'PROMOTION_FIRE2', 'PROMOTION_FIRE3']
				iIceNum = getNumAvailBonues( Mana["Ice"])
				if iIceNum > 0:
					lPromoList = lPromoList + ['PROMOTION_ICE1', 'PROMOTION_ICE2', 'PROMOTION_ICE3']
				iLawNum = getNumAvailBonues( Mana["Law"])
				if iLawNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LAW1', 'PROMOTION_LAW2', 'PROMOTION_LAW3']
				iLifNum = getNumAvailBonues( Mana["Life"])
				if iLifNum > 0:
					lPromoList = lPromoList + ['PROMOTION_LIFE1', 'PROMOTION_LIFE2', 'PROMOTION_LIFE3']
				iMetNum = getNumAvailBonues( Mana["Metamagic"])
				if iMetNum > 0:
					lPromoList = lPromoList + ['PROMOTION_METAMAGIC1', 'PROMOTION_METAMAGIC2', 'PROMOTION_METAMAGIC3']
				iMinNum = getNumAvailBonues( Mana["Mind"])
				if iMinNum > 0:
					lPromoList = lPromoList + ['PROMOTION_MIND1', 'PROMOTION_MIND2', 'PROMOTION_MIND3']
				iNatNum = getNumAvailBonues( Mana["Nature"])
				if iNatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_NATURE1', 'PROMOTION_NATURE2', 'PROMOTION_NATURE3']
				iShaNum = getNumAvailBonues( Mana["Shadow"])
				if iShaNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SHADOW1', 'PROMOTION_SHADOW2', 'PROMOTION_SHADOW3']
				iSpiNum = getNumAvailBonues( Mana["Spirit"])
				if iSpiNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SPIRIT1', 'PROMOTION_SPIRIT2', 'PROMOTION_SPIRIT3']
				iSunNum = getNumAvailBonues( Mana["Sun"])
				if iSunNum > 0:
					lPromoList = lPromoList + ['PROMOTION_SUN1', 'PROMOTION_SUN2', 'PROMOTION_SUN3']
				iWatNum = getNumAvailBonues( Mana["Water"])
				if iWatNum > 0:
					lPromoList = lPromoList + ['PROMOTION_WATER1', 'PROMOTION_WATER2', 'PROMOTION_WATER3']
				iDimNum = getNumAvailBonues( Mana["Dimensional"])
				if iDimNum > 0:
					lPromoList = lPromoList + ['PROMOTION_CREATION1', 'PROMOTION_CREATION2', 'PROMOTION_CREATION3']
				iCreNum = getNumAvailBonues( Mana["Creation"])
				if iCreNum > 0:
					lPromoList = lPromoList + ['PROMOTION_FORCE1', 'PROMOTION_FORCE2', 'PROMOTION_FORCE3']
				iForNum = getNumAvailBonues( Mana["Force"])
				if iForNum > 0:
					lPromoList = lPromoList + ['PROMOTION_DIMENSIONAL1', 'PROMOTION_DIMENSIONAL2', 'PROMOTION_DIMENSIONAL3']
				sPromo = lPromoList[game.getSorenRandNum(len(lPromoList), "Pick Promotion")-1]
				if game.getSorenRandNum(100, "Spell Gain Check") < ((iAirNum + iBodyNum + iChaosNum + iDeathNum + iEarthNum + iEncNum + iEntNum + iFirNum + iIceNum + iLawNum + iLifNum + iMetNum + iMinNum + iNatNum + iShaNum + iSpiNum + iSunNum + iWatNum + iDimNum + iCreNum + iForNum)*3):
					setPromo(getInfoType(sPromo), True)

		if getNumB(Building["Cave of Ancestors"]) > 0 and iCombatType == UnitCombat["Adept"]:
			i = 0
			for iBonus in xrange(gc.getNumBonusInfos()):
				if gc.getBonusInfo(iBonus).getBonusClassType() == Mana["Mana Class"]:
					if pCity.hasBonus(iBonus):
						i = i + 1
			if i >= 1:
				pUnit.changeExperience(i, -1, False, False, False)

		if pUnit.isHasPromotion( Race["Golem"]):
			if getNumB( Building["Blasting Workshop"]) > 0:
				setPromo( Generic["Fire II"], True)
			if getNumB( Building["Pallens Engine"]) > 0:
				setPromo( Generic["Perfect Sight"], True)
			if getNumB( Building["Alduria Chamber"]) > 0:
				setPromo( Promo["Hidden"], True)

		if getNumB( Building["Asylum"]) > 0:
			if pUnit.isAlive():
				if isWorldUnitClass(pUnit.getUnitClassType()) == False:
					if game.getSorenRandNum(100, "Asylum Crazed Application") <= 10:
						setPromo( Generic["Crazed"], True)
						setPromo( Generic["Enraged"], True)

		if pUnit.getUnitType() == Hero["Acheron"]:
			pCity.setNumRealBuilding( Building["Dragons Hoard"], 1)
			iX = pCity.getX()
			iY = pCity.getY()
			setPromo( Promo["Acheron Leashed"], True)
			getPlot = map.plot
			for iiX,iiY in RANGE1:
				pPlot = getPlot(iX+iiX,iY+iiY)
				if (pPlot.getFeatureType() == Terrain["Forest"] or pPlot.getFeatureType() == Terrain["Jungle"]):
					pPlot.setFeatureType( Terrain["Flames"], 0)

		if pUnit.isHasPromotion( Race["Dwarven"]):
			if getNumB( Building["Brewery"]) > 0:
				pUnit.changeExperience(2, -1, False, False, False)

		if pUnit.isHasPromotion( Race["Demon"]):
			if getNumB( Building["Demons Altar"]) > 0:
				pUnit.changeExperience(2, -1, False, False, False)

		if pUnit.getFreePromotionPick() < iFreeProm:
			pUnit.changeFreePromotionPick(iFreeProm - pUnit.getFreePromotionPick())

		if pPlayer.getCivilizationType() == Civ["Austrin"]:
			if iCombatType == UnitCombat["Naval"]:
				setPromo( Generic["Eastwinds"], True)

		CvAdvisorUtils.unitBuiltFeats(pCity, pUnit)
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onUnitBuilt']:
			module.onUnitBuilt(self, argsList)

		## Modular Python End
		## *******************


		if (not self.__LOG_UNITBUILD):
			return
		CvUtil.pyPrint('%s was finished by Player %d Civilization %s'
			%(PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))

	def onUnitKilled(self, argsList):
		'Unit Killed'
		pUnit, iAttacker = argsList
		gc 			= CyGlobalContext()
		cf			= self.cf
		getInfoType	= gc.getInfoTypeForString
		map 		= CyMap()
		game 		= CyGame()
		iPlayer 	= pUnit.getOwner()
		player 		= PyPlayer(iPlayer)
		attacker 	= PyPlayer(iAttacker)
		pPlayer 	= gc.getPlayer(iPlayer)
		Civ			= self.Civilizations
		Rel			= self.Religions
		Promo		= self.Promotions["Effects"]
		Generic		= self.Promotions["Generic"]
		Hero		= self.Heroes
		iCiv 		= pPlayer.getCivilizationType()
		iRel		= pUnit.getReligion()
		iUnitType	= pUnit.getUnitType()
		iUnitCombat	= pUnit.getUnitCombatType()
		iOwner 		= pUnit.getOwner()
		pPlot 		= pUnit.plot()
		getPlot		= map.plot
		giftUnit	= cf.giftUnit
		getXP		= pUnit.getExperienceTimes100
		hasPromo	= pUnit.isHasPromotion
		Tech		= self.Techs
		Frozen		= self.Units["Frozen"]
		Infernal	= self.Units["Infernal"]
		Mercurian	= self.Units["Mercurian"]
		Building 	= self.Buildings
		UnitCombat 	= self.UnitCombats
		addMsg		= CyInterface().addMessage
		getText 	= CyTranslator().getText
		iNoAI		= UnitAITypes.NO_UNITAI
		iSouth		= DirectionTypes.DIRECTION_SOUTH
		randNum		= CyGame().getSorenRandNum
		
		if (pUnit.isAlive() and pUnit.isImmortal() == False):
			iX = pUnit.getX()
			iY = pUnit.getY()
			for iiX,iiY in RANGE1:
				pPlot2 = getPlot(iX+iiX,iY+iiY)
				if pPlot2.isCity():
					pCity = pPlot2.getPlotCity()
					if pCity.getNumBuilding( Building["Soul Forge"]) > 0:
						pCity.changeProduction(getXP()/100 + 10)
						addMsg(pCity.getOwner(),True,25,getText("TXT_KEY_MESSAGE_SOUL_FORGE",()),'AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/Buildings/Soulforge.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
			pPlot = getPlot(iX,iY)
			if pPlot.isCity():
				pCity = pPlot.getPlotCity()
				if pCity.getNumBuilding( Building["Mokkas Cauldron"]) > 0:
					if pCity.getOwner() == iOwner:
						iUnit = cf.getUnholyVersion(pUnit)
						if iUnit != -1:
							newUnit = pPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), iNoAI, iSouth)
							newUnit.setHasPromotion( Race["Demon"], True)
							newUnit.setDamage(50, PlayerTypes.NO_PLAYER)
							newUnit.finishMoves()
							szBuffer = gc.getUnitInfo(newUnit.getUnitType()).getDescription()
							addMsg(iOwner,True,25,getText("TXT_KEY_MESSAGE_MOKKAS_CAULDRON",((szBuffer, ))),'AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/Buildings/Mokkas Cauldron.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

			if game.countKnownTechNumTeams(Tech["Infernal Pact"]) > 0 and game.getNumCivActive(Civ["Infernal"]) > 0:
				if (iRel == Rel["Council of Esus"] or iRel == Rel["Ashen Veil"] or iRel == Rel["Octopus Overlords"] or hasPromo(Generic["Death I"]) or hasPromo(Generic["Entropy I"])):

	#Added to block a unit from creating both manes and angels, Frozen: TC01
					if (iRel == Rel["Empyrean"] or iRel == Rel["Order"] or iRel == Rel["Runes of Kilmorph"] or (iUnitCombat != UnitCombat["Animal"] and iCiv == Civ["Mercurians"])):
						iOption = randNum(100, "A unit is both good and evil!")
						if iOption <= 50:
							giftUnit( Infernal["Manes"], Civ["Infernal"], 0, pPlot, iOwner)
							giftUnit( Infernal["Manes"], Civ["Infernal"], 0, pPlot, iOwner)
						else:
							giftUnit( Mercurian["Angel"], Civ["Mercurians"], pUnit.getExperience(), pPlot, iOwner)
	#End of block
					else:
						giftUnit( Infernal["Manes"], Civ["Infernal"], 0, pPlot, iOwner)
						giftUnit( Infernal["Manes"], Civ["Infernal"], 0, pPlot, iOwner)

			if (iRel == Rel["Empyrean"] or iRel == Rel["Order"] or iRel == Rel["Runes of Kilmorph"] or (iUnitCombat != UnitCombat["Animal"] and iCiv == Civ["Mercurians"])):
				giftUnit( Mercurian["Angel"], Civ["Mercurians"], getXP(), pPlot, iOwner)

#Added in Frozen: TC01
#	If units with the Winterborn or Wintered promotion die, the Frozen get Frozen Souls.
			if (hasPromo( Promo["Winterborn"]) or hasPromo(Promo["Wintered"])): 
			#or hasPromo(getInfoType('PROMOTION_WINTERBORN_DOVIELLO'))):
				if not iOwner == Civ["Frozen"]:
					bValid = True
					if (iRel == Rel["Council of Esus"] or iRel == Rel["Ashen Veil"] or iRel == Rel["Octopus Overlords"] or hasPromo(Generic["Death I"]) or hasPromo(Generic["Entropy I"])):
						bValid = False		#We will have already created two Manes, so don't create a Frozen Soul! - To balance Frozen population
					if (iRel == Rel["Empyrean"] or iRel == Rel["Order"] or iRel == Rel["Runes of Kilmorph"] or (iUnitCombat != UnitCombat["Animal"] and iCiv == Civ["Mercurians"])):
						bValid = False		#We will have already created an Angel, so don't create a Frozen Soul! 	- To balance Frozen population
					if bValid == True:
						giftUnit( Frozen["Frozen Souls"], Civ["Frozen"], 0, pPlot, iOwner)
#End of Frozen

			if hasPromo( Promo["Spirit Guide"]):
				if pUnit.getExperience() > 0:
					py = PyPlayer(iPlayer)
					lUnits = []
					for pLoopUnit in py.getUnitList():
						if pLoopUnit.isAlive():
							if not pLoopUnit.isOnlyDefensive():
								if not pLoopUnit.isDelayedDeath():
									lUnits.append(pLoopUnit)
					if len(lUnits) > 0:
						pUnit = lUnits[randNum(len(lUnits), "Spirit Guide")-1]
						iXP = getXP() / 2
						pUnit.changeExperienceTimes100(iXP, -1, False, False, False)
						pUnit.changeExperienceTimes100(iXP * -1, -1, False, False, False)
						addMsg(iOwner,True,25,getText("TXT_KEY_MESSAGE_SPIRIT_GUIDE",()),'AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/Promotions/SpiritGuide.dds',ColorTypes(7),pUnit.getX(),pUnit.getY(),True,True)

		if iUnitType == Hero["Acheron"]:
			pUnit.setHasPromotion(Promo["Held"], False)

		if game.getWBMapScript():
			sf.onUnitKilled(pUnit, iAttacker)
					
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onUnitKilled']:
			module.onUnitKilled(self, argsList)

		## Modular Python End
		## *******************


		if (not self.__LOG_UNITKILLED):
			return
		CvUtil.pyPrint('Player %d Civilization %s Unit %s was killed by Player %d'
			%(player.getID(), player.getCivilizationName(), PyInfo.UnitInfo(iUnitType).getDescription(), attacker.getID()))

	def onUnitLost(self, argsList):
		'Unit Lost'
		pUnit = argsList[0]
		player = PyPlayer(pUnit.getOwner())
		if (not self.__LOG_UNITLOST):
			return
		CvUtil.pyPrint('%s was lost by Player %d Civilization %s'
			%(PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))

	def onUnitPromoted(self, argsList):
		'Unit Promoted'
		pUnit, iPromotion = argsList
		player 		= PyPlayer(pUnit.getOwner())
		hasPromo 	= pUnit.isHasPromotion
		iUnitType 	= pUnit.getUnitType()
		Promo		= self.Promotions["Effects"]
		Race		= self.Promotions["Race"]
		Generic		= self.Promotions["Generic"]
		Sheaim		= self.Units["Sheaim"]
		Scions		= self.Units["Scions"]
		UnitAI		= self.UnitAI

# Trying to get high experienced units out of City Defense Notque 03/10/09
		if hasPromo(Promo["Hero"]):
			if pUnit.getExperience() > 25:
				if pUnit.getUnitAIType() != UnitAI["Counter"]: 
					pUnit.setUnitAIType(UnitAI["Counter"])

		# Change to see if I can get Pyre Zombies to stop City Defense Notque 03/10/09
		if iUnitType == Sheaim["Pyre Zombie"]:
			if pUnit.getUnitAIType() == UnitAI["City Defense"]: 
				if CyGame().getSorenRandNum(100, "PYRE_UNITAI") >= 50:
					pUnit.setUnitAIType(UnitAI["Collateral"])
				else:
					pUnit.setUnitAIType(UnitAI["Counter"])

#scions start - Handles Centeni promotions, makes sure they lose Undead status and get Chosen promotion when upgraded to another unit type
		if hasPromo(Promo["Unreliable"]):
			if iUnitType != Scions["Centeni"] or hasPromo(Generic["Headless"]):
				pUnit.setHasPromotion(Promo["Alive"], False)
				pUnit.setHasPromotion(Race["Undead"], True)
				pUnit.setHasPromotion(Promo["Chosen"], True)
				pUnit.setHasPromotion(Promo["Unreliable"], False)

# scions end

		if (not self.__LOG_UNITPROMOTED):
			return
		CvUtil.pyPrint('Unit Promotion Event: %s - %s' %(player.getCivilizationName(), pUnit.getName(),))

	def onUnitSelected(self, argsList):
		'Unit Selected'
		unit = argsList[0]
		player = PyPlayer(unit.getOwner())
				
		if (not self.__LOG_UNITSELECTED):
			return
		CvUtil.pyPrint('%s was selected by Player %d Civilization %s'
			%(PyInfo.UnitInfo(unit.getUnitType()).getDescription(), player.getID(), player.getCivilizationName()))

	def onUnitRename(self, argsList):
		'Unit is renamed'
		pUnit = argsList[0]
		if (pUnit.getOwner() == CyGame().getActivePlayer()):
			self.__eventEditUnitNameBegin(pUnit)

	def onUnitPillage(self, argsList):
		'Unit pillages a plot'
		pUnit, iImprovement, iRoute, iOwner = argsList
		iPlotX = pUnit.getX()
		iPlotY = pUnit.getY()
		gc = CyGlobalContext() 
		getInfoType	= gc.getInfoTypeForString
		map 	= CyMap()
		pPlot 	= map.plot(iPlotX, iPlotY)
		pPlayer = gc.getPlayer(iOwner)
		Status 	= self.LeaderStatus
		Trait 	= self.Traits
		
		if pPlayer.getLeaderStatus() == Status["Emergent"] and not pPlayer.hasTrait(Trait["Raiders"]):
			if CyGame().getSorenRandNum(100, "Bob") <= 3:
				pPlayer.initTriggeredData(self.EventTriggers["Raiders"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		if (not self.__LOG_UNITPILLAGE):
			return
		CvUtil.pyPrint("Player %d's %s pillaged improvement %d and route %d at plot at (%d, %d)"
			%(iOwner, PyInfo.UnitInfo(pUnit.getUnitType()).getDescription(), iImprovement, iRoute, iPlotX, iPlotY))

	def onUnitSpreadReligionAttempt(self, argsList):
		'Unit tries to spread religion to a city'
		pUnit, iReligion, bSuccess = argsList
		map = CyMap()
		
		iX = pUnit.getX()
		iY = pUnit.getY()
		pPlot = map.plot(iX, iY)
		pCity = pPlot.getPlotCity()

	def onUnitGifted(self, argsList):
		'Unit is gifted from one player to another'
		pUnit, iGiftingPlayer, pPlotLocation = argsList

	def onUnitBuildImprovement(self, argsList):
		'Unit begins enacting a Build (building an Improvement or Route)'
		pUnit, iBuild, bFinished = argsList

	def onGoodyReceived(self, argsList):
		'Goody received'
		iPlayer, pPlot, pUnit, iGoodyType = argsList
		gc = CyGlobalContext() 
		if (not self.__LOG_GOODYRECEIVED):
			return
		CvUtil.pyPrint('%s received a goody' %(gc.getPlayer(iPlayer).getCivilizationDescription(0)),)

	def onGreatPersonBorn(self, argsList):
		'Unit Promoted'
		pUnit, iPlayer, pCity = argsList
		gc 			= CyGlobalContext() #Cause local variables are faster
		getInfoType	= gc.getInfoTypeForString
		game = CyGame()
		player = PyPlayer(iPlayer)
		pPlayer = gc.getPlayer(iPlayer)
		Status 	= self.LeaderStatus
		Trait 	= self.Traits

		if pPlayer.getLeaderStatus() == Status["Emergent"] and not pPlayer.hasTrait(Trait["Philosophical"]):
			if game.getSorenRandNum(100, "Bob") <= 10:
				pPlayer.initTriggeredData(self.EventTriggers["Philosophical"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		if pUnit.isNone() or pCity.isNone():
			return
		if (not self.__LOG_GREATPERSON):
			return
		CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))

	def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object

		self.verifyLoaded()
		
		gc = CyGlobalContext() 
		getInfoType	= gc.getInfoTypeForString
		cf			= self.cf
		game 		= CyGame()
		map 		= CyMap()
		plotByIndex = map.plotByIndex
		randNum 	= game.getSorenRandNum
		getPlayer 	= gc.getPlayer
		getTeam 	= gc.getTeam
		Option		= self.GameOptions
		Tech		= self.Techs
		Civ			= self.Civilizations
		Rel			= self.Religions
		Trait		= self.Traits
		Status		= self.LeaderStatus
		Promo		= self.Promotions["Effects"]
		Race		= self.Promotions["Race"]
		Generic		= self.Promotions["Generic"]
		Unit		= self.Units["Generic"]
		iNoAI		= UnitAITypes.NO_UNITAI
		iNorth		= DirectionTypes.DIRECTION_NORTH
		

		# Show tech splash when applicable
		if (iPlayer > -1 and bAnnounce and not CyInterface().noTechSplash()):
			if (game.isFinalInitialized() and not game.GetWorldBuilderMode()):
				if ((not game.isNetworkMultiPlayer()) and (iPlayer == game.getActivePlayer())):
					popupInfo = CyPopupInfo()
					popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
					popupInfo.setData1(iTechType)
					popupInfo.setText(u"showTechSplash")
					popupInfo.addPopup(iPlayer)

		if (iPlayer != -1 and iPlayer != gc.getORC_PLAYER() and iPlayer != gc.getANIMAL_PLAYER() and iPlayer != gc.getDEMON_PLAYER()):
			pPlayer = getPlayer(iPlayer)
			iReligion = -1
			if   iTechType == Tech["Corruption of Spirit"]:
				iUnit 	    = self.Units["Veil"]["Disciple"]
				iReligion   = Rel["Ashen Veil"]
			elif iTechType == Tech["Orders from Heaven"]:
				iUnit  	    = self.Units["Order"]["Disciple"]
				iReligion   = Rel["Order"]
			elif iTechType == Tech["Way of the Forests"]:
				iUnit 	    = self.Units["Leaves"]["Disciple"]
				iReligion   = Rel["Fellowship"]
			elif iTechType == Tech["Way of the Earthmother"]:
				iUnit 	    = self.Units["Runes"]["Disciple"]
				iReligion   = Rel["Runes of Kilmorph"]
			elif iTechType == Tech["Message from the Deep"]:
				iUnit 	    = self.Units["Overlords"]["Disciple"]
				iReligion   = Rel["Octopus Overlords"]
			elif iTechType == Tech["Honor"]:
				iUnit 	    = self.Units["Empyrean"]["Disciple"]
				iReligion   = Rel["Empyrean"]
			elif iTechType == Tech["Deception"]:
				iUnit 	    = self.Units["Esus"]["Nightwatch"]
				iReligion   = Rel["Council of Esus"]
			if iReligion   != -1:			
				if game.isReligionFounded(iReligion):
					if cf.canReceiveReligionUnit(pPlayer):
						cf.giftUnit(iUnit, pPlayer.getCivilizationType(), 0, -1, -1)

		if not Option["No Hyborem or Basium"]:
			if (iTechType == Tech["Infernal Pact"] and iPlayer != -1):
				if game.countKnownTechNumTeams( Tech["Infernal Pact"]) == 1 and not game.isCivEverActive( Civ["Infernal"]):
					iInfernalPlayer = getOpenPlayer()
					pBestPlot = -1
					iBestPlot = -1
					for i in xrange(map.numPlots()):
						pPlot = plotByIndex(i)
						iPlot = -1
						if pPlot.isWater() == False:
							if pPlot.getNumUnits() == 0:
								if pPlot.isCity() == False:
									if pPlot.isImpassable() == False:
										iPlot = randNum(500, "Place Hyborem")
										iPlot = iPlot + (pPlot.area().getNumTiles() * 2)
										iPlot = iPlot + (pPlot.area().getNumUnownedTiles() * 10)
										if pPlot.isOwned() == False:
											iPlot = iPlot + 500
										if pPlot.getOwner() == iPlayer:
											iPlot = iPlot + 200
						if iPlot > iBestPlot:
							iBestPlot = iPlot
							pBestPlot = pPlot
					if (iInfernalPlayer != -1 and pBestPlot != -1):
						game.addPlayerAdvanced(iInfernalPlayer, -1, self.Leaders["Hyborem"], Civ["Infernal"])
						iFounderTeam  = getPlayer(iPlayer).getTeam()
						eFounderTeam  = getTeam(getPlayer(iPlayer).getTeam())
						iInfernalTeam = getPlayer(iInfernalPlayer).getTeam()
						eInfernalTeam = getTeam(iInfernalTeam)
						for iTech in xrange(gc.getNumTechInfos()):
							if eFounderTeam.isHasTech(iTech):
								eInfernalTeam.setHasTech(iTech, True, iInfernalPlayer, True, False)
						eFounderTeam.signOpenBorders(iInfernalTeam)
						eInfernalTeam.signOpenBorders(iFounderTeam)
						iBarbTeam = getPlayer(gc.getDEMON_PLAYER()).getTeam()
						eInfernalTeam.makePeace(iBarbTeam)
						declareWar = eInfernalTeam.declareWar
						for iTeam in xrange(gc.getMAX_TEAMS()):
							if iTeam != iBarbTeam:
								eTeam = getTeam(iTeam)
								if eTeam.isAlive():
									if eFounderTeam.isAtWar(iTeam) or eTeam.isBarbarian():
										declareWar(iTeam, False, WarPlanTypes.WARPLAN_LIMITED)
						pInfernalPlayer = getPlayer(iInfernalPlayer)
						pInfernalPlayer.AI_changeAttitudeExtra(iPlayer,4)
						initUnit = pInfernalPlayer.initUnit
						iX = pBestPlot.getX(); iY = pBestPlot.getY()
						newUnit1  = initUnit(self.Heroes["Hyborem"], iX, iY, iNoAI, iNorth)
						newUnit1.setHasPromotion( Promo["Immortal"], True)
						newUnit1.setHasCasted(True)
						newUnit1.setExperienceTimes100(2500, -1)

						newUnit2  = initUnit( Unit["Longbowman"], iX, iY, iNoAI, iNorth)
						newUnit2.setHasPromotion( Generic["Mobility I"], True)
						newUnit3  = initUnit( Unit["Longbowman"], iX, iY, iNoAI, iNorth)
						newUnit3.setHasPromotion( Generic["Mobility I"], True)
						newUnit4  = initUnit( Unit["Champion"], iX, iY, iNoAI, iNorth)
						newUnit4.setHasPromotion( Promo["Iron Weapons"], True)
						newUnit4.setHasPromotion( Generic["Mobility I"], True)
						newUnit5  = initUnit( Unit["Champion"], iX, iY, iNoAI, iNorth)
						newUnit5.setHasPromotion( Promo["Iron Weapons"], True)
						newUnit5.setHasPromotion( Generic["Mobility I"], True)
						newUnit6  = initUnit( Unit["Worker"], iX, iY, iNoAI, iNorth)
						newUnit6.setHasPromotion( Race["Demon"], True)
						newUnit7  = initUnit( Unit["Infernal"]["Imp"], iX, iY, iNoAI, iNorth)
						newUnit7.setHasPromotion( Generic["Mobility I"], True)
						newUnit8  = initUnit( Unit["Infernal"]["Manes"], iX, iY, iNoAI, iNorth)
						newUnit9  = initUnit( Unit["Infernal"]["Manes"], iX, iY, iNoAI, iNorth)
						newUnit10 = initUnit( Unit["Infernal"]["Manes"], iX, iY, iNoAI, iNorth)
						newUnit11 = initUnit( Unit["Settler"], iX, iY, iNoAI, iNorth)
						newUnit11.setHasPromotion( Promo["Starting Settler"], True)
						newUnit11.setHasPromotion( Race["Demon"], True)
						newUnit12 = initUnit( Unit["Settler"], iX, iY, iNoAI, iNorth)
						newUnit12.setHasPromotion( Promo["Starting Settler"], True)
						newUnit12.setHasPromotion( Race["Demon"], True)
						if getPlayer(iPlayer).isHuman() and not game.GetWorldBuilderMode():
							popupInfo = CyPopupInfo()
							popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON)
							popupInfo.setText(CyTranslator().getText("TXT_KEY_POPUP_CONTROL_INFERNAL",()))
							popupInfo.setData1(iPlayer)
							popupInfo.setData2(iInfernalPlayer)
							popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_YES", ()), "")
							popupInfo.addPythonButton(CyTranslator().getText("TXT_KEY_POPUP_NO", ()), "")
							popupInfo.setOnClickedPythonCallback("reassignPlayer")
							popupInfo.addPopup(iPlayer)
				else:
					if game.getNumCivActive( Civ["Infernal"]) > 0:
						pInfernalPlayer = getPlayer(game.getCivActivePlayer(Civ["Infernal"], 0))
						if not game.isUnitClassMaxedOut( self.UnitClasses["Hyborem"], 0):
							pCapital = pInfernalPlayer.getCapitalCity()
							if not pCapital.isNone():
								newUnit1 = pInfernalPlayer.initUnit( self.Heroes["Hyborem"], pCapital.getX(), pCapital.getY(), iNoAI, iNorth)
								newUnit1.setHasPromotion( Promo["Immortal"], True)
								newUnit1.setHasCasted(True)
								newUnit1.setExperienceTimes100(2500, -1)


		if (iPlayer != -1):
			pPlayer 		 = getPlayer(iPlayer)
			iLeaderStatus 	 = pPlayer.getLeaderStatus()
			if iLeaderStatus == Status["Emergent"]:
				trigger	 	 	 = pPlayer.initTriggeredData
				findInfo 		 = CvUtil.findInfoTypeNum
				getNumTriggers 	 = gc.getNumEventTriggerInfos
				eventTriggerInfo = gc.getEventTriggerInfo
				hasTrait 		 = pPlayer.hasTrait
				Event			 = self.EventTriggers
				
				iRand  = randNum(100, "Bob")
				iRand2 = randNum(100, "Bob")
				
				if   iTechType == Tech["Construction"]:
					if not hasTrait( Trait["Industrious"]) and iRand <= 3:
						trigger( Event["Industrious"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Expansive"]) and iRand2 <= 5:
							trigger( Event["Expansive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
							
				elif iTechType == Tech["Smelting"]:
					if not hasTrait( Trait["Industrious"]) and iRand <= 10:
						trigger( Event["Industrious"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Aggressive"]) and iRand2 <= 5:
							trigger( Event["Aggressive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
				elif iTechType == Tech["Engineering"]:
					if not hasTrait( Trait["Industrious"]) and iRand <= 20:
						trigger(Event["Industrious"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Expansive"]) and iRand2 <= 5:
							trigger(Event["Expansive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
							
				elif iTechType == Tech["Trade"]:
					if not hasTrait( Trait["Financial"]) and iRand <= 10:
						trigger(Event["Financial"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						if not hasTrait( Trait["Ingenuity"]) and iRand2 <= 5:
							trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
				elif iTechType == Tech["Currency"]:
					if not hasTrait( Trait["Financial"]) and iRand <= 20:
						trigger(Event["Financial"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Organized"]) and iRand2 <= 5:
							trigger(Event["Organized"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Math"]:
					if not hasTrait( Trait["Financial"]) and iRand <= 50:
						trigger(Event["Financial"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Philosophical"]) and iRand2 <= 5:
							trigger(Event["Philosophical"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Code of Laws"]:
					if not hasTrait( Trait["Organized"]) and iRand <= 3:
						trigger(Event["Organized"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Financial"]) and iRand2 <= 5:
							trigger(Event["Financial"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Military Strategy"]:
					if not hasTrait( Trait["Organized"]) and iRand <= 20:
						trigger(Event["Organized"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Aggressive"]) and iRand2 <= 5:
							trigger(Event["Aggressive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Writing"]:
					if not hasTrait( Trait["Philosophical"]) and iRand <= 3:
						trigger(Event["Philosophical"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Creative"]) and iRand2 <= 5:
							trigger(Event["Creative"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
					if not hasTrait( Trait["Expansive"]) and randNum(100, "Bob") <= 10:
						trigger(Event["Expansive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
						if not hasTrait( Trait["Ingenuity"]) and randNum(100, "Bob") <= 5:
							trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
					if not hasTrait( Trait["Creative"]) and randNum(100, "Bob") <= 3:
						trigger(Event["Creative"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Ingenuity"]) and randNum(100, "Bob") <= 5:
							trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
					
				elif iTechType == Tech["Philosophy"]:
					if not hasTrait( Trait["Philosophical"]) and iRand <= 50:
						trigger(Event["Philosophical"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
						if not hasTrait( Trait["Creative"]) and iRand2 <= 5:
							trigger(Event["Creative"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Medicine"]:
					if not hasTrait( Trait["Expansive"]) and iRand <= 20:
						trigger(Event["Expansive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Ingenuity"]) and iRand2 <= 5:
							trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Drama"]:
					if not hasTrait( Trait["Creative"]) and iRand <= 20:
						trigger(Event["Creative"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Ingenuity"]) and iRand2 <= 5:
							trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Feudalism"]:
					if not hasTrait( Trait["Ingenuity"]) and iRand <= 20:
						trigger(Event["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Creative"]) and iRand2 <= 5:
							trigger(Event["Creative"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Knowledge of the Ether"]:
					if not hasTrait( Trait["Arcane"]) and iRand <= 5:
						trigger(Event["Arcane"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Summoner"]) and iRand2 <= 5:
							trigger(Event["Summoner"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
						
					if not hasTrait( Trait["Summoner"]) and randNum(100, "Bob") <= 3:
						trigger(Event["Summoner"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
						if not hasTrait( Trait["Arcane"]) and randNum(100, "Bob") <= 5:
							trigger(Event["Arcane"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Sorcery"]:
					if not hasTrait( Trait["Arcane"]) and iRand <= 50:
						trigger(Event["Arcane"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
						if not hasTrait( Trait["Summoner"]) and iRand2 <= 5:
							trigger(Event["Summoner"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
					if not hasTrait( Trait["Summoner"]) and randNum(100, "Bob") <= 50:
						trigger(Event["Summoner"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
						if not hasTrait( Trait["Arcane"]) and randNum(100, "Bob") <= 5:
							trigger(Event["Arcane"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
				
				elif iTechType == Tech["Optics"]:
					if not hasTrait( Trait["Swashbuckler"]):
						trigger(Event["Swashbuckler"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)
					
			if iTechType == Tech["Strength of Will"]:
				if getPlayer(iPlayer).getCivilizationType() == Civ["Kahdi"]:
					if iPlayer == game.getActivePlayer():
						addPopup(CyTranslator().getText("TXT_KEY_POPUP_KAHDI_STRENGTH_OF_WILL",()),'art/interface/popups/Blank.dds')

		if game.getWBMapScript():
			sf.onTechAcquired(iTechType, iTeam, iPlayer, bAnnounce)
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onTechAcquired']:
			module.onTechAcquired(self, argsList)

		## Modular Python End
		## *******************

			
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was finished by Team %d'
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))

	def onTechSelected(self, argsList):
		'Tech Selected'
		iTechType, iPlayer = argsList
		if (not self.__LOG_TECH):
			return
		CvUtil.pyPrint('%s was selected by Player %d' %(PyInfo.TechnologyInfo(iTechType).getDescription(), iPlayer))

	def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		gc 			= CyGlobalContext() #Cause local variables are faster
		getInfoType	= gc.getInfoTypeForString
		game 		= CyGame()
		player 		= PyPlayer(iFounder)
		pPlayer 	= gc.getPlayer(iFounder)
		Status		= self.LeaderStatus
		Trait		= self.Traits
		Rel			= self.Religions

		if (pPlayer.getLeaderStatus() == Status["Emergent"]) and not pPlayer.hasTrait(Trait["Spiritual"]):
			if game.getSorenRandNum(100, "Bob") <= 50:
				pPlayer.initTriggeredData(self.EventTriggers["Spiritual"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		iCityId = game.getHolyCity(iReligion).getID()
		if (game.isFinalInitialized() and not game.GetWorldBuilderMode()):
			if ((not game.isNetworkMultiPlayer()) and (iFounder == game.getActivePlayer())):
				popupInfo = CyPopupInfo()
				popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
				popupInfo.setData1(iReligion)
				popupInfo.setData2(iCityId)
				if (iReligion == Rel["Empyrean"] or iReligion == Rel["Council of Esus"]):
					popupInfo.setData3(3)
				else:
					popupInfo.setData3(1)
				popupInfo.setText(u"showWonderMovie")
				popupInfo.addPopup(iFounder)

		if game.getWBMapScript():
			sf.onReligionFounded(iReligion, iFounder)

		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))

	def onReligionSpread(self, argsList):
		'Religion Has Spread to a City'
		iReligion, iOwner, pSpreadCity = argsList
		gc 			= CyGlobalContext() #Cause local variables are faster
		getInfoType	= gc.getInfoTypeForString
		game 		= CyGame()
		player 		= PyPlayer(iOwner)
		Rel			= self.Religions
		pPlayer 	= gc.getPlayer(iOwner)

		if iReligion == Rel["Order"] and game.getGameTurn() != game.getStartTurn():
			if (pPlayer.getStateReligion() == Rel["Order"] and pSpreadCity.getOccupationTimer() <= 0):
				if (game.getSorenRandNum(100, "Order Spawn") < self.Defines["Order Spawn"]):
					Order 	= self.Units["Order"]
					eTeam 	= gc.getTeam(pPlayer.getTeam())
					iSouth	= DirectionTypes.DIRECTION_SOUTH
					iNoAI	= UnitAITypes.NO_UNITAI
					if eTeam.isHasTech( self.Techs["Fanaticism"]):
						iUnit = Order["Crusader"]
						CyInterface().addMessage(iOwner,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_ORDER_SPAWN_CRUSADER",()),'AS2D_UNIT_BUILD_UNIT',1,'Art/Interface/Buttons/Units/Crusader.dds',ColorTypes(8),pSpreadCity.getX(),pSpreadCity.getY(),True,True)
					else:
						iUnit = Order["Disciple"]
						CyInterface().addMessage(iOwner,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_ORDER_SPAWN_ACOLYTE",()),'AS2D_UNIT_BUILD_UNIT',1,'Art/Interface/Buttons/Units/Disciple Order.dds',ColorTypes(8),pSpreadCity.getX(),pSpreadCity.getY(),True,True)
					newUnit = pPlayer.initUnit(iUnit, pSpreadCity.getX(), pSpreadCity.getY(), iNoAI, iSouth)

		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onReligionRemove(self, argsList):
		'Religion Has been removed from a City'
		iReligion, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		gc = CyGlobalContext() 
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))

	def onCorporationFounded(self, argsList):
		'Corporation Founded'
		iCorporation, iFounder = argsList
		player = PyPlayer(iFounder)
		gc = CyGlobalContext() 
		if (not self.__LOG_RELIGION):
			return
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getCorporationInfo(iCorporation).getDescription()))

	def onCorporationSpread(self, argsList):
		'Corporation Has Spread to a City'
		iCorporation, iOwner, pSpreadCity = argsList
		player = PyPlayer(iOwner)
		gc = CyGlobalContext() 
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has spread to Player %d Civilization %s city of %s'
			%(gc.getCorporationInfo(iCorporation).getDescription(), iOwner, player.getCivilizationName(), pSpreadCity.getName()))

	def onCorporationRemove(self, argsList):
		'Corporation Has been removed from a City'
		iCorporation, iOwner, pRemoveCity = argsList
		player = PyPlayer(iOwner)
		gc = CyGlobalContext() 
		if (not self.__LOG_RELIGIONSPREAD):
			return
		CvUtil.pyPrint('%s has been removed from Player %d Civilization %s city of %s'
			%(gc.getReligionInfo(iReligion).getDescription(), iOwner, player.getCivilizationName(), pRemoveCity.getName()))

	def onGoldenAge(self, argsList):
		'Golden Age'
		iPlayer = argsList[0]
		gc = CyGlobalContext() #Cause local variables are faster
		getInfoType	= gc.getInfoTypeForString 
		player 		= PyPlayer(iPlayer)
		pPlayer 	= gc.getPlayer(iPlayer)
		Status		= self.LeaderStatus
		Trait		= self.Traits
		
		if (pPlayer.getLeaderStatus() == Status["Emergent"]) and not pPlayer.hasTrait( Trait["Magic Resistant"]):  
			if not pPlayer.hasTrait( Trait["Ingenuity"]) and CyGame().getSorenRandNum(100, "Bob") <= 50: 
				pPlayer.initTriggeredData(self.EventTriggers["Ingenuity"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		if (not self.__LOG_GOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s has begun a golden age'
			%(iPlayer, player.getCivilizationName()))

	def onEndGoldenAge(self, argsList):
		'End Golden Age'
		iPlayer = argsList[0]
		player = PyPlayer(iPlayer)
		if (not self.__LOG_ENDGOLDENAGE):
			return
		CvUtil.pyPrint('Player %d Civilization %s golden age has ended'
			%(iPlayer, player.getCivilizationName()))

	def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iTeam = argsList[1]
		iRivalTeam = argsList[2]
		if (not self.__LOG_WARPEACE):
			return
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		CvUtil.pyPrint('Team %d has %s on Team %d'
			%(iTeam, strStatus, iRivalTeam))

	def onChat(self, argsList):
		'Chat Message Event'
		chatMessage = "%s" %(argsList[0],)

	def onSetPlayerAlive(self, argsList):
		'Set Player Alive Event'
		iPlayerID = argsList[0]
		bNewValue = argsList[1]
		gc 			= CyGlobalContext() #Cause local variables are faster
		cf			= self.cf
		getInfoType	= gc.getInfoTypeForString 
		getPlayer 	= gc.getPlayer
		game 		= CyGame()
		CvUtil.pyPrint("Player %d's alive status set to: %d" %(iPlayerID, int(bNewValue)))

		if (bNewValue == False and game.getGameTurnYear() >= 5):
			pPlayer = getPlayer(iPlayerID)
			if pPlayer.getAlignment() == self.Alignments["Good"]:
				game.changeGlobalCounter(5)
			if pPlayer.getAlignment() == self.Alignments["Evil"]:
				game.changeGlobalCounter(-5)
			if game.getWBMapScript():
				sf.playerDefeated(pPlayer)
			else:
				if game.getAIAutoPlay() == 0:
					activePlayer = getPlayer(game.getActivePlayer())
					sPlayerName = pPlayer.getName()
					sQuote = gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getDefeatQuote()
					sPopupText = CyTranslator().getText('TXT_KEY_MISC_DEFEAT_POPUP',(sPlayerName, sQuote))
					addPopup(sPopupText, str(gc.getLeaderHeadInfo(pPlayer.getLeaderType()).getImage()))

	def onPlayerChangeStateReligion(self, argsList):
		'Player changes his state religion'
		iPlayer, iNewReligion, iOldReligion = argsList
		gc = CyGlobalContext() #Cause local variables are faster
		getInfoType	= gc.getInfoTypeForString 
		getPlayer 	= gc.getPlayer
		pPlayer 	= getPlayer(iPlayer)
		getTeam		= gc.getTeam
		Rel			= self.Religions
		Civ			= self.Civilizations
		iCiv 		= pPlayer.getCivilizationType()

		if iOldReligion == Rel["Ashen Veil"] and iNewReligion != Rel["Ashen Veil"]:
			eDemonTeam = getTeam(getPlayer(gc.getDEMON_PLAYER()).getTeam())
			iTeam = pPlayer.getTeam()
			if eDemonTeam.isAtWar(iTeam) == False:
				eDemonTeam.declareWar(iTeam, False, WarPlanTypes.WARPLAN_TOTAL)

		if iNewReligion == Rel["Ashen Veil"] and (iCiv == Civ["Sheaim"] or iCiv == Civ["Infernal"]):
			iDemonTeam = gc.getDEMON_TEAM()
			eTeam = getTeam(pPlayer.getTeam())
			eTeam.makePeace(iDemonTeam)

#FF: Added by Jean Elcard 03/01/2009 (State Names)
		pPlayer.updateStateNameType()
#FF: End Add

	def onPlayerGoldTrade(self, argsList):
		'Player Trades gold to another player'
		iFromPlayer, iToPlayer, iGoldAmount = argsList

	def onCityBuilt(self, argsList):
		'City Built'
		pCity 		= argsList[0]
		gc 			= CyGlobalContext()
		cf			= self.cf
		getInfoType	= gc.getInfoTypeForString
		game 		= CyGame()
		pPlot		= pCity.plot()
		iOwner		= pCity.getOwner()
		iSouth 		= DirectionTypes.DIRECTION_SOUTH
		iNoAI		= UnitAITypes.NO_UNITAI
		pPlayer		= gc.getPlayer(iOwner)
		Tech 		= self.Techs
		Speed 		= self.GameSpeeds 
		eSpeed 		= game.getGameSpeedType()
		Civ			= self.Civilizations
		Rel			= self.Religions
		eCiv 		= pPlayer.getCivilizationType()
		Leader		= self.Leaders
		eLeader		= pPlayer.getLeaderType()
		Status		= self.LeaderStatus
		eStatus 	= pPlayer.getLeaderStatus()
		hasTrait	= pPlayer.hasTrait
		Trait		= self.Traits
		hasTrait	= pPlayer.hasTrait
		Building	= self.Buildings
		setNumB		= pCity.setNumRealBuilding
		iNumCities	= pPlayer.getNumCities()
		Unit		= self.Units["Generic"]
		Promo		= self.Promotions["Effects"]
		Race		= self.Promotions["Race"]
		Event		= self.EventTriggers
		iX = pCity.getX(); iY = pCity.getY()
		
		if hasTrait( Trait["Imperialist"]):
			iCulture = 10
			if   eSpeed == Speed["Marathon"]:	iCulture = iCulture * 3
			elif eSpeed == Speed["Epic"]:	   	iCulture = iCulture * 3 / 2
			elif eSpeed == Speed["Quick"]:	   	iCulture = iCulture / 3 * 2
			pCity.changeCulture(iOwner, iCulture, True)
		
		if eStatus == Status["Minor"]:
			if iNumCities >= 4:
				pPlayer.setLeaderStatus( Status["Major"])
		
		if eStatus == Status["Emergent"] and not hasTrait( Trait["Expansive"]):
			if iNumCities > 7:
				if game.getSorenRandNum(100, "Bob") <= 20:
					pPlayer.initTriggeredData( Event["Expansive"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		if pPlot.getBonusType(-1) == self.Mana["Mana"]:
			pPlot.setBonusType(-1)

		if eCiv == Civ["Infernal"]:
			setNumB( Building["Demonic Citizens"], 1)
			pCity.setPopulation(3)
			if game.countKnownTechNumTeams( Tech["Infernal Pact"]) > 0:
				pCity.setHasReligion( Rel["Ashen Veil"], True, True, True)
				setNumB( Building["Elder Council"], 1)
				setNumB( Building["Barracks"], 		1)
				setNumB( Building["Obsidian Gate"], 1)
				setNumB( Building["Forge"], 		1)
				setNumB( Building["Mage Guild"], 	1)

#	Added in Frozen: TC01
#	Builds buildings in Frozen cities if the Frozen module is loaded. Triggers Temple of the Hand custom function.
		#Added in Frozen: TC01
		elif eCiv == Civ["Frozen"]:
			pCity.setPopulation(2)
			setNumB( Building["Elder Council"], 1)
			setNumB( Building["Obsidian Gate"], 1)
			setNumB( Building["Forge"], 		1)
			setNumB( Building["Mage Guild"], 	1)
			setNumB( Building["Frozen Souls"], 	1)
			
			cf.doTempleWhiteHand(pCity)
#End of Frozen

		elif eCiv == Civ["Austrin"]:
			setNumB( Building["Austrin Settlement"], 1)
			
		elif eCiv == Civ["Balseraphs"]:
			pCity.setHasCorporation( self.Corporations["Masquerade"], True, False, False)
			
		elif eCiv == Civ["Barbarian (Orc)"]:
			eEra 		= game.getStartEra()
			eTeam 		= gc.getTeam(gc.getPlayer(gc.getORC_PLAYER()).getTeam())
			iUnit 		= Unit["Warrior"]
			Era 		= self.Eras
			isHasTech	= eTeam.isHasTech
			if (isHasTech( Tech["Bronze Working"]) or eEra > Era["Ancient"]):
				iUnit 	= Unit["Axeman"]
			if (isHasTech( Tech["Iron Working"]) or eEra > Era["Classical"]):
				iUnit 	= Unit["Champion"]
			newUnit 	= pPlayer.initUnit(iUnit, iX, iY, iNoAI, iSouth)
			newUnit.setHasPromotion( Race["Orcish"], True)
			
			iUnit = Unit["Archer"]
			if (isHasTech( Tech["Bowyers"]) or eEra > Era["Classical"]):	
				iUnit 	= Unit["Longbowman"]
			newUnit2 	= pPlayer.initUnit(iUnit, iX, iY, iNoAI, iSouth)
			newUnit3 	= pPlayer.initUnit(iUnit, iX, iY, iNoAI, iSouth)			
			newUnit2.setHasPromotion( Race["Orcish"], True)
			newUnit3.setHasPromotion( Race["Orcish"], True)			
			if ((not isHasTech( Tech["Archery"])) or eEra == Era["Ancient"]):				
				newUnit2.setHasPromotion( Promo["Weak"], True)			
				newUnit3.setHasPromotion( Promo["Weak"], True)							

		elif eCiv == Civ["Mechanos"]:
			if pCity.getNumBuilding( Building["Mechanos Palace"]) > 0:
				pCity.setHasReligion( Rel["Machinarum"], True, True, True)
#FF: Added by Jean Elcard 03/01/2009 (State Names)
		pPlayer.updateStateNameType()
#FF: End Add

		if hasTrait( Trait["Tyrant"]):
			setNumB( Building["Tyrant"], 1)

		if eLeader == Leader["Risen Emperor"]:
			setNumB( Building["Emperors Mark"], 1)

		if game.getWBMapScript():
			sf.onCityBuilt(pCity)
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onCityBuilt']:
			module.onCityBuilt(self, argsList)

		## Modular Python End
		## *******************
			
		if pCity.getOwner() == game.getActivePlayer() and game.getAIAutoPlay() == 0:
			self.__eventEditCityNameBegin(pCity, False)
		CvUtil.pyPrint('City Built Event: %s' %(pCity.getName()))

	def onCityRazed(self, argsList):
		'City Razed'
		city, iPlayer 	= argsList
		gc 				= CyGlobalContext()
		cf				= self.cf
		game 			= CyGame()
		iOwner 			= city.findHighestCulture()
		iOriginalOwner 	= city.getOriginalOwner()
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		iPopulation 	= city.getPopulation()
		iNewOwner 		= city.getOwner()
		iOriginalAlignment = getPlayer(iOriginalOwner).getAlignment()
		pPlot = city.plot()
		iX = city.getX(); iY = city.getY()
		eCiv 			= getPlayer(iOriginalOwner).getCivilizationType()
		eNewOwnerCiv	= getPlayer(iNewOwner).getCivilizationType()
		getCivActive 	= game.getNumCivActive
		hasTrait	 	= pPlayer.hasTrait
		Civ				= self.Civilizations
		Unit			= self.Units
		Status			= self.LeaderStatus
		Trait			= self.Traits
		Event			= self.EventTriggers
		Tech			= self.Techs
		Alignment		= self.Alignments
		Building		= self.Buildings
		Frozen			= self.Units["Frozen"]
		Infernal		= self.Units["Infernal"]
		Mercurian		= self.Units["Mercurian"]
		iNoAI 			= UnitAITypes.NO_UNITAI
		iSouth			= DirectionTypes.DIRECTION_SOUTH
		
		# Minor Trait Criteria : Raiders
		if (pPlayer.getLeaderStatus() == Status["Emergent"]) and not hasTrait(Trait["Raiders"]):
			if game.getSorenRandNum(100, "Bob") <= 10:
				pPlayer.initTriggeredData(Event["Raiders"], True, -1, -1, -1, -1, -1, -1, -1, -1, -1)

		iAngel 		= Mercurian["Angel"]
		iInfernal 	= Civ["Infernal"]
		iManes 		= Infernal["Manes"]
		iMercurians = Civ["Mercurians"]
		giftUnit = cf.giftUnit
		if game.countKnownTechNumTeams(Tech["Infernal Pact"]) > 0 and getCivActive(iInfernal) > 0:
			if iOriginalAlignment == Alignment["Evil"]:
				if eCiv != iInfernal:
					for i in xrange(iPopulation):
						giftUnit(iManes, iInfernal, 0, pPlot, iNewOwner)

#	Added in Frozen: TC01
#	The equivalent of the Infernal code, it gifts Frozen Souls to the Frozen when a city belonging to a Winterborn player (Doviello, Illians) is razed. Triggered if module is loaded.
		iFrozen = Civ["Frozen"]		#Added in Frozen: TC01
		iFrozenSoul = Frozen["Frozen Souls"] #Added in Frozen: TC01
		if hasTrait(Trait["Winterborn"]):
			if eCiv != iFrozen:
				for i in xrange(iPopulation):
					giftUnit(iFrozenSoul, iFrozen, 0, pPlot, iNewOwner)
#End of Frozen

			if iOriginalAlignment == Alignment["Neutral"]:
				for i in xrange((iPopulation / 4) + 1):
					giftUnit(iManes, iInfernal, 0, pPlot, iNewOwner)
					giftUnit(iManes, iInfernal, 0, pPlot, iNewOwner)

		if game.getBuildingClassCreatedCount(Building["Mercurian Gate"]) > 0 and getCivActive(iMercurians) > 0:
			if iOriginalAlignment == Alignment["Neutral"]:
				for i in xrange((iPopulation / 4) + 1):
					giftUnit(iAngel, iMercurians, 0, pPlot, iNewOwner)

			if iOriginalAlignment == Alignment["Good"]:
				for i in xrange((iPopulation / 2) + 1):
					giftUnit(iAngel, iMercurians, 0, pPlot, iNewOwner)

		if game.getWBMapScript():
			sf.onCityRazed(city, iPlayer)

# scions start - Gives Reborn when razing cities.  The function reducing the population of Scion conquests kicks in first.  Currently Reborn given = that population -1.  Requires Sorc. and Priestood.  It's been suggested that be changed to requiring the civic "Glory."
		iReborn  = Unit["Scions"]["Reborn"]
		eTeam 	 = gc.getTeam(pPlayer.getTeam())
		hasTech	 = eTeam.isHasTech
		initUnit = pPlayer.initUnit
		if eNewOwnerCiv == Civ["Scions"]:
			if hasTech(Tech["Sorcery"]) and hasTech(Tech["Priesthood"]):
				if pPlayer.isCivic(Civic["God King"]):
					if (iPopulation - 1) >= 1:
						CyInterface().addMessage(iNewOwner,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_REBORN_SPAWNED_RAZED", ()),'',1,'Art/Interface/Buttons/Units/Scions/reborn.dds',ColorTypes(8),iX,iY,True,True)
					for i in xrange((iPopulation) - 1):
						spawnUnit = initUnit(iReborn, iX, iY, iNoAI, iSouth)
				else:
					if (iPopulation - 1) >= 1:
						CyInterface().addMessage(iNewOwner,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_REBORN_SPAWNED_RAZED", ()),'',1,'Art/Interface/Buttons/Units/Scions/reborn.dds',ColorTypes(8),iX,iY,True,True)
					for i in xrange((iPopulation) - 1):
						spawnUnit = initUnit(iReborn, iX, iY, iNoAI, iSouth)

# scions end
# Legion
		iSlave = Unit["Generic"]["Slave"]
		iVessel = Unit["D'Tesh"]["Vessel of D'tesh"]
		if eNewOwnerCiv == Civ["D'Tesh"]:
			if hasTech(Tech["Legion"]):
				for i in xrange((iPopulation * 3 / 4) + 1):
					spawnUnit = initUnit(iSlave, iX, iY, iNoAI, iSouth)
			spawnUnit = initUnit(iVessel, iX, iY, iNoAi, iSouth)
# End Legion
		
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onCityRazed']:
			module.onCityRazed(self, argsList)

		## Modular Python End
		## *******************

		CvUtil.pyPrint("City Razed Event: %s" %(city.getName(),))

	def onCityAcquired(self, argsList):
		'City Acquired'
		iPreviousOwner,iNewOwner,pCity,bConquest,bTrade = argsList
		gc 			= CyGlobalContext()
		cf			= self.cf
		game 		= CyGame()
		getPlayer 	= gc.getPlayer
#Scions Start - extra reduction of Scions-acquired cities.  Use of a decimal seems to break it.
		pPlayer 	= getPlayer(iNewOwner)
		hasTrait 	= pPlayer.hasTrait
		pPlot 		= pCity.plot()
		setBuilding = pCity.setNumRealBuilding
		changePop	= pCity.changePopulation
		iCiv		= pPlayer.getCivilizationType()
		iCityOwner 	= pCity.getOwner()
		pCityOwner	= getPlayer(iCityOwner)
		Civ	 		= self.Civilizations
		Trait 		= self.Traits
		Leader 		= self.Leaders
		Civic 		= self.Civics
		Rel	 		= self.Religions
		Building 	= self.Buildings
		Unit		= self.Units
		iPop		= pCity.getPopulation()
		pPrevious	= getPlayer(iPreviousOwner)
		iNoAI 		= UnitAITypes.NO_UNITAI
		iSouth		= DirectionTypes.DIRECTION_SOUTH
		
		if hasTrait(Trait["Slaver"]):
			initUnit 	= pPlayer.initUnit
			iSlave 		= Unit["Generic"]["Slave"]
			eTeam 		= gc.getTeam(pPlayer.getTeam())
			iSlavePop 	= iPop / 2
			changePop(-iSlavePop)
			for i in xrange(iPop / 2):
				spawnUnit = initUnit(iSlave, pCity.getX(), pCity.getY(), iNoAI, iSouth)

		if pCityOwner.getCivilizationType() == Civ["Scions"] and pCityOwner.getLeaderType() != Leader["Koun"]:
			if pCityOwner.isCivic(Civic["God King"]):
				iBCPop = iPop * 10
				iBCMod = iBCPop / 16
				changePop(-iBCMod)
			else:
				iBCPop = iPop * 10
				iBCMod = iBCPop / 12
				changePop(-iBCMod)
				
		if pCityOwner.getCivilizationType() == Civ["D'Tesh"]:
			iBCPop = iPop * 10
			iBCMod = iBCPop / 30
			changePop(-iBCMod)


		if getPlayer(pCity.getPreviousOwner()).getCivilizationType() == Civ["Scions"]:
			iBCPop = iPop * 10
			iBCMod = iBCPop / 12
			changePop(-iBCMod)
#Scions End

		setBuilding(Building["Demonic Citizens"], 0)
		setBuilding(Building["Austrin Settlement"], 0)
		setBuilding(Building["Tyrant"], 0)
		setBuilding(Building["Frozen Souls"], 0)		#Added in Frozen: TC01
		
		if iCiv == Civ["Infernal"]:
			setBuilding(Building["Demonic Citizens"], 1)
			pCity.setHasReligion(Rel["Order"], False, True, True)
			if game.countKnownTechNumTeams(self.Techs["Infernal Pact"]) > 0:
				pCity.setHasReligion(Rel["Ashen Veil"], True, True, True)
				setBuilding(Building["Elder Council"], 1)
				setBuilding(Building["Barracks"], 1)
				setBuilding(Building["Obsidian Gate"], 1)
				setBuilding(Building["Forge"], 1)
				setBuilding(Building["Mage Guild"], 1)
		
		if hasTrait(Trait["Tyrant"]):
			setBuilding(Building["Tyrant"], 1)

		if iCiv == Civ["Austrin"]:
			setBuilding(Building["Austrin Settlement"], 1)

#	Added in Frozen: TC01
#Changed in Frozen: TC01
		iPrevCiv = pPrevious.getCivilizationType()
		if (iPrevCiv == Civ["Infernal"] or iPrevCiv == Civ["Frozen"]):
			setBuilding(Building["Obsidian Gate"], 0)
			setBuilding(Building["Frozen Souls"], 0)
#	Added in Frozen: TC01
#	Adds buildings to Frozen cities when a city is captured and kept. Also does Temple of the Hand ability. Also removes all religions from the city.
		if iCiv == Civ["Frozen"]:
			setBuilding(Building["Demonic Citizens"], 0)
			setBuilding(Building["Elder Council"], 1)
			setBuilding(Building["Obsidian Gate"], 1)
			setBuilding(Building["Forge"], 1)
			setBuilding(Building["Mage Guild"], 1)
			setBuilding(Building["Frozen Souls"], 1)
			
			cf.doTempleWhiteHand(pCity)
			
			setReligion 	 = pCity.setHasReligion
			isHolyCityByType = pCity.isHolyCityByType
			getBuildingInfo  = gc.getBuildingInfo
			iNumBuildings 	 = gc.getNumBuildingInfos()
			for iTarget in xrange(gc.getNumReligionInfos()):
				if iTarget != Rel["White Hand"]:
					if not isHolyCityByType(iTarget):
						setReligion(iTarget, False, True, True)
						for i in xrange(iNumBuildings):
							if getBuildingInfo(i).getPrereqReligion() == iTarget:
								setBuilding(i, 0)
#End of Frozen

		if game.getWBMapScript():
			sf.onCityAcquired(iPreviousOwner, iNewOwner, pCity, bConquest, bTrade)

		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onCityAcquired']:
			module.onCityAcquired(self, argsList)

		## Modular Python End
		## *******************

			
		CvUtil.pyPrint('City Acquired Event: %s' %(pCity.getName()))

	def onCityAcquiredAndKept(self, argsList):
		'City Acquired and Kept'
		iOwner,pCity= argsList
		gc 			= CyGlobalContext()
		game 		= CyGame()
		pPlayer 	= gc.getPlayer(iOwner)
		Status 		= self.LeaderStatus

		#Functions added here tend to cause OOS issues

		if pPlayer.getLeaderStatus() == Status["Minor"]:
			if pPlayer.getNumCities() >= 4:
				pPlayer.setLeaderStatus(Status["Major"])

#FF: Added by Jean Elcard 03/01/2009 (State Names)
		pPlayer.updateStateNameType()
#FF: End Add

		CvUtil.pyPrint('City Acquired and Kept Event: %s' %(pCity.getName()))

	def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())
		if (not self.__LOG_CITYLOST):
			return
		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s'
			%(city.getName(), player.getID(), player.getCivilizationName()))

	def onCultureExpansion(self, argsList):
		'City Culture Expansion'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("City %s's culture has expanded" %(pCity.getName(),))

	def onCityGrowth(self, argsList):
		'City Population Growth'
		pCity = argsList[0]
		iPlayer = argsList[1]
		CvUtil.pyPrint("%s has grown" %(pCity.getName(),))

	def onCityDoTurn(self, argsList):
		'City Production'
		pCity = argsList[0]
		iPlayer = argsList[1]

		gc = CyGlobalContext() 
		cf				= self.cf
		getInfoType		= gc.getInfoTypeForString
		game 			= CyGame()
		map 			= CyMap()
		getPlot 		= map.plot
		isHasReligion 	= pCity.isHasReligion
		setHasReligion 	= pCity.setHasReligion
		pPlot 			= pCity.plot()
		iPlayer 		= pCity.getOwner()
		getPlayer 		= gc.getPlayer
		pPlayer 		= getPlayer(iPlayer)
		randNum 		= game.getSorenRandNum
		hasTrait 		= pPlayer.hasTrait
		pyPlayer 		= PyPlayer(iPlayer)
		Speed			= self.GameSpeeds
		iGameSpeed 		= game.getGameSpeedType()
		numB 			= pCity.getNumBuilding
		countNumB 		= pPlayer.countNumBuildings
		globalCounter 	= game.getGlobalCounter()
		setB			= pCity.setNumRealBuilding
		getBuilding 	= gc.getBuildingInfo
		getBClassCount  = pPlayer.getBuildingClassCount
		iCiv 			= pPlayer.getCivilizationType()
		getUnitCCount 	= pPlayer.getUnitClassCount
		message			= CyInterface().addMessage
		initUnit 		= pPlayer.initUnit
		Building		= self.Buildings
		Civ				= self.Civilizations
		Rel				= self.Religions
		Trait			= self.Traits
		Civic			= self.Civics
		Unit			= self.Units
		Promo			= self.Promotions["Effects"]
		Race			= self.Promotions["Race"]
		iNoAI 			= UnitAITypes.NO_UNITAI
		iSouth			= DirectionTypes.DIRECTION_SOUTH
		iNorth			= DirectionTypes.DIRECTION_NORTH
		iX = pCity.getX(); iY = pCity.getY()
		
#scions start - Gives chance to remove some of Pelemoc's effects.
		if numB( Building["Riot and Sedition"]) > 0:
			if randNum(100, "Spell expiration") < 16: setB( Building["Riot and Sedition"], 0)

		if numB( Building["Poison Words"]) > 0:
			if randNum(100, "Spell expiration") < 5: setB( Building["Poison Words"], 0)

		if numB( Building["Hall of Mirrors"]) > 0:
			if randNum(100, "Hall of Mirrors") <= 100:
				pUnit = -1
				iX = iX
				iY = iY
				eTeam = gc.getTeam(pPlayer.getTeam())
				for iiX,iiY in RANGE1:
					pPlot2 = getPlot(iX+iiX,iY+iiY)
					if pPlot2.isVisibleEnemyUnit(iPlayer):
						getUnit = pPlot2.getUnit
						for i in xrange(pPlot2.getNumUnits()):
							pUnit2 = getUnit(i)
							if eTeam.isAtWar(pUnit2.getTeam()):
								pUnit = pUnit2
				if pUnit != -1:
					newUnit = initUnit(pUnit.getUnitType(), iX, iY, iNoAI, iNorth)
					newUnit.setHasPromotion( Race["Illusion"], True)
					if pPlayer.hasTrait( Trait["Summoner"]):
						newUnit.setDuration(5)
					else:
						newUnit.setDuration(3)

		if numB( Building["Eyes and Ears"]) > 0:
			eTeam = gc.getTeam(pPlayer.getTeam())
			listTeams = []
			for iPlayer2 in xrange(gc.getMAX_PLAYERS()):
				pPlayer2 = gc.getPlayer(iPlayer2)
				if (pPlayer2.isAlive() and iPlayer2 != iPlayer):
					iTeam2 = pPlayer2.getTeam()
					if eTeam.isOpenBorders(iTeam2):
						listTeams.append(gc.getTeam(iTeam2))
			if len(listTeams) >= 3:
				setTech 	= eTeam.setHasTech
				canResearch = pPlayer.canEverResearch
				for iTech in xrange(gc.getNumTechInfos()):
					if (canResearch(iTech)):
						if hasTech(iTech) == False:
							iCount = 0
							for i in xrange(len(listTeams)):
								if listTeams[i].isHasTech(iTech):
									iCount = iCount + 1
							if iCount >= 3:
								setTech(iTech, True, iPlayer, False, True)
								message(iPlayer,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_EYES_AND_EARS_NETWORK_FREE_TECH",()),'AS2D_TECH_DING',1,'Art/Interface/Buttons/Buildings/Eyesandearsnetwork.dds',ColorTypes(8),iX,iY,True,True)

		if numB( Building["Planar Gate"]) > 0:
			cf.doCityTurnPlanarGate(iPlayer, pCity)
		
		if numB( Building["Memorial Refugee"]) > 0:
			cf.doCityTurnMemorial(iPlayer, pCity)


# FF: KAHDI GATES
### If you have a gate, you can summon gate creatures
### Maximum number of creatures empire-wide is based on number of Gates, Repositories, Meditation Halls and Mage Guilds.

		if numB( Building["Kahdi Gate"]) > 0:
			iMax = 1
			iMult = 1

			# Only a redeemed Kahd has Spiritual
			if hasTrait( Trait["Spiritual"]):
				iMult = 3
				iMax = 1.5  # Yes, I know that 1.5 isn't an integer but never mind...

			if numB( Building["Kahdi Library"]) > 0: iMult += 0.5

			if randNum(10000, "Planar Gate") <= (Define["Planar Gate"] * iMult):
				listUnits = []
				iGates 			= countNumB( Building["Kahdi Gate"])
				iLibraries 		= countNumB( Building["Kahdi Library"])
				iMedHalls 		= countNumB( Building["Kahdi Meditation Hall"])
				iGreatLibrary 	= countNumB( Building["The Great Library"]) * 4
				iMageGuild 		= countNumB( Building["Mage Guild"])
				iManaCount	 	= cf.countMana(pPlayer)

				iMax = iMax *  (iGates + iLibraries + iMedHalls + iGreatLibrary + iMageGuild)

				### Gnoslings always available, others requre a building
				UnitClass = self.UnitClasses
				Kahdi 	= self.Units["Kahdi"]
				Summon 	= self.Units["Summons"]
				Mana 	= self.Mana
				if getUnitCCount( UnitClass["Gnossling"]) < (iMax/2):
					listUnits.append( Kahdi["Gnosling"])
				if numB( Building["Mage Guild"]) > 0:
					if getUnitCCount( UnitClass["Thade"]) < (iMax/3):
						listUnits.append( Kahdi["Thade"])
				if numB( Building["Kahdi Meditation Hall"]) > 0:
					iNum = getNumAvailBonuses( Mana["Metamagic"] )
					if iNum > 0:
						if getUnitCCount( UnitClass["Djinn"]) < (iMax/4)*iNum :
							listUnits.append( Summon["Djinn"])

					iNum = getNumAvailBonuses( Mana["Fire"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Fire Elemental"]) < (iMax/6)*iNum :
							listUnits.append( Summon["Fire Elemental"])

					iNum = getNumAvailBonuses( Mana["Air"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Air Elemental"]) < (iMax/6)*iNum :
							listUnits.append( Summon["Air Elemental"])

					iNum = getNumAvailBonuses( Mana["Death"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Spectre"]) < (iMax/4)*iNum :
							listUnits.append( Summon["Spectre"])

					iNum = getNumAvailBonuses( Mana["Body"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Flesh Golem"]) < (iMax/6)*iNum :
							listUnits.append( Summon["Flesh Golem"])

					iNum = getNumAvailBonuses( Mana["Entropy"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Pit Beast"]) < (iMax/4)*iNum :
							listUnits.append( Summon["Pit Beast"])

					iNum = getNumAvailBonuses( Mana["Ice"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Ice Elemental"]) < (iMax/4)*iNum :
							listUnits.append( Summon["Ice Elemental"])

					iNum = getNumAvailBonuses( Mana["Law"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Einherjar"]) < (iMax/4)*iNum :
							listUnits.append( Summon["Einherjar"])

					iNum = getNumAvailBonuses( Mana["Shadow"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Mistform"]) < (iMax/8)*iNum :
							listUnits.append( Summon["Mistform"])

					iNum = getNumAvailBonuses( Mana["Sun"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Aurealis"]) < (iMax/6)*iNum :
							listUnits.append( Summon["Aurealis"])

					iNum = getNumAvailBonuses( Mana["Water"])
					if iNum > 0:
						if getUnitCCount( UnitClass["Water Elemental"]) < (iMax/6)*iNum :
							listUnits.append( Summon["Water Elemental"])

				#Psions
				if pPlayer.isFeatAccomplished(FeatTypes.FEAT_GLOBAL_SPELL):
					if hasTrait( Trait["Ambitious"]):
						if getUnitCCount( UnitClass["Psion"] ) < 1 + (iMax/10):
							listUnits.append( Kahdi["Psion"])

				if len(listUnits) > 0:
					iUnit = listUnits[randNum(len(listUnits), "Kahdi Gate")]
					newUnit = initUnit(iUnit, iX, iY, iNoAI, iNorth)
					message(iPlayer,True,25,CyTranslator().getText("TXT_KEY_MESSAGE_KAHDI_GATE",()),'AS2D_DISCOVERBONUS',1,gc.getUnitInfo(newUnit.getUnitType()).getButton(),ColorTypes(8),iX,iY,True,True)
					if iUnit == Kahdi["Thade"]:
						promotions = ['PROMOTION_AIR1','PROMOTION_BODY1','PROMOTION_EARTH1','PROMOTION_ENCHANTMENT1','PROMOTION_LAW1','PROMOTION_LIFE1','PROMOTION_MIND1','PROMOTION_NATURE1','PROMOTION_SPIRIT1','PROMOTION_WATER1', 'PROMOTION_AIR2', 'PROMOTION_FIRE2', 'PROMOTION_EARTH2', 'PROMOTION_NATURE2', 'PROMOTION_LAW2']
						newUnit.setLevel(4)
						newUnit.setExperienceTimes100(1400, -1)
						setPromo = newUnit.setHasPromotion
						for i in promotions:
							if randNum(5, "Thade Free Promotions") == 1:
								setPromo(getInfoType(i), True)
 

					# Redeemed Kahd summons are strong
					if hasTrait( Trait["Spiritual"]):
						newUnit.setHasPromotion( Promo["Strong"], True)

# FF: End
		
# 	Jotnar
		if iCiv == Civ["Jotnar"]:
			cf.doCityTurnJotnar(iPlayer, pCity)		
# 	Doviello		
		elif iCiv == Civ["Doviello"]:
			cf.doCityTurnDoviello(iPlayer, pCity)	
# 	Infernals
		elif iCiv == Civ["Infernal"]:
			if isHasReligion(Rel["Order"]):
				setHasReligion(Rel["Order"], False, True, True)
			if game.countKnownTechNumTeams(getInfoType('TECH_INFERNAL_PACT')) > 0:
				if isHasReligion(Rel["Ashen Veil"]) == False:
					setHasReligion(Rel["Ashen Veil"], True, True, True)

#	Added in Frozen: TC01
		elif iCiv == Civ["Frozen"]:
			iNumBuildings = gc.getNumBuildingInfos()
			isHolyCityByType = pCity.isHolyCityByType
			for iTarget in xrange(gc.getNumReligionInfos()):
				if iTarget != Rel["White Hand"]:
					if not isHolyCityByType(iTarget):
						setHasReligion(iTarget, False, True, True)
						for i in xrange(iNumBuildings):
							if getBuilding(i).getPrereqReligion() == iTarget:
								setB(i, 0)

				if numB( Building["Swamp of Souls"]) > 0:
					scriptData = (pickle.loads(pCity.getScriptData()))
					iNumSpawns = scriptData["iNumTarSpawns"]
					if not iNumSpawns > 3: 		#City can only spawn 3 more tar demons... this number can be changed for balance
						if iNumSpawns == 0:
							iRandMax = 100
							iMax = 10
							iRand = CyGame().getSorenRandNum(iRandMax, "Tar Demon")
							if iRand <= iMax:
								pTar = pPlayer.initUnit( Unit["Sheaim"]["Tar Demon"], iX, iY, iNoAI, iSouth)
								scriptData["iNumTarSpawns"] = (iNumSpawns + 1)
								pCity.setScriptData(pickle.dumps(scriptData))
						else:
							iRandMax = (iNumSpawns * 100) ^ 2
							iMax = iNumSpawns * 100		#After the first tar demon, 1 in 100, 1 in 200, 1 in 300 chances.
							iRand = CyGame().getSorenRandNum(iRandMax, "Tar Demon")
							if iRand <= iMax:
								pTar = pPlayer.initUnit( Unit["Sheaim"]["Tar Demon"], iX, iY, iNoAI, iSouth)
								scriptData["iNumTarSpawns"] = (iNumSpawns + 1)
								pCity.setScriptData(pickle.dumps(scriptData))

#End of Frozen

# 	Mercurians
		elif iCiv == Civ["Mercurians"]:
			if isHasReligion(Rel["Ashen Veil"]):
				setHasReligion(Rel["Ashen Veil"], False, True, True)

		if numB(Building["Shrine of Sirona"]) > 0:
			pPlayer.setFeatAccomplished(FeatTypes.FEAT_HEAL_UNIT_PER_TURN, True)
			
		## *******************
		## Modular Python: ANW 29-may-2010

		for module in command['onCityDoTurn']:
			module.onCityDoTurn(self, argsList)

		## Modular Python End
		## *******************

		CvAdvisorUtils.cityAdvise(pCity, iPlayer)

	def onCityBuildingUnit(self, argsList):
		'City begins building a unit'
		pCity = argsList[0]
		iUnitType = argsList[1]
		gc = CyGlobalContext() 

		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getUnitInfo(iUnitType).getDescription()))

	def onCityBuildingBuilding(self, argsList):
		'City begins building a Building'
		pCity = argsList[0]
		iBuildingType = argsList[1]
		gc = CyGlobalContext() 
		if (not self.__LOG_CITYBUILDING):
			return
		CvUtil.pyPrint("%s has begun building a %s" %(pCity.getName(),gc.getBuildingInfo(iBuildingType).getDescription()))

	def onCityRename(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		game = CyGame()
		if (pCity.getOwner() == game.getActivePlayer()):
			self.__eventEditCityNameBegin(pCity, True)

	def onCityHurry(self, argsList):
		'City is renamed'
		pCity = argsList[0]
		iHurryType = argsList[1]

	def onVictory(self, argsList):
		'Victory'
		iTeam, iVic = argsList
		gc = CyGlobalContext() 
		game 		= CyGame()
		getPlayer 	= gc.getPlayer
		
		if (iVic >= 0 and iVic < gc.getNumVictoryInfos()):
			trophy 		= game.changeTrophyValue
			Option		= self.GameOptions
			Civ 		= self.Civilizations
			Victory		= self.Victories
			for iPlayer in xrange(gc.getMAX_PLAYERS()):
				pPlayer = getPlayer(iPlayer)
				if pPlayer.isAlive() and pPlayer.isHuman() and pPlayer.getTeam() == iTeam:
					if game.getWBMapScript():
						sf.onVictory(iPlayer, iVic)
					else:
						iCiv = pPlayer.getCivilizationType()
						if   iCiv == Civ["Amurites"]:		trophy("TROPHY_VICTORY_AMURITES", 1)
						elif iCiv == Civ["Balseraphs"]:		trophy("TROPHY_VICTORY_BALSERAPHS", 1)
						elif iCiv == Civ["Bannor"]:			trophy("TROPHY_VICTORY_BANNOR", 1)
						elif iCiv == Civ["Calabim"]:		trophy("TROPHY_VICTORY_CALABIM", 1)
						elif iCiv == Civ["Clan of Embers"]:	trophy("TROPHY_VICTORY_CLAN_OF_EMBERS", 1)
						elif iCiv == Civ["Doviello"]:		trophy("TROPHY_VICTORY_DOVIELLO", 1)
						elif iCiv == Civ["Elohim"]:			trophy("TROPHY_VICTORY_ELOHIM", 1)
						elif iCiv == Civ["Grigori"]:		trophy("TROPHY_VICTORY_GRIGORI", 1)
						elif iCiv == Civ["Hippus"]:			trophy("TROPHY_VICTORY_HIPPUS", 1)
						elif iCiv == Civ["Illians"]:		trophy("TROPHY_VICTORY_ILLIANS", 1)
						elif iCiv == Civ["Infernal"]:		trophy("TROPHY_VICTORY_INFERNAL", 1)
						elif iCiv == Civ["Khazad"]:			trophy("TROPHY_VICTORY_KHAZAD", 1)
						elif iCiv == Civ["Kuriotates"]:		trophy("TROPHY_VICTORY_KURIOTATES", 1)
						elif iCiv == Civ["Lanun"]:			trophy("TROPHY_VICTORY_LANUN", 1)
						elif iCiv == Civ["Ljosalfar"]:		trophy("TROPHY_VICTORY_LJOSALFAR", 1)
						elif iCiv == Civ["Luchuirp"]: 		trophy("TROPHY_VICTORY_LUCHUIRP", 1)
						elif iCiv == Civ["Malakim"]:		trophy("TROPHY_VICTORY_MALAKIM", 1)
						elif iCiv == Civ["Mercurians"]:		trophy("TROPHY_VICTORY_MERCURIANS", 1)
						elif iCiv == Civ["Sheaim"]:			trophy("TROPHY_VICTORY_SHEAIM", 1)
						elif iCiv == Civ["Sidar"]:			trophy("TROPHY_VICTORY_SIDAR", 1)
						elif iCiv == Civ["Svartalfar"]:		trophy("TROPHY_VICTORY_SVARTALFAR", 1)
						elif iCiv == Civ["Mazatl"]:   		trophy("TROPHY_VICTORY_MAZATL", 1)
						elif iCiv == Civ["Mechanos"]: 		trophy("TROPHY_VICTORY_MECHANOS", 1)
						elif iCiv == Civ["Scions"]:	  		trophy("TROPHY_VICTORY_SCIONS", 1)
						elif iCiv == Civ["Archos"]:	  		trophy("TROPHY_VICTORY_ARCHOS", 1)
						elif iCiv == Civ["Austrin"]:  		trophy("TROPHY_VICTORY_AUSTERIN", 1)
						elif iCiv == Civ["Chislev"]:  		trophy("TROPHY_VICTORY_CHISLEV", 1)
						elif iCiv == Civ["Cualli"]:	  		trophy("TROPHY_VICTORY_CUALLI", 1)
						elif iCiv == Civ["Dural"]:	  		trophy("TROPHY_VICTORY_DURAL", 1)
						elif iCiv == Civ["Jotnar"]:   		trophy("TROPHY_VICTORY_JOTNAR", 1)
						elif iCiv == Civ["Khadi"]:    		trophy("TROPHY_VICTORY_KHADI", 1)
						elif iCiv == Civ["D'Tesh"]:   		trophy("TROPHY_VICTORY_DTESH", 1)
						elif iCiv == Civ["Frozen"]:   		trophy("TROPHY_VICTORY_FROZEN", 1)

						if   iVic == Victory["Altar"]: 		trophy("TROPHY_VICTORY_ALTAR_OF_THE_LUONNOTAR", 1)
						elif iVic == Victory["Conquest"]:	trophy("TROPHY_VICTORY_CONQUEST", 1)
						elif iVic == Victory["Cultural"]:	trophy("TROPHY_VICTORY_CULTURAL", 1)
						elif iVic == Victory["Domination"]:	trophy("TROPHY_VICTORY_DOMINATION", 1)
						elif iVic == Victory["Religious"]:	trophy("TROPHY_VICTORY_RELIGIOUS", 1)
						elif iVic == Victory["Score"]:		trophy("TROPHY_VICTORY_SCORE", 1)
						elif iVic == Victory["Time"]:		trophy("TROPHY_VICTORY_TIME", 1)
						elif iVic == Victory["Tower"]:		trophy("TROPHY_VICTORY_TOWER_OF_MASTERY", 1)

						if Option["Barbarian World"]: 		trophy("TROPHY_VICTORY_BARBARIAN_WORLD", 1)
						if Option["Cut Losers"]:			trophy("TROPHY_VICTORY_FINAL_FIVE", 1)
						if Option["High to Low"]:			trophy("TROPHY_VICTORY_HIGH_TO_LOW", 1)
						if Option["Increasing Difficulty"]:	trophy("TROPHY_VICTORY_INCREASING_DIFFICULTY", 1)

			victoryInfo = gc.getVictoryInfo(int(iVic))
			CvUtil.pyPrint("Victory!  Team %d achieves a %s victory"
				%(iTeam, victoryInfo.getDescription()))

	def onVassalState(self, argsList):
		'Vassal State'
		iMaster, iVassal, bVassal = argsList
		gc = CyGlobalContext() 
		
		if (bVassal):
			CvUtil.pyPrint("Team %d becomes a Vassal State of Team %d"
				%(iVassal, iMaster))
		else:
			CvUtil.pyPrint("Team %d revolts and is no longer a Vassal State of Team %d"
				%(iVassal, iMaster))
		getPlayer = gc.getPlayer
#FF: Added by Jean Elcard 03/01/2009 (State Names)
		for iPlayer in xrange(gc.getMAX_PLAYERS()):
			pPlayer = getPlayer(iPlayer)
			if pPlayer.isAlive():
				if (pPlayer.getTeam() == iMaster) or (pPlayer.getTeam() == iVassal):
					print "Update State Name for %d." % iPlayer
					print "Vassal: %s" % gc.getTeam(pPlayer.getTeam()).isAVassal()
					pPlayer.updateStateNameType()
#FF: End Add

	def onGameUpdate(self, argsList):
		'sample generic event, called on each game turn slice'
		genericArgs = argsList[0][0]	# tuple of tuple of my args
		turnSlice = genericArgs[0]
		
		# Added by Gerikes for OOS logging.
		if CyInterface().isOOSVisible():
			CyMessageControl().sendModNetMessage(CvUtil.AutoPlay, 0, -1, -1, -1)
			OOSLogger.doGameUpdate()
		# End added by Gerikes for OOS logging.

	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
		if ( px!=-1 and py!=-1 ):
			if ( eventType == self.EventLButtonDown ):
				if (self.bAllowCheats and self.bCtrl and self.bAlt and map.plot(px,py).isCity() and not interfaceConsumed):
					# Launch Edit City Event
					self.beginEvent( CvUtil.EventEditCity, (px,py) )
					return 1

				elif (self.bAllowCheats and self.bCtrl and self.bShift and not interfaceConsumed):
					# Launch Place Object Event
					self.beginEvent( CvUtil.EventPlaceObject, (px, py) )
					return 1

		if ( eventType == self.EventBack ):
			return CvScreensInterface.handleBack(screens)
		elif ( eventType == self.EventForward ):
			return CvScreensInterface.handleForward(screens)

		return 0


#################### TRIGGERED EVENTS ##################

	def __eventEditCityNameBegin(self, city, bRename):
		popup = PyPopup.PyPopup(CvUtil.EventEditCityName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((city.getID(), bRename))
		popup.setHeaderString(localText.getText("TXT_KEY_NAME_CITY", ()))
		popup.setBodyString(localText.getText("TXT_KEY_SETTLE_NEW_CITY_NAME", ()))
		popup.createEditBox(city.getName())
		popup.setEditBoxMaxCharCount( 15 )
		popup.launch()

	def __eventEditCityNameApply(self, playerID, userData, popupReturn):
		'Edit City Name Event'
		iCityID = userData[0]
		bRename = userData[1]
		player = CyGlobalContext().getPlayer(playerID)
		city = player.getCity(iCityID)
		cityName = popupReturn.getEditBoxString(0)
		if (len(cityName) > 30):
			cityName = cityName[:30]
		city.setName(cityName, not bRename)

	def __eventEditCityBegin(self, argsList):
		'Edit City Event'
		px,py = argsList
		CvWBPopups.CvWBPopups().initEditCity(argsList)

	def __eventEditCityApply(self, playerID, userData, popupReturn):
		'Edit City Event Apply'
		if (getChtLvl() > 0):
			CvWBPopups.CvWBPopups().applyEditCity( (popupReturn, userData) )

	def __eventPlaceObjectBegin(self, argsList):
		'Place Object Event'
		CvDebugTools.CvDebugTools().initUnitPicker(argsList)

	def __eventPlaceObjectApply(self, playerID, userData, popupReturn):
		'Place Object Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyUnitPicker( (popupReturn, userData) )

	def __eventAwardTechsAndGoldBegin(self, argsList):
		'Award Techs & Gold Event'
		CvDebugTools.CvDebugTools().cheatTechs()

	def __eventAwardTechsAndGoldApply(self, playerID, netUserData, popupReturn):
		'Award Techs & Gold Event Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyTechCheat( (popupReturn) )

	def __eventShowWonderBegin(self, argsList):
		'Show Wonder Event'
		CvDebugTools.CvDebugTools().wonderMovie()

	def __eventShowWonderApply(self, playerID, netUserData, popupReturn):
		'Wonder Movie Apply'
		if (getChtLvl() > 0):
			CvDebugTools.CvDebugTools().applyWonderMovie( (popupReturn) )

	def __eventEditUnitNameBegin(self, argsList):
		pUnit = argsList
		popup = PyPopup.PyPopup(CvUtil.EventEditUnitName, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setUserData((pUnit.getID(),))
		popup.setBodyString(localText.getText("TXT_KEY_RENAME_UNIT", ()))
		popup.createEditBox(pUnit.getNameNoDesc())
		popup.launch()

	def __eventEditUnitNameApply(self, playerID, userData, popupReturn):
		'Edit Unit Name Event'
		iUnitID = userData[0]
		unit = CyGlobalContext().getPlayer(playerID).getUnit(iUnitID)
		newName = popupReturn.getEditBoxString(0)
		if (len(newName) > 25):
			newName = newName[:25]
		unit.setName(newName)

	def __eventWBAllPlotsPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().allPlotsCB()
		return
	
	def __eventWBAllPlotsPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getButtonClicked() >= 0):
			CvScreensInterface.getWorldBuilderScreen().handleAllPlotsCB(popupReturn)
		return

	def __eventWBLandmarkPopupBegin(self, argsList):
		CvScreensInterface.getWorldBuilderScreen().setLandmarkCB("")
		#popup = PyPopup.PyPopup(CvUtil.EventWBLandmarkPopup, EventContextTypes.EVENTCONTEXT_ALL)
		#popup.createEditBox(localText.getText("TXT_KEY_WB_LANDMARK_START", ()))
		#popup.launch()
		return

	def __eventWBLandmarkPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szLandmark = popupReturn.getEditBoxString(0)
			if (len(szLandmark)):
				CvScreensInterface.getWorldBuilderScreen().setLandmarkCB(szLandmark)
		return

	def __eventWBScriptPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBScriptPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString(localText.getText("TXT_KEY_WB_SCRIPT", ()))
		popup.createEditBox(CvScreensInterface.getWorldBuilderScreen().getCurrentScript())
		popup.launch()
		return

	def __eventWBScriptPopupApply(self, playerID, userData, popupReturn):
		if (popupReturn.getEditBoxString(0)):
			szScriptName = popupReturn.getEditBoxString(0)
			CvScreensInterface.getWorldBuilderScreen().setScriptCB(szScriptName)
		return

	def __eventWBStartYearPopupBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventWBStartYearPopup, EventContextTypes.EVENTCONTEXT_ALL)
		popup.createSpinBox(0, "", game.getStartYear(), 1, 5000, -5000)
		popup.launch()
		return

	def __eventWBStartYearPopupApply(self, playerID, userData, popupReturn):
		iStartYear = popupReturn.getSpinnerWidgetValue(int(0))
		CvScreensInterface.getWorldBuilderScreen().setStartYearCB(iStartYear)
		return

	def __eventSetHasTraitBegin(self, argslist):
		return 0
	
	def __eventSetHasTraitApply(self, playerID, userData, popupReturn):
		iPlayer, iTrait, bHas = userData
		pPlayer = CyGlobalContext().getPlayer(iPlayer)
		pPlayer.setHasTrait(iTrait, bHas)
		return 0

	def __eventSetTurnsAutoPlayBegin(self, argslist):
		return 0

	def __eventSetTurnsAutoPlayApply(self, playerID, userData, popupReturn):
		if popupReturn.getButtonClicked() == 0: # ok button
			sEditBoxContent = popupReturn.getEditBoxString(0)
			if sEditBoxContent.isdigit():
				iNumTurns = int(sEditBoxContent)
				# AI Autoplay Sounds by Grey Fox
				profile = CyUserProfile()
				SoundSettings["SOUND_MASTER_VOLUME"] 	= profile.getMasterVolume()
				SoundSettings["SOUND_SPEECH_VOLUME"] 	= profile.getSpeechVolume()
				SoundSettings["SOUND_MASTER_NO_SOUND"] 	= profile.isMasterNoSound()
				profile.setMasterNoSound(True)
				# End
				CyMessageControl().sendModNetMessage(CvUtil.AutoPlay, iNumTurns, -1, -1, -1)
		return 0

## FfH Card Game: begin
	def __EventSelectSolmniumPlayerBegin(self):
		gc = CyGlobalContext() 
		game = CyGame()
		iHUPlayer = game.getActivePlayer()
		getPlayer = gc.getPlayer
		getLeaderHeadInfo = gc.getLeaderHeadInfo
		
		if iHUPlayer == -1 : return 0
		if not cs.canStartGame(iHUPlayer) : return 0

		popup = PyPopup.PyPopup(CvUtil.EventSelectSolmniumPlayer, EventContextTypes.EVENTCONTEXT_ALL)

		sResText = CyUserProfile().getResolutionString(CyUserProfile().getResolution())
		sX, sY = sResText.split("x")
		iXRes = int(sX)
		iYRes = int(sY)

		iW = 620
		iH = 650

		popup.setSize(iW, iH)
		popup.setPosition((iXRes - iW) / 2, 30)

		lStates = []
		startMPGame = cs.getStartGameMPWith
		startAIGame = cs.getStartGameAIWith
		for iPlayer in xrange(gc.getMAX_CIV_PLAYERS()) :
			pPlayer = getPlayer(iPlayer)

			if pPlayer.isNone() : continue

			if pPlayer.isHuman() :
				lPlayerState = startMPGame(iHUPlayer, iPlayer)
				if lPlayerState[0][0] in ["No", "notMet"] : continue
				lStates.append([iPlayer, lPlayerState])
			else :
				lPlayerState = startAIGame(iHUPlayer, iPlayer)
				if lPlayerState[0][0] in ["No", "notMet"] : continue
				lStates.append([iPlayer, lPlayerState])

		lPlayerButtons = []

		popup.addDDS(CyArtFileMgr().getInterfaceArtInfo("SOMNIUM_POPUP_INTRO").getPath(), 0, 0, 512, 128)
		popup.addSeparator()
		#popup.setHeaderString(localText.getText("TXT_KEY_SOMNIUM_START", ()), CvUtil.FONT_CENTER_JUSTIFY)
		if len(lStates) == 0 :
			popup.setBodyString(localText.getText("TXT_KEY_SOMNIUM_NOONE_MET", ()))
		else :
			#popup.setBodyString(localText.getText("TXT_KEY_SOMNIUM_PLAY_WITH", ()))
			popup.addSeparator()
			popup.addSeparator()

			sText = u""
			for iPlayer, lPlayerState in lStates :
				pPlayer 		= getPlayer(iPlayer)
				sPlayerName 	= pPlayer.getName()
				eType 			= pPlayer.getLeaderType()
				getMemory 		= getLeaderHeadInfo(eType).getMemoryAttitudePercent
				iPositiveChange = getMemory(MemoryTypes.MEMORY_SOMNIUM_POSITIVE) / 100
				iNegativeChange = getMemory(MemoryTypes.MEMORY_SOMNIUM_NEGATIVE) / 100
				bShift = True

				for item in lPlayerState :

					sTag = item[0]
					if (sTag == "atWar") :
						if len(sText) > 0 : sText += localText.getText("[NEWLINE]", ())
						sText += localText.getText("TXT_KEY_SOMNIUM_AT_WAR", (sPlayerName, ))

					elif (sTag == "InGame") :
						if len(sText) > 0 : sText += localText.getText("[NEWLINE]", ())
						sText += localText.getText("TXT_KEY_SOMNIUM_IN_GAME", (sPlayerName, ))

					elif (sTag == "relation") :
						delay = item[1]
						if (delay > 0) :
								if len(sText) > 0 : sText += localText.getText("[NEWLINE]", ())
								sText += localText.getText("TXT_KEY_SOMNIUM_GAME_DELAYED", (sPlayerName, delay))
						else :
								if bShift :
										bShift = False
										popup.addSeparator()
								popup.addButton(localText.getText("TXT_KEY_SOMNIUM_GAME_RELATION", (sPlayerName, iPositiveChange, iNegativeChange)))
								lPlayerButtons.append((iPlayer, -1))

					elif (sTag == "gold") :
						for iGold in item[1] :
							if bShift :
								bShift = False
								popup.addSeparator()
							if iGold == 0 :
								popup.addButton(localText.getText("TXT_KEY_SOMNIUM_GAME_FUN", (sPlayerName, )))
								lPlayerButtons.append((iPlayer, iGold))
							else :
								popup.addButton(localText.getText("TXT_KEY_SOMNIUM_GAME_GOLD", (sPlayerName, iGold)))
								lPlayerButtons.append((iPlayer, iGold))

		if len(sText) > 0 :
			popup.addSeparator()
			popup.addSeparator()
			popup.setBodyString(sText)

		popup.setUserData(tuple(lPlayerButtons))
		popup.launch()

	def __EventSelectSolmniumPlayerApply(self, playerID, userData, popupReturn):
		if userData :
			idButtonCliked = popupReturn.getButtonClicked()
			if idButtonCliked in xrange(len(userData)) :
				iOpponent, iGold = userData[idButtonCliked]
				gc = CyGlobalContext()
				pLeftPlayer = gc.getPlayer(playerID)
				pRightPlayer = gc.getPlayer(iOpponent)

				if not pRightPlayer.isHuman() :
					if (cs.canStartGame(playerID)) and (pLeftPlayer.isAlive()) and (pRightPlayer.isAlive()) :
						cs.startGame(playerID, iOpponent, iGold)
					else :
						CyInterface().addMessage(playerID, True, 25, CyTranslator().getText("TXT_KEY_SOMNIUM_CANT_START_GAME", (gc.getPlayer(iOpponent).getName(), )), '', 1, '', ColorTypes(7), -1, -1, False, False)
				else :
					if (cs.canStartGame(playerID)) and (cs.canStartGame(iOpponent)) and (pLeftPlayer.isAlive()) and (pRightPlayer.isAlive()) :
						if (iOpponent == game.getActivePlayer()):
							self.__EventSolmniumAcceptGameBegin((playerID, iOpponent, iGold))
					else :
						CyInterface().addMessage(playerID, True, 25, CyTranslator().getText("TXT_KEY_SOMNIUM_CANT_START_GAME", (gc.getPlayer(iOpponent).getName(), )), '', 1, '', ColorTypes(7), -1, -1, False, False)

	def __EventSolmniumAcceptGameBegin(self, argslist):
		iPlayer, iOpponent, iGold = argslist
		gc = CyGlobalContext()
		if not gc.getPlayer(iOpponent).isAlive() : return 0

		popup = PyPopup.PyPopup(CvUtil.EventSolmniumAcceptGame, EventContextTypes.EVENTCONTEXT_ALL)

		popup.setUserData(argslist)

		popup.setHeaderString(localText.getText("TXT_KEY_SOMNIUM_START", ()))
		if iGold > 0 :
			popup.setBodyString(localText.getText("TXT_KEY_SOMNIUM_ACCEPT_GAME", (gc.getPlayer(iPlayer).getName(), iGold)))
		else :
			popup.setBodyString(localText.getText("TXT_KEY_SOMNIUM_ACCEPT_GAME_FUN", (gc.getPlayer(iPlayer).getName(), )))

		popup.addButton( localText.getText("AI_DIPLO_ACCEPT_1", ()) )
		popup.addButton( localText.getText("AI_DIPLO_NO_PEACE_3", ()) )

		popup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)

	def __EventSolmniumAcceptGameApply(self, playerID, userData, popupReturn):
		gc = CyGlobalContext()
		if userData :
			iPlayer, iOpponent, iGold = userData
			idButtonCliked = popupReturn.getButtonClicked()
			if idButtonCliked == 0 :
				if (cs.canStartGame(iPlayer)) and (cs.canStartGame(iOpponent)) and (gc.getPlayer(iPlayer).isAlive()) and (gc.getPlayer(iOpponent).isAlive()) :
					cs.startGame(iPlayer, iOpponent, iGold)
				else :
					CyInterface().addMessage(iPlayer, True, 25, CyTranslator().getText("TXT_KEY_SOMNIUM_CANT_START_GAME", (gc.getPlayer(iOpponent).getName(), )), '', 1, '', ColorTypes(7), -1, -1, False, False)
					CyInterface().addMessage(iOpponent, True, 25, CyTranslator().getText("TXT_KEY_SOMNIUM_CANT_START_GAME", (gc.getPlayer(iPlayer).getName(), )), '', 1, '', ColorTypes(7), -1, -1, False, False)
			else :
				CyInterface().addMessage(iPlayer, True, 25, CyTranslator().getText("TXT_KEY_SOMNIUM_REFUSE_GAME", (gc.getPlayer(iOpponent).getName(), iGold)), '', 1, '', ColorTypes(7), -1, -1, False, False)

	def __EventSolmniumConcedeGameBegin(self, argslist):
		popup = PyPopup.PyPopup(CvUtil.EventSolmniumConcedeGame, EventContextTypes.EVENTCONTEXT_ALL)

		popup.setUserData(argslist)

		popup.setHeaderString( localText.getText("TXT_KEY_SOMNIUM_START", ()))
		popup.setBodyString( localText.getText("TXT_KEY_SOMNIUM_CONCEDE_GAME", ()))

		popup.addButton( localText.getText("AI_DIPLO_ACCEPT_1", ()) )
		popup.addButton( localText.getText("AI_DIPLO_NO_PEACE_3", ()) )

		popup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)

	def __EventSolmniumConcedeGameApply(self, playerID, userData, popupReturn):
		if userData :
			idButtonCliked = popupReturn.getButtonClicked()
			if idButtonCliked == 0 :
				cs.endGame(userData[0], userData[1])
## FfH Card Game: end
